Package: pmp
Type: Package
Title: Peak Matrix Processing and signal batch correction for
        metabolomics datasets
Version: 1.23.1
Authors@R: c(
    person(
        c("Andris"),
          "Jankevics",
           role=c("aut"),
           email="a.jankevics@bham.ac.uk"),
    person(
        c("Gavin","Rhys"),
          "Lloyd",
           role=c("aut","cre"),
           email="g.r.lloyd@bham.ac.uk"),
    person(
        c("Ralf","Johannes", "Maria"),
          "Weber",
           role=c("aut"),
           email="r.j.weber@bham.ac.uk")
    )
Maintainer@R: c( person( c("Gavin","Rhys"), "Lloyd",
        role=c("aut","cre"), email="g.r.lloyd@bham.ac.uk") )
Description: Methods and tools for (pre-)processing of metabolomics datasets
    (i.e. peak matrices), including filtering, normalisation, missing value
    imputation, scaling, and signal drift and batch effect correction methods.
    Filtering methods are based on: the fraction of missing values (across
    samples or features); Relative Standard Deviation (RSD) calculated from the
    Quality Control (QC) samples; the blank samples. Normalisation methods
    include Probabilistic Quotient Normalisation (PQN) and normalisation to
    total signal intensity. A unified user interface for several commonly used
    missing value imputation algorithms is also provided. Supported methods
    are: k-nearest neighbours (knn), random forests (rf), Bayesian PCA missing
    value estimator (bpca), mean or median value of the given feature and a
    constant small value. The generalised logarithm (glog) transformation
    algorithm is available to stabilise the variance across low and high
    intensity mass spectral features. Finally, this package provides an
    implementation of the Quality Control-Robust Spline Correction (QCRSC)
    algorithm for signal drift and batch effect correction of mass
    spectrometry-based datasets.
License: GPL-3
biocViews: MassSpectrometry, Metabolomics, Software, QualityControl,
        BatchEffect
Depends: R (>= 4.0)
Imports: stats, impute, pcaMethods, missForest, ggplot2, methods,
        SummarizedExperiment, S4Vectors, matrixStats, grDevices,
        reshape2, utils
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.1.1
Suggests: testthat, covr, knitr, rmarkdown, BiocStyle, gridExtra,
        magick
VignetteBuilder: knitr
Collate: 'checkPeakMatrix.R' 'utils.R' 'data.R' 'filters.R'
        'glog_transformation.R' 'mv_imputation.R' 'normalisation.R'
        'sbc_main.R' 'sbc_methods.R' 'sbc_plot.R'
git_url: https://git.bioconductor.org/packages/pmp
git_branch: devel
git_last_commit: 0f6f235
git_last_commit_date: 2025-11-06
Repository: Bioconductor 3.23
Date/Publication: 2025-11-06
NeedsCompilation: no
Packaged: 2025-11-07 00:14:56 UTC; biocbuild
Author: Andris Jankevics [aut],
  Gavin Rhys Lloyd [aut, cre],
  Ralf Johannes Maria Weber [aut]
Maintainer: Gavin Rhys Lloyd <g.r.lloyd@bham.ac.uk>

# Generated by roxygen2: do not edit by hand

export(QCRSC)
export(filter_peaks_by_blank)
export(filter_peaks_by_fraction)
export(filter_peaks_by_rsd)
export(filter_samples_by_mv)
export(glog_plot_optimised_lambda)
export(glog_transformation)
export(mv_imputation)
export(normalise_to_sum)
export(pqn_normalisation)
export(processing_history)
export(remove_peaks)
export(sbc_plot)
import(SummarizedExperiment)
import(ggplot2)
importFrom(S4Vectors,DataFrame)
importFrom(S4Vectors,metadata)
importFrom(grDevices,dev.off)
importFrom(grDevices,pdf)
importFrom(impute,impute.knn)
importFrom(matrixStats,rowAnyMissings)
importFrom(matrixStats,rowCounts)
importFrom(matrixStats,rowMedians)
importFrom(matrixStats,rowSds)
importFrom(matrixStats,rowVars)
importFrom(methods,as)
importFrom(methods,is)
importFrom(missForest,missForest)
importFrom(pcaMethods,pca)
importFrom(reshape2,melt)
importFrom(stats,median)
importFrom(stats,optimise)
importFrom(stats,predict)
importFrom(stats,sd)
importFrom(stats,smooth.spline)
importFrom(stats,var)
importFrom(utils,capture.output)

#' @importFrom  impute impute.knn
#' @importFrom missForest missForest
#' @importFrom pcaMethods pca
#' @importFrom matrixStats rowMedians
NULL

#' Replace missing values for specific peak matrix feature.
#' @param x \code{integer(1)}, index of matrix feature.
#' @param df a \code{numeric()}, peak matrix with features in the rows,
#' samples in the columns.
#' @param vals \code{numeric()}, vector object with the same length as number
#' of features in peak matrix to replca NA values with.
#' 
#' @return feature at specified index 'x' with replaced missing values
#' 
#' @noRd
replace_na <- function(x, df, vals){
    col_vals <- df[x, ]
    na_val <- vals[x]
    col_vals[is.na(col_vals)] <- na_val
    col_vals
}

#' Imput missing values using mode of each feature
#' @param df \code{numeric()}, a peak matrix with features in the rows, samples
#' in the columns
#' @param method \code{character(1)}, mode to use for missing value imputation.
#' 'mn' for mean and md'for median.
#' 
#' @return data frame of missing value imputed peak intensity matrix
#' 
#' @noRd
impute_mode <- function (df, method){
    if (method=="mn"){
        replacement_vals <- rowMeans(df, na.rm=TRUE)
    }
    if (method=="md"){
        replacement_vals <- rowMedians(df, na.rm=TRUE)
    }
    feature_names <- rownames(df)
    df <- do.call(rbind, lapply(seq_len(nrow(df)), replace_na, df=df, 
        vals=replacement_vals))
    rownames(df) <- feature_names
    df
}

#' Missing value imputation using different algorithms
#' 
#' Missing values in metabolomics data sets occur widely and can originate from 
#' a number of sources, including technical and biological reasons. \cr
#' Missing values imputation is applied to replace non-existing values 
#' with an estimated values while maintaining the data structure. A number of 
#' different methods are available as part of this function. \cr
#' 
#' Supported missing value imputation methods are: \cr
#' \cr
#' \code{knn} - K-nearest neighbour. For each feature in each sample, missing 
#' values are replaced by the mean average value (non-weighted) calculated 
#' from its \code{k} closest neighbours in multivariate space (default distance 
#' metric: euclidean distance); \cr
#' \cr
#' \code{rf} - Random Forest. This method is a wrapper of 
#' \link[missForest]{missForest} function. For each feature, missing values are 
#' iteratively imputed until a maximum number of iterations (10), or until the
#' difference between consecutively-imputed matrices becomes positive. 
#' Trees per forest are set to 100, variables included per tree are calculate 
#' using formula \eqn{sqrt(total number of variables)}; \cr
#' \cr
#' \code{bpca} - Bayesian principal component analysis. This method is a 
#' wrapper of \link[pcaMethods]{pca} function. Missing values are replaced by
#' the values obtained from principal component analysis regression with a 
#' Bayesian method. Therefore every imputed missing value does not occur 
#' multiple times, neither across the samples nor across the metabolite 
#' features; \cr
#' \cr
#' \code{sv} - Small value. For each feature, replace missing values with half
#' of the lowest value recorded in the entire data matrix; \cr
#' \cr
#' \code{'mn'} - Mean. For each feature, replace missing values with the mean 
#' average (non-weighted) of all other non-missing values for that variable;\cr
#' \cr
#' \code{'md'} - Median. For each feature, replace missing values with the 
#' median of all other non-missing values for that variable. \cr
#' \cr
#' 
#' @inheritParams filter_peaks_by_blank
#' @param method \code{character(1)}, missing value imputation method.
#' Supported methods are \code{knn}, \code{rf}, \code{bpca}, \code{sv},
#' \code{'mn'} and \code{'md'}.
#' @param k \code{numeric(1)}, for a given sample containing a missing value,
#' the number of nearest neighbours to include to calculate a replacement 
#' value. Used only for method \code{knn}. 
#' @param rowmax \code{numeric(1)}, the maximum percentage of missing data 
#' allowed in any row. For any rows exceeding given limit, missing values are 
#' imputed using the overall mean per sample. Used only for method \code{knn}.
#' @param colmax \code{numeric(1)}, the maximum percent missing data allowed in
#' any column. If any column exceeds given limit, the function will report an
#' error Used only for method \code{knn}.
#' @param maxp \code{integer(1)}, number of features to run on single core.
#' If set to NULL will use total number of features.
#' @param check_df \code{logical(1)}, if set to TRUE will check if input data 
#' needs to be transposed, so that features are in rows.
#' @return Object of class \code{SummarizedExperiment}. If input data are a 
#' matrix-like (e.g. an ordinary matrix, a data frame) object, function returns 
#' the same R data structure as input with all value of data type 
#' \code{numeric()}.
#' 
#' @examples 
#' df <- MTBLS79[ ,MTBLS79$Batch == 1]
#' out <- mv_imputation(df=df, method='knn')
#' 
#' @export

mv_imputation <- function(df, method, k=10, rowmax=0.5, colmax=0.5, 
    maxp=NULL, check_df=TRUE) {
    if (check_df == TRUE) {
        df <- check_input_data(df=df)
    } else {
        # Avoid transposing peak matrix using fake class label
        df <- check_input_data(df=df, classes=rep(1, ncol(df)))
    }
    if (is.null(maxp)) {
        maxp <- max(dim(df))
    }
    if (any(rowSums(is.na(assay(df))) == ncol(df))) {
        stop("Error occurred. Rows with 100% missing values detected - please
    remove these rows using the peak filter tool")
    }
    if (any(colSums(is.na(assay(df))) == nrow(df))) {
        stop("Error occurred. Columns with 100% missing values detected - please
    remove these columns using the sample filter tool")
    }
    if (tolower(method) == "knn") {
        obj <- (impute.knn(assay(df), k=k, 
            rowmax=rowmax, colmax=colmax, maxp=maxp))
        assay(df) <- obj$data
    } else if (tolower(method) == "rf") {
        mf_out <- missForest(t(assay(df)))
        print(mf_out$OOBerror)
        assay(df) <- t(mf_out$ximp)
    } else if (tolower(method) == "bpca") {
        pcaOb <- pcaMethods::pca(t(assay(df)), method="bpca", scale="none")
        bpca_out <- t(pcaOb@completeObs)
        ##GUARD AGAINST NEGATIVE VALUES
        bpca_out[bpca_out < 0] <- min(bpca_out[bpca_out > 0])  
        assay(df) <- bpca_out
    } else if (tolower(method) == "sv") {
        assay(df)[is.na(assay(df))] <- min(assay(df), na.rm=TRUE)/2  ##SMV
    } else if (tolower(method) == "mn") {
        assay(df) <- impute_mode(df=assay(df), method="mn")
    } else if (tolower(method)=="md") {
        assay(df) <- impute_mode(df=assay(df), method="md")
    } else {
        stop("Error occurred. No method selected")
    }
    meta_data <- metadata(df)
    meta_data$processing_history$mv_imputation <- return_function_args()
    metadata(df) <- meta_data
    df <- return_original_data_structure(df)
    return(df)
}

context("test-mv_imputation")

test_that("Missing value imputation using knn method returns expected output", {
  out <- mv_imputation(df=testData$data, method="knn", check_df=FALSE)
  attributes(out)$processing_history <- NULL
  out <- as.data.frame(out)
  expect_equal(out, testData$mv_imputation_knn)
})

test_that("processing_history works with SummarizedExperiment class", {
  out <- mv_imputation(df=MTBLS79[1:20, ], method="knn", check_df=FALSE)
  expect_equal(processing_history(out)[[1]][[1]], "knn")
})

test_that("Missing value imputation using rf method returns expected output", {
  out <- mv_imputation(df=testData$data, method="rf")
  attributes(out)$processing_history <- NULL
  out <- as.data.frame(out)
  expect_equal(out, testData$mv_imputation_rf, tolerance = 0.15)
})

test_that("Missing value imputation using bpca method returns expected output", {
  expect_warning (out <- mv_imputation(df=t(testData$data), method="bpca"))
  attributes(out)$processing_history <- NULL
  out <- as.data.frame(out)
  expect_equal(out, testData$mv_imputation_bpca)
})

test_that("Missing value imputation using sv method returns expected output", {
  expect_warning (out <- mv_imputation(df=t(testData$data), method="sv"))
  attributes(out)$processing_history <- NULL
  out <- as.data.frame(out)
  expect_equal(out, testData$mv_imputation_sv)
})

test_that("Missing value imputation using mn method returns expected output", {
  expect_warning (out <- mv_imputation(df=t(testData$data), method="mn"))
  attributes(out)$processing_history <- NULL
  out <- as.data.frame(out)
  expect_equal(out, testData$mv_imputation_mn)
})

test_that("Missing value imputation using md method returns expected output", {
  expect_warning (out <- mv_imputation(df=t(testData$data), method="md"))
  attributes(out)$processing_history <- NULL
  out <- as.data.frame(out)
  expect_equal(out, testData$mv_imputation_md)
})

test_that("Missing value imputation stops if all values in the row are NA's", {
    out <- testData$data
    out[3,] <- NA
    expect_error(out <- mv_imputation(df=out, method="md"))
})

test_that("Missing value imputation stops if all values in the column are NA's", {
  out <- testData$data
  out[,3] <- NA
  expect_error(out <- mv_imputation(df=out, method="md"))
})

test_that("Missing value imputation stops if wrong method is selected", {
  out <- testData$data
  expect_error(out <- mv_imputation(df=out, method="Kn"))
})

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
    collapse = TRUE,
    comment = "#>"
)

## ----echo=TRUE, message=FALSE, warning=FALSE, eval=FALSE----------------------
# install.packages("gridExtra")
# 
# if (!requireNamespace("BiocManager", quietly = TRUE))
#     install.packages("BiocManager")
# BiocManager::install("pmp")

## ----message=FALSE, warning=FALSE---------------------------------------------
library(S4Vectors)
library(SummarizedExperiment)
library(pmp)
library(ggplot2)
library(reshape2)
library(gridExtra)

## ----eval=FALSE---------------------------------------------------------------
# help ("MTBLS79")

## -----------------------------------------------------------------------------
data("MTBLS79")

class <- MTBLS79$Class
batch <- MTBLS79$Batch
sample_order <- c(1:ncol(MTBLS79))

# Input data structure
MTBLS79

class[1:10]
batch[1:10]
sample_order[1:10]

## -----------------------------------------------------------------------------
data <- filter_peaks_by_fraction(df=MTBLS79, classes=class, method="QC",
    qc_label="QC", min_frac=0.8)

## -----------------------------------------------------------------------------
corrected_data <- QCRSC(df=data, order=sample_order, batch=batch, 
    classes=class, spar=0, minQC=4)

## ----message=FALSE, warning=FALSE, fig.height=5, fig.width=5------------------
plots <- sbc_plot (df=MTBLS79, corrected_df=corrected_data, classes=class, 
    batch=batch, output=NULL, indexes=c(1, 5, 30))
plots

## ----fig.width=6, fig.height=8------------------------------------------------

manual_color = c("#386cb0", "#ef3b2c", "#7fc97f", "#fdb462", "#984ea3", 
    "#a6cee3", "#778899", "#fb9a99", "#ffff33")

pca_data <- pqn_normalisation(MTBLS79, classes=class, qc_label="QC")
pca_data <- mv_imputation(pca_data, method="KNN", k=5, rowmax=0.5,
    colmax=0.5, maxp=NULL, check_df=FALSE)
pca_data <- glog_transformation(pca_data, classes=class, qc_label="QC")

pca_corrected_data <- pmp::pqn_normalisation(corrected_data, classes=class,
    qc_label="QC")
pca_corrected_data <- pmp::mv_imputation(pca_corrected_data, method="KNN", k=5,
    rowmax=0.5, colmax=0.5, maxp=NULL, check_df=FALSE)
pca_corrected_data <- pmp::glog_transformation(pca_corrected_data, 
    classes=class, qc_label="QC")

pca_data <- prcomp(t(assay(pca_data)), center=TRUE, scale=FALSE)
pca_corrected_data <- prcomp(t(assay(pca_corrected_data)),
    center=TRUE, scale=FALSE)

# Calculate percentage of explained variance of the first two PC's
exp_var_pca <- round(((pca_data$sdev^2)/sum(pca_data$sdev^2)*100)[1:2],2)
exp_var_pca_corrected <- round(((pca_corrected_data$sdev^2) /
    sum(pca_corrected_data$sdev^2)*100)[1:2],2)

plots <- list()

plotdata <- data.frame(PC1=pca_data$x[, 1], PC2=pca_data$x[, 2], 
    batch=as.factor(batch), class=class)

plots[[1]] <- ggplot(data=plotdata, aes(x=PC1, y=PC2, col=batch)) +
    geom_point(size=2) + theme(panel.background=element_blank()) +
    scale_color_manual(values=manual_color) +
    ggtitle("PCA scores, before correction") +
    xlab(paste0("PC1 (", exp_var_pca[1] ," %)")) +
    ylab(paste0("PC2 (", exp_var_pca[2] ," %)"))

plots[[2]] <- ggplot(data=plotdata, aes(x=PC1, y=PC2, col=class)) +
    geom_point(size=2) + theme(panel.background=element_blank()) +
    scale_color_manual(values=manual_color) +
    ggtitle("PCA scores, before correction") +
    xlab(paste0("PC1 (", exp_var_pca[1] ," %)")) +
    ylab(paste0("PC2 (", exp_var_pca[2] ," %)"))

plotdata_corr <- data.frame(PC1=pca_corrected_data$x[, 1], 
    PC2=pca_corrected_data$x[, 2], batch=as.factor(batch), class=class)

plots[[3]] <- ggplot(data=plotdata_corr, aes(x=PC1, y=PC2, col=batch)) +
    geom_point(size=2) +
    theme(panel.background=element_blank()) +
    scale_color_manual(values=manual_color) +
    ggtitle("PCA scores, after correction") +
    xlab(paste0("PC1 (", exp_var_pca_corrected[1] ," %)")) +
    ylab(paste0("PC2 (", exp_var_pca_corrected[2] ," %)"))

plots[[4]] <- ggplot(data=plotdata_corr, aes(x=PC1, y=PC2, col=class)) +
    geom_point(size=2) +
    theme(panel.background=element_blank()) +
    scale_color_manual(values=manual_color) +
    ggtitle("PCA scores, after correction") +
    xlab(paste0("PC1 (", exp_var_pca_corrected[1] ," %)")) +
    ylab(paste0("PC2 (", exp_var_pca_corrected[2] ," %)"))

grid.arrange(ncol=2, plots[[1]], plots[[2]], plots[[3]], plots[[4]])

## -----------------------------------------------------------------------------
sessionInfo()

#' Return list of function argument values
#' @return List of function argument values in the current function call
#' @noRd

return_function_args <- function()
{
    function_argument_names <- names(formals(sys.function(sys.parent(n=1))))
    # Don't list argument values for large objects. E.g. peak matrix, class
    # labels etc.
    args_black_list <- c("df", "classes", "ref_mean")
    function_argument_names <- function_argument_names[!function_argument_names
        %in% args_black_list]
    return(mget(function_argument_names, envir = parent.frame()))
}

#' Return history of applied functions and argument from pmp package.
#' 
#' @inheritParams filter_peaks_by_blank
#' @return List of function  names and argument values.
#' 
#' @examples
#' df <- MTBLS79[ ,MTBLS79$Batch == 1]
#' df$Class[1:2] <- "Blank"
#' out <- filter_peaks_by_blank(df=df, fold_change=1.2, 
#'    classes=df$Class, blank_label="Blank", qc_label=NULL, 
#'    remove_samples=FALSE, remove_peaks=TRUE, fraction_in_blank=0)
#' processing_history(out)
#' 
#' @export

processing_history <- function(df){
    if(is(df, "SummarizedExperiment")){
        processing_history <- metadata(df)$processing_history
    } else {
        processing_history <- attributes(df)$processing_history
    }
        
    return (processing_history)
}

#' @importFrom stats optimise
#' @import ggplot2
#' @importFrom methods as
#' @importFrom matrixStats rowVars
#'
NULL

#' Variance stabilising (extended) generalised logarithm transformation
#'
#' https://doi.org/10.1186/1471-2105-8-234
#'
#' @param y \code{numeric()}, values to be tranformed.
#' @param y0 \code{numeric(1)}, offset applied to y (default=0).
#' @param lambda \code{numeric(1)}, transform parameter.
#' @return vector \code{numeric()}, of transformed values.
#' @noRd

glog <- function(y, y0=0, lambda){
    z <- log((y - y0) + sqrt((y - y0)^2 + lambda))
    return(z)
}

#' Internal function for max. likelihood optimisation of glog params.
#' Calculates the alternative Jacobian function described in
#' https://doi.org/10.1186/1471-2105-8-234
#'
#' @param y \code{numeric()}, values.
#' @param y0 \code{numeric1}, offset applied to y (default=0).
#' @param lambda \code{numeric1}, lambda.
#' @return \code{numeric(1)}, optimised glog parameter.
#' @noRd

jglog <- function(y, y0=0, lambda){
    z <- glog(y, y0, lambda)
    D <- log(sqrt((y - y0)^2 + lambda))
    # average over all features (bins)
    gmn <- exp(colMeans(D, na.rm=TRUE))
    zj <- z * gmn  # ML estimate
    return(zj)
}

#' Internal function to estimate SSE for optimising glog params
#'
#' @param y \code{numeric()}, values.
#' @param y0 \code{numeric(1)}, offset applied to y (default=0).
#' @param lambda \code{numeric(1)}, transform parameter.
#' @return \code{numeric()}, sum of squared difference.
#' @noRd

SSE <- function(lambda, y0=0, y) {
    # calculate ML estimate
    z <- jglog(y, y0, lambda)
    # average over all features
    mean_spec <- rowMeans(z, 1, na.rm=TRUE)
    # calculate sum of squared difference between true and estimate
    s <- sum((z - mean_spec)^2, na.rm=TRUE)
    return(s)
}

#' If glog optimisation fails, this function will scale values in the 
#' peak matrix to the 1/mean(total signal) over all samples.
#' 
#' @param df \code{numeric()}, peak intensity matrix.
#' @return \code{numeric()}, scaled peak matrix.
#' @noRd

glog_rescale_data <- function(df){
    scal_fact <- colSums(df, na.rm=TRUE)
    scal_fact <- mean(scal_fact)
    scal_fact <- 1 / scal_fact
    df <- df * scal_fact # apply scaling factor
    return(df)
}

#' Search for optimal value of lambda by minimasing SSE.
#' 
#' @param upper_lim \code{numeric(1)}, upper limit to use for optimisation.
#' @param df_qc \code{numeric()}, peak matrix of QC samples.
#'  
#' @return \code{numeric(1)}, optimised glog lambda value.
#' @noRd

glog_omptimise_lambda <- function(upper_lim, df_qc){
    lambda <- optimise(f=SSE, interval=c(0, upper_lim), y0=0,
        y=df_qc, tol=1e-16)
    lambda
}

#' Plot SSE error of lambda optimisation process
#' @inheritParams filter_peaks_by_blank
#' @param optimised_lambda \code{numeric(1)}, value of optimised lambda from 
#'  glog_transformation output.
#' @param plot_grid \code{integer(1)}, number of data points to use for SSE 
#' optimisation.
#' @return Class \code{ggplot} object containing optimisation plot.
#' 
#' @examples 
#' data <- MTBLS79[, MTBLS79$Batch == 1]
#' classes <- data$Class
#' 
#' data <- mv_imputation(df=data, method='knn')
#' out <- glog_transformation (df=data, classes=classes,
#'     qc_label='QC')
#' 
#' optimised_lambda <- S4Vectors::metadata(out)
#' optimised_lambda <- 
#'     optimised_lambda$processing_history$glog_transformation$lambda_opt
#' 
#' glog_plot_optimised_lambda(df=data, classes=classes,
#'     qc_label="QC", optimised_lambda=optimised_lambda)
#' @export

glog_plot_optimised_lambda <- function(df, optimised_lambda, classes, qc_label,
    plot_grid=100){
    
    df <- check_input_data(df, classes=classes)
    df_qc <- df[, classes == qc_label]
    
    offset <- min(assay(df_qc), na.rm=TRUE)#set offset to the minimum QC samples
    assay(df_qc) <- assay(df_qc) - offset # set minimum of qc data to 0
    
    lambda_lim <- c(optimised_lambda, optimised_lambda) +
        c(-optimised_lambda*0.8, optimised_lambda*0.8) 
    sse_df <- data.frame(lambda=seq(lambda_lim[1], lambda_lim[2], 
        length.out=plot_grid))
    sse_df$SSE <- vapply(X=sse_df$lambda, FUN=SSE, y0=0, y=assay(df_qc), 
        FUN.VALUE=numeric(1))
    g <- ggplot(data=sse_df, aes(x=.data[['lambda']],y=.data[['SSE']])) +
        geom_vline(xintercept=optimised_lambda, color="red") +
        geom_line(linewidth=1.1) + theme_bw() +
        labs (title="glog parameter optimisation",
            caption=paste("optimum = ",round(optimised_lambda,2), sep=""))+
        xlab('lambda') + ylab('SSE')
    return (g)
}

#' Variance stabilising generalised logarithm (glog) transformation
#' 
#' Performs glog transformation on the data set. QC samples can be used to 
#' estimate technical variation in the data set and calculate transformation
#' parameter \eqn{\lambda} (lambda). QC samples usually comprise a pool of 
#' aliquots taken from all other samples in the study  and analysed repeatedly 
#' throughout an analytical batch.
#' 
#' Many univariate and multivariate statistical tests require homogeneity and
#' n ormality of dataset variance. Real-world metabolomics datasets often fail 
#' to meet these criteria due to asymmetric (i.e. non-'normal') and/or 
#' heteroscedatic (i.e. non-homogenous) variance structure. To address this 
#' issue, \code{glog} data transformations may be applied. \cr
#' For each cell within the data matrix, transform the raw value (x) according 
#' to:  \eqn{log10(x + sqrt(x^2 + \lambda))}. The parameter \eqn{\lambda} is 
#' typically calculated using quality control (QC) samples analysed throughout
#' an analysis batch.
#' 
#' @references Parsons HM et. al., BMC Bionf., 8(234), 2007. 
#' https://doi.org/10.1186/1471-2105-8-234
#'
#' @inheritParams filter_peaks_by_blank
#' @param lambda \code{NULL} or \code{numeric(1)}, if not \code{NULL} will use
#' provided value for glog lambda.
#' 
#' @return Object of class \code{SummarizedExperiment}. If input data are a 
#' matrix-like (e.g. an ordinary matrix, a data frame) object, function returns 
#' the same R data structure as input with all value of data type 
#' \code{numeric()}.
#' 
#' @examples
#' df <- MTBLS79[, MTBLS79$Batch == 1]
#' out <- mv_imputation(df=df, method="knn")
#' out <- glog_transformation (df=out, classes=df$Class,
#'     qc_label="QC")
#' 
#' @export glog_transformation

glog_transformation <- function(df, classes, qc_label, lambda=NULL) {
    # check if qc_label is present in the classes vector
    if (length(which(classes %in% qc_label)) == 0) {
        stop("QC sample label is not present. Check your qc_label parameter.")
    }
    df <- check_input_data(df=df, classes=classes)
    df_qc <- df[, classes == qc_label]
    
    # offset for QC samples
    offset <- min(assay(df_qc), na.rm=TRUE)#set offset to the minimum QC samples
    
    assay(df_qc) <- assay(df_qc) - offset # set minimum of qc data to 0
    assay(df) <- assay(df) - offset # apply qc offset to samples
    
    VF <- rowVars(assay(df_qc), na.rm=TRUE) # variance of all features
    # Upper limit max var or largest ratio max(var)/min(var)
    upper_lim <- max(pmax(VF, max(VF) / sort(VF)[sort(VF) > 0][1]))
    if (is.null(lambda)){
        lambda <- glog_omptimise_lambda (upper_lim=upper_lim, 
            df_qc=assay(df_qc))
        lambda <- lambda$minimum
    }
    lambda_opt <- lambda
    error_flag <- FALSE
    # if optimisation reached upper/lower limit then trigger use of fixed value
    if (abs(upper_lim - lambda) <= 1e-05 | abs(0 - lambda) <= 1e-05) {
        cat("Error!Lambda tending to infinity! Using standard\n")
        error_flag <- TRUE
    } 
    # if flag triggered then apply scale factor
    if (error_flag) {
        lambda <- 5.0278 * 10^(-9)
        assay(df) <- glog_rescale_data(assay(df))
    }

    # set minimum over all values to 0
    assay(df) <- glog(assay(df), 0, lambda) # apply glog
    meta_data <- metadata(df)
    meta_data$processing_history$glog_transformation <- 
        c(return_function_args(), list(lambda_opt=lambda_opt, 
            error_flag=error_flag))
    metadata(df) <- meta_data
    df <- return_original_data_structure(df)
    return(df)
}

context("test-glog_transformation")

test_that("Glog function returns expected output", {
  data <- mv_imputation(df=testData$data, method="knn")
  out <- glog_transformation (df=data, classes=testData$class,
    qc_label="QC")
  
  lambda <- as.integer(7865716)
  testthat::expect_true(lambda == as.integer(attributes(out)$
    processing_history$glog_transformation$lambda_opt))
  
  optimised_lambda <- attributes(out)
  optimised_lambda <- 
  optimised_lambda$processing_history$glog_transformation$lambda_opt
  
  graph <- glog_plot_optimised_lambda(
    df=data, 
    optimised_lambda=optimised_lambda,
    classes=testData$class, 
    qc_label="QC")
  
  expect_equal(graph$labels$x, "lambda")
  expect_equal(graph$labels$y, "SSE")
  
  attributes(out)$processing_history <- NULL
  expect_equal (as.data.frame(out), testData$glog_transformation)
})

test_that("Glog function fails if qc_label is wrong or QC samples don't exist", {
  out <- mv_imputation(df=testData$data, method="knn")
  expect_error (glog_transformation (df=out, classes=testData$class,
    qc_label="A"))
})

test_that("Scale factor is applied if optimisation fails", {
    df <- iris
    classes <- iris$Species
    df <- df[,-5]
    expect_output(glog_transformation(df,classes,qc_label='versicolor'),
     regexp='Error!Lambda')
})

test_that("If feature variance is 0 replace it with small value", {
    out <- mv_imputation(df=testData$data, method="knn")
    out[3,] <- 3
    expect_output(out <- glog_transformation (df=out, classes=testData$class,
      qc_label="QC"), regexp='Error!Lambda')
    out <- out[3,]
    testthat::expect_true(all(out[1:3] == out[4:6]))
})

#' @importFrom stats smooth.spline predict
#' @importFrom utils capture.output
NULL

##' Transform data using natural logarithm
##'
##' @param x Values to convert.
##' @param a log offset.
##' @param inverse To apply log or reverse back to original values.
##' 
##' @return vector of log transformed values
##' @noRd

logFit <- function (x, a=1, inverse=FALSE) {
    if (inverse) {
        out <- 0.25 * exp(-x) * (4 * exp(2 * x) - (a * a))
    }else{
        out <- log((x + sqrt(x^2 + a^2))/2)
    }
    return(out)
}

##' Cubic smoothing spline fitting function for batch effect correction
##'
##' @param x QC sample measurement order indices.
##' @param y Signal intensities.
##' @param newX Range of sample indices to predict.
##' @param log TRUE or FALSE to perform the signal correction fit on the log
##'scaled data. Default is TRUE.
##' @param a log scaling offset.
##' @param spar Smoothing parameter of the fitted curve. Should be in the range
##'0 to 1. If set to 0 it will be estimated using leave-one-out
##'cross-validation.
##' @param spar_lim A 2 element numeric vector containing the min and max
##'values of spar when searching for an optimum. Default \code{spar_lim = c(-1.5,1.5)}
##' @return vector of cubic smoothing spline fitted values
##' @noRd

splineSmoother <- function(x, y, newX, log=log, a=1, spar, 
    spar_lim=c(-1.5,1.5)) {
    if(log == TRUE) {
        y <- logFit(y, a=a)
    }
    
    control.spar=list(
        low = spar_lim[1],
        high = spar_lim[2]
    )
    
    if (spar == 0) {
        # Supress spline fitting CV messages cluttering terminal output
        smoothSplineMessages <- capture.output(sp.obj 
            <- smooth.spline(x, y, cv=TRUE,control.spar=control.spar), 
            file=NULL, type="message")
    } else {
        smoothSplineMessages <- character()
        sp.obj <- smooth.spline(x, y, spar=spar)
    }

    valuePredict <- predict(sp.obj, newX)

    if(log==TRUE) {
        valuePredict$y <- logFit(valuePredict$y, a=a, inverse=TRUE)
    }
    
    return(valuePredict$y)
}


##' Signal batch correction wrapper function
##'
##' @param id Feature id.
##' @param qcData Data frame with QC samples.
##' @param qcBatch QC batch numbers.
##' @param qcOrder QC measurement order.
##' @param spar Spline smoothing parameter. Should be in the range 0 to 1.
##'If set to 0 it will be estimated using leave-one-out cross-validation.
##' @param log TRUE or FALSE to perform the signal correction fit on the
##'log scaled data. Default is TRUE.
##' @param batch A vector indicating the batch each sample was measured in.
##'If only one batch was measured then all values should be set to 1.
##' @param minQC Minimum number of QC samples required for signal correction.
##' @param order A numeric vector indicating the order in which samples
##'were measured.
##' @param spar_lim A 2 element numeric vector containing the min and max
##'values of spar when searching for an optimum. Default \code{spar_lim = c(-1.5,1.5)}
##' @return vector of corrected values of selected feature for QC data
##' @noRd

sbcWrapper <- function(id, qcData, order, qcBatch, qcOrder, log=log, spar=spar,
    batch=batch, minQC, spar_lim = c(-1.5,1.5)) {
    
    out <- tryCatch ({

    # qc data for feature
    subData <- qcData[id, ]
    
    # unique batches
    nbatch <- unique(qcBatch)

    # preallocate output
    outl <- numeric(length(order))

    # for each batch
    for (nb in seq_len(length(nbatch))) {
        # get the injection order for the QCs
        x <- qcOrder[qcBatch == nbatch[nb]]
        # get the response for the QCs
        y <- subData[qcBatch == nbatch[nb]]
        
        # remove any index with NA in the response
        NAhits <- which(is.na(y))

        if (length(NAhits) > 0) {
            x <- x[-c(NAhits)]
            y <- y[-c(NAhits)]
        }

        # if we have enough QCs values
        if (length(y) >= minQC) {
            # fit spline to QCs and get predictions for all samples in batch
            outl[batch==nbatch[nb]] <- splineSmoother(x=x, y=y, newX=order[batch==nbatch[nb]], log=log,
            a=1, spar=spar, spar_lim=spar_lim)
        } else {
            # otherwise replace with NA
            outl[batch==nbatch[nb]] = NA
        }
    }

    outp=outl
    },

    error = function(e){
        print (e, "\n")
        return(NULL)
    })
    return(outp)
}

context ("sbcWrapper function")

classes <- MTBLS79$Class
batch <- MTBLS79$Batch
order <- c(1 : ncol(MTBLS79))

Data <- assay(MTBLS79)
qcData <- Data[,classes == "QC"]
qc_batch <- batch[classes == "QC"]
qc_order <- order[classes == "QC"]

test_that ("sbcWrapper returns expected output", {

  out <- sbcWrapper(id=4, qcData=qcData, order=order, qcOrder=qc_order, qcBatch=qc_batch,
                       log=TRUE, spar=0, batch=batch, minQC=4)

  expect_equal (out, testData$sbcWrapper, tolerance=10^-7)

})

test_that ("sbcWrapper returns expected output, with predefined spar value", {
  
  out <- sbcWrapper(id=4, qcData=qcData, order=order, qcOrder=qc_order, qcBatch=qc_batch,
                    log=TRUE, spar=1, batch=batch, minQC=4)
  
  expect_equal (out, testData$sbcWrapper_spar_1)
  
})

test_that ("sbcWrapper returns NA if not enough data points are measured", {
  
  qcData[4, -c(sample(38, 3))] <- NA
  
  out <- sbcWrapper(id=4, qcData=qcData, order=order, qcOrder=qc_order, qcBatch=qc_batch,
                    log=TRUE, spar=1, batch=batch, minQC=4)
  
  expect_true (all(is.na(out)))
  
})

test_that ("sbcWrapper returns error", {
  dim(qcData) <- NULL
  expect_error(sbcWrapper(id=4, qcData=qcData, order=order, qcOrder=qc_order, qcBatch=qc_batch,
                    log=TRUE, spar=1, batch=batch, minQC=4))
})

context ("test-splineSmoother function")

y_qc <- c(69573.34, 70434.15, 71633.42, 79970.91, 91370.66, 89932.12,
          83033.16)
x_qc <- c(1, 2, 3, 7, 13, 18, 23)
x_sample <- c(4, 5, 6, 8, 9, 10, 11, 12, 14, 15, 16, 17, 19, 20, 21, 22)

test_that ("splineSmoother returns expected output", {
  out <- splineSmoother(x=x_qc, y=y_qc, newX=x_sample,
    log=TRUE, spar=0, a=1)
  expect_equal (out, testData$splineSmoother, tolerance=10^-5)

})

#' @import SummarizedExperiment
#' @importFrom methods as
#' @importFrom methods is
#' @importFrom S4Vectors DataFrame
#' @importFrom S4Vectors metadata
NULL

#' Check if peak matrix is in format features in rows, samples in columns 
#' and that all cells contain numeric values. 
#' 
#' All functions in pmp pacakge expect input peak matrix to have samples 
#' as columns and measured features in rows. This function will check input 
#' matrix orientation and will transpose it if needed. If class labels are 
#' provided this function will check if the length of labels matches number 
#' of samples in peak matrix.
#'
#' @param df A matrix-like (e.g. and ordinary matrix, a data frame) object with
#' all values of class \code{numeric()} or \code{integer()} of peak
#' intensities, areas or other quantitative characteristic.
#' @param classes \code{character()}, vector of class labels. Must be the same 
#' length as the number of sample in the input peak table.
#' @return \code{numeric()}, matrix-like object where samples are represented
#' in columns and features in rows.
#' 
#' @noRd

check_peak_matrix <- function(df, classes=NULL) {
    dims <- dim(df)
    if (dims[1] < dims[2] & is.null(classes)) {
        df <- t(df)
        warning("Peak table was transposed to have features as rows and samples
    in columns. \n
    There were no class labels available please check that peak table is \n
    still properly rotated. \n
    Use 'check_df=FALSE' to keep original peak matrix orientation.")
    }
    if (!is.numeric(as.matrix(df))){
        stop ("Peak matrix contains non-numeric values. Check your inputs!")
    }
    if (!is.null(classes)) {
        hits <- which(dims == length(classes))
        if (length(hits) == 2) {
            warning(" Number of samples and features is the same in your
            data matrix, 
            please make sure that you samples are in columns. \n")
        } else if (length(hits) == 0) {
            stop(" Length of sample classes doesn't match any dimension
            of input data. 
            Sample labels should match number of samples. \n")
        } else if (hits == 1) {
            # If samples are in rows, transpose data matrix
            df <- t(df)
        }
    }
    df
}

#' If needed convert input data to object of class 
#' \link[SummarizedExperiment]{RangedSummarizedExperiment-class}
#' 
#' @param df A matrix-like (e.g. an ordinary matrix, a data frame) object with
#' all values of class \code{numeric()} or \code{integer()} of peak
#' intensities, areas or other quantitative characteristic.
#' @param classes \code{character()}, vector of class labels. Must be the same 
#' length as the number of sample in the input peak table.
#' @return object of class of 
#' \link[SummarizedExperiment]{RangedSummarizedExperiment-class}. 
#' 
#' @noRd
check_input_data <- function (df, classes=NULL){
    meta_data <- list(original_data_structure=class(df)[1])
    if(is(df, "SummarizedExperiment")){
        metadata(df)$original_data_structure <- "SummarizedExperiment"
    } else {
        if (meta_data$original_data_structure != "matrix"){
            df <- as.matrix(df)
        }
        df <- check_peak_matrix(df=df,
            classes=classes)
        col_names <- colnames(df)
        df <- SummarizedExperiment(assays=list(df))
        metadata(df) <- meta_data
        if (!is.null(classes)){
            colData(df) <- DataFrame(classes=classes)
        }
        colnames(df) <- col_names
    }
    return(df)
}

#' If input data object was not of class of 
#' \link[SummarizedExperiment]{RangedSummarizedExperiment-class},
#' convert output to the original R data structure. 
#' 
#' All values stored in \code{metadata} element of \code{SummarizedExperiment}
#' will be exported as \link[base]{attributes} of the output object.
#'
#' @param summarized_experiment_object object of class
#' \code{SummarizedExperiment}
#' @return A matrix-like (e.g. an ordinary matrix, a data frame) object with
#' all values of class \code{numeric()}. 
#' Values from \code{colData} and \code{rowData} elements are not returned. 
#' 
#' @noRd
return_original_data_structure <- function(summarized_experiment_object){
    meta_data <- metadata(summarized_experiment_object)
    if (!meta_data$original_data_structure == "SummarizedExperiment"){
        peak_data <- assay(summarized_experiment_object)
        # as() can't convert matrix to data.frame, but works with all other 
        # objects
        if (meta_data$original_data_structure == "data.frame"){
            peak_data <- as.data.frame(peak_data)
        } else if (meta_data$original_data_structure != "matrix"){
            peak_data <- as(peak_data, meta_data$original_data_structure)
        }
        # Add all metadata as output object attributes
        meta_data$original_data_structure <- NULL
        attributes(peak_data) <- c(attributes(peak_data), meta_data)
        if (ncol(rowData(summarized_experiment_object)) != 0){
            attributes(peak_data)$flags <- 
                rowData(summarized_experiment_object)
        }
        return (peak_data)
    } else {
        return (summarized_experiment_object)
    }
}

context("test data input/output functions")

test_that("Output data matrix is in correct orientation", {
  out <- check_peak_matrix(df=t(testData$data), classes=testData$class)
  expect_equal(out, testData$data)
})

test_that("Output data matrix is in correct orientation if no transpose is needed", {
  out <- check_peak_matrix(df=testData$data, classes=testData$class)
  expect_equal(out, testData$data)
})

test_that("Function fails", {
  expect_error(check_peak_matrix(df=testData$data, classes=testData$class[1:3]))
})

test_that("Function fails", {
  expect_error(check_peak_matrix(df=testData$data[,1:6], classes=testData$class))
})

test_that("Function fails, input is character", {
  out <- testData$data
  out[ ,1] <- as.character(out[ ,1])
  expect_error(check_peak_matrix(df=out, classes=testData$class))
})

test_that("Function works if class labels are not provided", {
  out <- check_peak_matrix(df=testData$data)
  expect_equal (out, testData$data)
})

test_that("Function works if class labels are not provided and matrix needs to be transposed", {
  expect_warning (out <- check_peak_matrix(df=t(testData$data)))
  expect_equal(out, testData$data)
})

test_that("Function works if class labels are not provided and matrix needs to be transposed,
          and data type is retained", {
  df <- data.frame(t(testData$data))
  expect_warning (out <- check_peak_matrix(df=df))
})

test_that("Function returns warning if input peak matrix has the same number of samples and features", {
  expect_warning (out <- check_peak_matrix(df=testData$data[1:9, ], 
    classes=testData$class))
})

test_that("return_original_data_structure work with bioconductor DataFrame object", {
  df <- S4Vectors::DataFrame(A=c(1:5), B=c(1:5))
  df <- check_input_data (df)
  meta_data <- metadata(df)
  meta_data$processing_history <- "testthat"
  metadata(df) <- meta_data
  expect_silent(df <- return_original_data_structure(df))
})

test_that("SummarizedExperiment has original data structure flag added", {
  df <- MTBLS79[, MTBLS79$Batch == 1]
  expect_silent(df <- check_input_data(df))
  expect_true(metadata(df)$original_data_structure == "SummarizedExperiment")
  expect_silent(df <- return_original_data_structure(df))
  expect_true (class(df)[1] == "SummarizedExperiment")
})

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
    collapse = TRUE,
    comment = "#>",
    fig.width=5,
    fig.height=5
)

## ----echo=TRUE, message=FALSE, warning=FALSE, eval=FALSE----------------------
# install.packages("gridExtra")
# 
# if (!requireNamespace("BiocManager", quietly = TRUE))
#     install.packages("BiocManager")
# BiocManager::install("pmp")

## ----message=FALSE, warning=FALSE---------------------------------------------
library(S4Vectors)
library(SummarizedExperiment)
library(pmp)
library(ggplot2)
library(reshape2)
library(gridExtra)

## ----eval=FALSE---------------------------------------------------------------
# help ("MTBLS79")

## -----------------------------------------------------------------------------
feature_names <- c("70.03364", "133.07379", "146.16519", "163.04515", 
    "174.89483", "200.03196", "207.07818", "221.05062", "240.02445",
    "251.03658", "266.01793", "304.99115", "321.07923", "338.98131", 
    "376.03962", "393.35878", "409.05716", "430.24353", "451.01086",
    "465.14937")
summary(t(SummarizedExperiment::assay(MTBLS79[feature_names, ])))

## -----------------------------------------------------------------------------
#number of samples:
ncol(MTBLS79)

## -----------------------------------------------------------------------------
#Batches:
unique(MTBLS79$Batch)

## -----------------------------------------------------------------------------
#Sample classes:
unique(MTBLS79$Class)

## ----fig.height=5, fig.width=4, message=FALSE, warning=FALSE------------------
#  separate the LCMS data from the meta data
data(MTBLS79)
data <- SummarizedExperiment::assay(MTBLS79[feature_names, ])
class <- SummarizedExperiment::colData(MTBLS79)$Class
batch <- SummarizedExperiment::colData(MTBLS79)$Batch
order <- c(1:ncol(data))

# get index of QC samples
QChits <- which(class == "QC")

# small function to calculate RSD%
FUN <- function(x) sd(x, na.rm=TRUE) / mean(x, na.rm=TRUE) * 100

# RSD% of biological and QC samples within all 8 batches:
out <- matrix(ncol=2, nrow=nrow(data))
colnames(out) <- c("Sample","QC")
rownames(out) <- rownames(data)

# for each feature calculate RSD% for the samples and the QCs
for (i in 1:nrow(data)) {
    out[i, 1] <- FUN(data[i, -QChits]) # for samples
    out[i, 2] <- FUN(data[i, QChits]) # for QCs
}

# prepare data for plotting
plotdata <- melt(data.frame(out), variable.name="Class", value.name="RSD")
plotdata$feature <- rownames(data)

plotdata$RSD <- round(plotdata$RSD,0)
plotdata$feature <- factor(plotdata$feature, ordered=TRUE,
    levels=unique(plotdata$feature))

# plot
ggplot(data=plotdata, aes(x=Class, y=feature, fill=RSD)) + 
    geom_tile() + 
    geom_text(aes(label=RSD)) +
    scale_fill_gradient2(low="black", mid="white", high="red")

## -----------------------------------------------------------------------------
ggplot(data=plotdata, aes(x=Class, y=RSD, fill=Class)) +
    geom_violin(draw_quantiles=c(0.25,0.5,0.75)) +
    ylab("RSD%") + 
    guides(fill=FALSE) +
    theme(panel.background=element_blank())

## ----message=FALSE, warning=FALSE, fig.height=6-------------------------------
# prepare some matrices to store the results
RSDQC <- matrix(ncol=8, nrow=nrow(data))
RSDsample <- matrix(ncol=8, nrow=nrow(data))
colnames(RSDQC) <- unique(batch)
colnames(RSDsample) <- unique(batch)

rownames(RSDQC) <- rownames(data)
rownames(RSDsample) <- rownames(data)

# for each feature
for (i in 1:nrow(data)) {
    # for each batch
    for (nb in 1:8) {
        # RSD% of QCs in this batch
        RSDQC[i, nb] <- FUN(data[i, which(class == "QC" & batch == nb)])
        # RSD% of samples in this batch
        RSDsample[i, nb] <- FUN(data[i, which(!class == "QC" & batch == nb)])
    }
}

# prepare results for plotting
plotdataQC <- melt(as.data.frame(RSDQC), variable.name="batch",
    value.name="RSD")
plotdataQC$Class <- "QC"

plotdataBio <- melt(as.data.frame(RSDsample), variable.name="batch",
    value.name="RSD")
plotdataBio$Class <- "Sample"

plotdata <- rbind(plotdataQC, plotdataBio)

plotdata$Class <- as.factor(plotdata$Class)

# plot
ggplot(data=plotdata, aes(x=Class, y=RSD, fill=Class)) + geom_boxplot() +
    facet_wrap(~ batch, ncol=3) +
    ylab("RSD%") +
    xlab("") +
    scale_x_discrete(labels=NULL) +
    theme(panel.background=element_blank(), axis.text.x=element_blank(),
        axis.ticks.x=element_blank())

## -----------------------------------------------------------------------------
summary(RSDQC)

## -----------------------------------------------------------------------------
summary(RSDsample)

## ----message=FALSE, warning=FALSE---------------------------------------------

# prepare a list of colours for plotting
manual_color = c("#386cb0", "#ef3b2c", "#7fc97f", "#fdb462", "#984ea3", 
    "#a6cee3", "#778899", "#fb9a99", "#ffff33")

# Function to calculate median absolute deviation
DRatfun <- function(samples, qcs) mad(qcs) / mad(samples)

# prepare matrix for dratio output
dratio <- matrix(ncol=8, nrow=nrow(data))
colnames(dratio) <- unique(batch)
rownames(dratio) <- rownames(data)

# calculate dratio for each feature, per batch
for (i in 1:nrow(dratio)){
    for (nb in 1:8) {
        dratio[i, nb] <- DRatfun(samples=data[i, which(!class == "QC" &
        batch == nb)], qcs=data[i, which(class == "QC" & batch == nb)])
    }
}

# prepare data for plotting
dratio <- as.data.frame(round(dratio, 2))

plotdata2 <- melt(dratio, variable.name="batch")
plotdata2$index <- rownames(data)
plotdata2$index <- factor(plotdata2$index, ordered=TRUE,
    levels=unique(plotdata2$index))

ggplot(data=plotdata2, aes(x=index, y=value, color=batch)) +
    geom_point(size=2) +
    xlab("index") + ylab("D-ratio") +
    geom_hline(yintercept=1) + theme(panel.background=element_blank()) +
    scale_color_manual(values=manual_color) +
    theme(axis.text.x=element_text(angle=90))

## ----fig.width=6.5, fig.height=5----------------------------------------------
pca_data <- MTBLS79[feature_names, ]

pca_data <- pqn_normalisation(pca_data, classes=class, qc_label="QC")
pca_data <- mv_imputation(pca_data, method="KNN", k=5, rowmax=0.5,
    colmax=0.5, maxp=NULL, check_df=FALSE)
pca_data <- glog_transformation(pca_data, classes=class, qc_label="QC")

pca_data <- prcomp(t(SummarizedExperiment::assay(pca_data)), center=TRUE, 
    scale=FALSE)
exp_var_pca <- round(((pca_data$sdev^2)/sum(pca_data$sdev^2)*100)[1:2], 2)

plots <- list()

plotdata <- data.frame(PC1=pca_data$x[, 1], PC2=pca_data$x[, 2],
    batch=as.factor(batch), class=class)

plots[[1]] <- ggplot(data=plotdata, aes(x=PC1, y=PC2, col=batch)) +
    geom_point(size=2) +
    theme(panel.background=element_blank()) +
    scale_color_manual(values=manual_color) +
    ggtitle("PCA scores, before correction") +
    xlab(paste0("PC1 (", exp_var_pca[1] ," %)")) +
    ylab(paste0("PC2 (", exp_var_pca[2] ," %)"))

plots[[2]] <- ggplot(data=plotdata, aes(x=PC1, y=PC2, col=class)) +
    geom_point(size=2) +
    theme(panel.background=element_blank()) +
    scale_color_manual(values=manual_color) +
    ggtitle("PCA scores plot, before correction") +
    xlab(paste0("PC1 (", exp_var_pca[1] ," %)")) +
    ylab(paste0("PC2 (", exp_var_pca[2] ," %)"))

grid.arrange(ncol=2, plots[[1]], plots[[2]])

## ----message=FALSE, warning=FALSE, fig.height=10------------------------------

# autoscale the QC data
QCdata <- data[ ,QChits]
QCdata2 <- as.data.frame(scale(t(QCdata), scale=TRUE, center=TRUE))

# prepare the data for plotting
plotdata <- melt(QCdata2, value.name="intensity")
plotdata$index <- rep(1:nrow(QCdata2), ncol(QCdata2))

plotdata$batch <- as.factor(batch[QChits])

# plot
ggplot(data=plotdata, aes(x=index, y=intensity, col=batch)) +
    geom_point(size=2) +
    facet_wrap(~ variable, ncol=4) +
    theme(panel.background=element_blank()) +
    scale_color_manual(values=manual_color)

## ----warning=FALSE, fig.height=10---------------------------------------------
ggplot(data=plotdata, aes(x=index, y=intensity, col=batch)) + 
    geom_point(size=2) +
    facet_wrap(~ variable, ncol=4) +
    geom_smooth(method="lm", se=TRUE, colour="black") +
    theme(panel.background=element_blank()) +
    scale_color_manual(values=manual_color)

## ----message=FALSE, warning=FALSE, fig.height=7.5-----------------------------
sampleorder <- c(1:ncol(QCdata))

correlations <- matrix(ncol=2, nrow=nrow(data))
rownames(correlations) <- rownames(data)
colnames(correlations) <- c("tau","p.value")
correlations <- as.data.frame(correlations)

for (coln in 1:nrow(data)) {
    stat <- cor.test(sampleorder, QCdata[coln, ], method="kendall")
    correlations$tau[coln] <- stat$estimate
    correlations$p.value[coln] <- stat$p.value
}

correlations


## ----warning=FALSE, message=FALSE---------------------------------------------

correlations <- matrix(ncol=8, nrow=nrow(data))
rownames(correlations) <- rownames(data)
colnames(correlations) <- unique(batch)

QCbatch <- batch[QChits]

for (coln in 1:nrow(data)) {
    for (bch in 1:8) {
        sampleorder <- scale(c(1:length(which(QCbatch==bch))),
        center=TRUE, scale=TRUE)

        if ((length(sampleorder) - 
        length(which(is.na(QCdata[coln, which(QCbatch==bch)])))) >= 3){
            correlations[coln, bch] <- cor.test(sampleorder,
            QCdata[coln, which(QCbatch==bch)], method="kendall")$estimate
        }
    }
}

round(correlations, 2)

## ----fig.wide=TRUE------------------------------------------------------------
plotdata <- as.data.frame(correlations)
plotdata$feature <- rownames(plotdata)

plotdata <- melt(plotdata, variable.name="batch")
plotdata$feature <- factor(plotdata$feature, ordered=TRUE,
    levels = unique(plotdata$feature))

ggplot(data=plotdata, aes(x=batch, y=feature, fill=value)) + 
    geom_tile() + scale_fill_gradient2()


## -----------------------------------------------------------------------------
sampleorder <- c(1:ncol(QCdata))

regressionout <- matrix(ncol=3, nrow=nrow(data))
rownames(regressionout) <- rownames(data)
colnames(regressionout) <- c("R2.adj","coefficient","p.value")
regressionout <- as.data.frame(regressionout)

for (coln in 1:nrow(data)) {
    tempdat <- data.frame(x=sampleorder, y=QCdata[coln, ])
    stat <- lm(x ~ y, data=tempdat)
    stat <- summary(stat)

    regressionout$R2.adj[coln] <- stat$adj.r.squared  
    regressionout$coefficient[coln] <- stat$coefficients[2,1]
    regressionout$p.value[coln] <- stat$coefficients[2,4]
}

regressionout

## -----------------------------------------------------------------------------
regPerBatch <- matrix(ncol=8, nrow=nrow(data))
rownames(regPerBatch) <- rownames(data)
colnames(regPerBatch) <- unique(batch)

QCbatch <- MTBLS79$Batch[QChits]

for (coln in 1:nrow(data)) {
    for (bch in 1:8) {
        sampleorder <- c(1:length(which(QCbatch == bch)))
        tempdat <- data.frame(x=sampleorder, y=QCdata[coln, 
            which(QCbatch==bch)])
        stat <- lm(x ~ y, data=tempdat)
        stat <- summary(stat)
        regPerBatch[coln,bch] <- stat$adj.r.squared
    }
}

round(regPerBatch,2)

## ----fig.wide=TRUE------------------------------------------------------------
plotdata <- as.data.frame(regPerBatch)
plotdata$feature <- rownames(plotdata)

plotdata <- melt(plotdata, variable.name="batch")
plotdata$feature <- factor(plotdata$feature, ordered=TRUE, 
    levels=unique(plotdata$feature))

ggplot(data=plotdata, aes(x=batch, y=feature, fill=value)) + 
    geom_tile() + scale_fill_gradient2()

## ----warning=FALSE, message=FALSE---------------------------------------------
data <- data.frame(data=
    as.vector(SummarizedExperiment::assay(MTBLS79["451.01086", ])), batch=batch,
    class=factor(class, ordered=TRUE))
data$order <- c(1:nrow(data))
data$batch <- as.factor(data$batch)

ggplot(data=data, aes(x=order, y=log(data,10), col=batch, shape=class)) +
    geom_point(size=2) + theme(panel.background=element_blank()) +
    scale_color_manual(values=manual_color)

## -----------------------------------------------------------------------------

QCdata <- data[data$class == "QC",]

ggplot(data=QCdata, aes(x=order, y=log(data,10), col=batch, shape=class)) +
    geom_point(size=2) + theme(panel.background=element_blank()) +
    scale_color_manual(values=manual_color, drop=FALSE) +
    scale_shape_manual(values=c(16, 17, 15), drop=FALSE)

## -----------------------------------------------------------------------------
FUN <- function(x) sd(x, na.rm=TRUE)/mean(x, na.rm=TRUE) * 100

# RSD% of biological and QC samples within all 6 batches:
out <- c(NA,NA)
names(out) <- c("Biological","QC")
out[1] <-FUN(data$data[-QChits])
out[2] <-FUN(data$data[QChits])
out

## -----------------------------------------------------------------------------
# RSD% per batch:
out <- matrix(ncol=8,nrow=2)
colnames(out) <- unique(batch)
rownames(out) <- c("Biological","QC")
for (i in 1:8) {
    out[1, i] <- FUN(data$data[which(!class=="QC" & batch==i)])
    out[2, i] <- FUN(data$data[which(class=="QC" & batch==i)])
}

out

## -----------------------------------------------------------------------------
qcData <- data$data[class == "QC"]
qc_batch <- batch[class == "QC"]
qc_order <- order[class == "QC"]

qcData


## -----------------------------------------------------------------------------
nbatch <- unique(qc_batch)

nb <- 6

# Sample injection order
x <- qc_order[qc_batch==nbatch[nb]]

# Measured peak intensity or area
y <- qcData[qc_batch==nbatch[nb]]
y

## -----------------------------------------------------------------------------
NAhits <- which(is.na(y))
if (length(NAhits)>0) {
    x <- x[-c(NAhits)]
    y <- y[-c(NAhits)]
    rbind(x,y)
}

## -----------------------------------------------------------------------------
y <- log((y + sqrt(y^2)) / 2)
y

## -----------------------------------------------------------------------------
sp.obj <- smooth.spline(x, y, cv=TRUE) 
sp.obj

out <- rbind(y,sp.obj$y)
row.names(out) <- c("measured","fitted")
out

## -----------------------------------------------------------------------------
valuePredict=predict(sp.obj, order[batch==nb])

plotchr <- as.numeric(data$class)

# reverse the log transformation to convert the predictions back to the
# original scale
valuePredict$y <- exp(valuePredict$y)

plotdata <- data.frame(measured=data$data[batch==nb], fitted=valuePredict$y,
    Class=class[batch==nb], order=order[batch==nb])
plotdata2 <- melt(plotdata, id.vars=c("Class","order"), value.name="intensity",
    variable.name="data")

ggplot(data=plotdata2, aes(x=order, y=log(intensity,10), color=data,
    shape=Class)) + geom_point(size=2) +
    theme(panel.background=element_blank()) +
    scale_color_manual(values=manual_color) +
    scale_shape_manual(values=c(16, 17, 15), drop=FALSE)

## ----warning=FALSE------------------------------------------------------------
fitmedian <- median(plotdata$measured, na.rm=TRUE)
plotdata$corrected_subt <- (plotdata$measured - plotdata$fitted) + fitmedian

plotdata2 <- melt(plotdata, id.vars=c("Class","order"), 
    value.name="intensity", variable.name="data")

plotdata_class <- as.character(plotdata2$Class)
plotdata_class[plotdata_class == "S"] <- "Sample"
plotdata_class[plotdata_class == "C"] <- "Sample"
plotdata2$Class <- factor(plotdata_class)

ggplot(data=plotdata2, aes(x=order, y=intensity, color=data, shape=Class)) +
    geom_point(size=2) + theme(panel.background=element_blank()) + 
    scale_color_manual(values=manual_color) +
    facet_grid(Class ~ .) +
    scale_shape_manual(values=c(17, 16), drop=FALSE)

## ----warning=FALSE------------------------------------------------------------
plotdata$corrected_div <- plotdata$measured/(plotdata$fitted/fitmedian)

plotdata3 <- plotdata[,c("Class", "order", "corrected_subt", "corrected_div")]

plotdata3 <- melt(plotdata3, id.vars=c("Class","order"), 
    value.name="intensity", variable.name="data")

plotdata_class <- as.character(plotdata3$Class)
plotdata_class[plotdata_class=="S"] <- "Sample"
plotdata_class[plotdata_class=="C"] <- "Sample"
plotdata3$Class <- factor(plotdata_class)

ggplot(data=plotdata3, aes(x=order, y=intensity, color=data, shape=Class)) +
    geom_point(size=2) + theme(panel.background=element_blank()) +
    scale_color_manual(values=manual_color) +
    geom_smooth(se=FALSE) + facet_grid(Class ~ .)


## ----warning=FALSE------------------------------------------------------------

outl <- rep(NA, nrow(data))

for (nb in 1:length(nbatch)){
    # assigning sample injection order for a batch to 'x', and corresponding 
    # intensities to 'y'
    x <- qc_order[qc_batch == nbatch[nb]]
    y <- qcData[qc_batch == nbatch[nb]]

    # remove measurements with missing values
    NAhits <- which(is.na(y))
    if (length(NAhits) > 0) {
        x <- x[-c(NAhits)]
        y <- y[-c(NAhits)]
    }

    # require at least 4 data points for QC fit
    if (length(y) >= 4) {
        range <- c(batch == nbatch[nb])
        # Order is a vector of sample indices for the current batch
        outl[range] <- pmp:::splineSmoother(x=x, y=y, newX=order[range], 
        log=TRUE, a=1, spar=0)

        # If less than 5 data points are present, return empty values   
    } else {
        range <- c(batch == nbatch[nb])
        outl[range] <- rep(NA, nrow(data))[range]
    }
}

plotdata <- data.frame(measured=data$data, fitted=outl, Class=class, 
    batch=batch, order=c(1:nrow(data)))
plotdata2 <- melt(plotdata, id.vars=c("batch","Class","order"),
    value.name="intensity", variable.name="data")

ggplot(data=plotdata2, aes(x=order, y=log(intensity,10), 
    color=data, shape=Class)) + geom_point(alpha=0.5, size=2) +
    theme(panel.background=element_blank()) + 
    scale_color_manual(values=manual_color)


## ----warning=FALSE------------------------------------------------------------

# median intensity value is used to adjust batch effect

mpa <- rep(NA, nrow(data))

for (bch in 1:8) {
    mpa[batch==bch] <- median(data$data[batch==bch], na.rm=TRUE)
}


QC_fit <- outl/mpa

# and correct actual data
res <- data$data/QC_fit

# correct data using subtratcion
res2 <- (data$data -outl) +mpa

plotdata <- data.frame(measured=data$data, corrected_subt=res2, 
    corrected_div=res, Class=class, batch=batch, order=c(1:nrow(data)))
plotdata2 <- melt(plotdata, id.vars=c("batch","Class","order"), 
    value.name="intensity", variable.name="data")

ggplot(data=plotdata2, aes(x=order, y=log(intensity,10), 
    color=data, shape=Class)) + geom_point(alpha=0.2, size=2) +
    theme(panel.background=element_blank()) +
    scale_color_manual(values=manual_color) +
    geom_smooth(se=FALSE) +
    facet_grid(Class ~ .)

## ----warning=FALSE------------------------------------------------------------
mpa <- rep(NA, nrow(data))

for (bch in 1:8) {
    mpa[batch == bch] <- median(res2[batch == bch], na.rm=TRUE)
}

grandMedian <- median(res2, na.rm=TRUE)

mpa <- mpa - grandMedian

plotdata$corrected_subt <- plotdata$corrected_subt - mpa

mpa <- rep(NA, nrow(data))

for (bch in 1:8) {
    mpa[batch == bch] <- median(res[batch == bch], na.rm=TRUE)
}

grandMedian <- median(res, na.rm=TRUE)

mpa <- mpa - grandMedian

plotdata$corrected_div <- plotdata$corrected_div - mpa

plotdata2 <- melt(plotdata, id.vars=c("batch","Class","order"),
    value.name="intensity", variable.name="data")

ggplot(data=plotdata2, aes(x=order, y=log(intensity,10),
    color=data, shape=Class)) + geom_point(alpha=0.2, size=2) +
    theme(panel.background=element_blank()) +
    scale_color_manual(values=manual_color) +
    geom_smooth(se=FALSE) +
    facet_grid(Class ~ .)

## ----warning=FALSE------------------------------------------------------------
FUN <- function(x) sd(x, na.rm=TRUE)/mean(x, na.rm=TRUE) * 100

# RSD% of biological and QC samples within all 6 batches:
out <- matrix(nrow=2, ncol=2)
colnames(out) <- c("Biological","QC")
rownames(out) <- c("measured", "corrected")
out[1,1] <-FUN(data$data[-QChits])
out[1,2] <-FUN(data$data[QChits])
out[2,1] <-FUN(res[-QChits])
out[2,2] <-FUN(res[QChits])

round(out, 2)

# RSD% per batch:
out <- matrix(ncol=8,nrow=4)
colnames(out) <- unique(batch)
rownames(out) <- c("Biological","QC","Corrected biological","Corrected QC")
for(i in 1:8) {
    out[1, i] <- FUN(data$data[which(!class=="QC" & batch==i)])
    out[2, i] <- FUN(data$data[which(class=="QC" & batch==i)])
    out[3, i] <- FUN(res[which(!class=="QC" & batch==i)])
    out[4, i] <- FUN(res[which(class=="QC" & batch==i)])
}

round(out, 2)

## ----message=FALSE, warning=FALSE, include=TRUE-------------------------------
data <- MTBLS79[feature_names,]

class <- MTBLS79$Class
batch <- MTBLS79$Batch
sample_order <- c(1:ncol(data))

corrected_data <- QCRSC(df=data, order=sample_order, batch=batch,
    classes=class, spar=0, minQC=4)

## ----warning=FALSE------------------------------------------------------------
data <- SummarizedExperiment::assay(data)
corrected_data <- SummarizedExperiment::assay(corrected_data)
RSDQC <- matrix(ncol=8, nrow=nrow(data))
RSDsample <- matrix(ncol=8, nrow=nrow(data))
colnames(RSDQC) <- unique(batch)
colnames(RSDsample) <- unique(batch)

RSDQC_corrected <- matrix(ncol=8, nrow=nrow(data))
RSDsample_corrected <- matrix(ncol=8, nrow=nrow(data))
colnames(RSDQC_corrected) <- unique(batch)
colnames(RSDsample_corrected) <- unique(batch)

rownames(RSDQC) <- rownames(data)
rownames(RSDsample) <- rownames(data)
rownames(RSDQC_corrected) <- rownames(data)
rownames(RSDsample_corrected) <- rownames(data)

# for each feature
for (i in 1:nrow(data)) {
    # for each batch
    for (nb in 1:8) {
        # RSD% of QCs in this batch
        RSDQC[i, nb] <- FUN(data[i, which(class == "QC" & batch == nb)])
        # RSD% of samples in this batch
        RSDsample[i, nb] <- FUN(data[i, which(!class == "QC" & batch == nb)])
        # RSD% of QCs in this batch after correction
        RSDQC_corrected[i, nb] <- FUN(corrected_data[i, which(class == "QC"
            & batch == nb)])
        # RSD% of samples in this batch after correction
        RSDsample_corrected[i, nb] <- FUN(corrected_data[i, which(!class == "QC"
            & batch == nb)]) 
    }
}

# prepare results for plotting
plotdataQC <- melt(as.data.frame(RSDQC), variable.name="batch",
    value.name="RSD")
plotdataQC$Class <- "QC"

plotdataBio <- melt(as.data.frame(RSDsample), variable.name="batch",
    value.name="RSD")
plotdataBio$Class <- "Sample"

plotdataQC_corrected <- melt(as.data.frame(RSDQC_corrected),
    variable.name="batch", value.name="RSD")
plotdataQC_corrected$Class <- "QC_corr"

plotdataBio_corrected <- melt(as.data.frame(RSDsample_corrected),
    variable.name="batch", value.name="RSD")
plotdataBio_corrected$Class <- "Sample_corr"

plotdata <- rbind(plotdataQC, plotdataQC_corrected)

plotdata$Class <- as.factor(plotdata$Class)

# plot
ggplot(data=plotdata, aes(x=Class, y=RSD, fill=Class)) + geom_boxplot() +
    facet_wrap(~ batch, ncol=3) +
    ylab("RSD%") +
    xlab("") +
    scale_x_discrete(labels=NULL) +
    theme(panel.background=element_blank(), axis.text.x=element_blank(),
        axis.ticks.x=element_blank()) +
    scale_y_continuous(limits=c(0, 50))

plotdata <- rbind(plotdataBio, plotdataBio_corrected)

plotdata$Class <- as.factor(plotdata$Class)

# plot
ggplot(data=plotdata, aes(x=Class, y=RSD, fill=Class)) + geom_boxplot() +
    facet_wrap(~ batch, ncol=3) +
    ylab("RSD%") +
    xlab("") +
    theme(panel.background=element_blank(), axis.text.x=element_blank(),
        axis.ticks.x=element_blank())


## ----fig.width=6, fig.height=8------------------------------------------------
# PQN used to normalise data
# KNN for missing value imputation
# glog scaling method 
# A more detailed overview is detailed in
# Di Guida et al, Metabolomics, 12:93, 2016
# https://dx.doi.org/10.1007/s11306-016-1030-9

pca_data <- pqn_normalisation(data, classes=class, qc_label="QC")
pca_data <- mv_imputation(pca_data, method="KNN", k=5, rowmax=0.5,
    colmax=0.5, maxp=NULL, check_df=FALSE)
pca_data <- glog_transformation(pca_data, classes=class, qc_label="QC")

pca_corrected_data <- pqn_normalisation(corrected_data, classes=class,
    qc_label="QC")
pca_corrected_data <- mv_imputation(pca_corrected_data, method="KNN", k=5,
    rowmax=0.5, colmax=0.5, maxp=NULL, check_df=FALSE)
pca_corrected_data <- glog_transformation(pca_corrected_data, 
    classes=class, qc_label="QC")

pca_data <- prcomp(t(pca_data), center=TRUE, scale=FALSE)
pca_corrected_data <- prcomp(t(pca_corrected_data), center=TRUE, scale=FALSE)

# Calculate percentage of explained variance of the first two PC's
exp_var_pca <- round(((pca_data$sdev^2)/sum(pca_data$sdev^2)*100)[1:2],2)
exp_var_pca_corrected <- round(((pca_corrected_data$sdev^2) /
    sum(pca_corrected_data$sdev^2)*100)[1:2],2)

plots <- list()

plotdata <- data.frame(PC1=pca_data$x[, 1], PC2=pca_data$x[, 2], 
    batch=as.factor(batch), class=class)

plots[[1]] <- ggplot(data=plotdata, aes(x=PC1, y=PC2, col=batch)) +
    geom_point(size=2) + theme(panel.background=element_blank()) +
    scale_color_manual(values=manual_color) +
    ggtitle("PCA scores, before correction") +
    xlab(paste0("PC1 (", exp_var_pca[1] ," %)")) +
    ylab(paste0("PC2 (", exp_var_pca[2] ," %)"))

plots[[2]] <- ggplot(data=plotdata, aes(x=PC1, y=PC2, col=class)) +
    geom_point(size=2) + theme(panel.background=element_blank()) +
    scale_color_manual(values=manual_color) +
    ggtitle("PCA scores, before correction") +
    xlab(paste0("PC1 (", exp_var_pca[1] ," %)")) +
    ylab(paste0("PC2 (", exp_var_pca[2] ," %)"))

plotdata_corr <- data.frame(PC1=pca_corrected_data$x[, 1], 
    PC2=pca_corrected_data$x[, 2], batch=as.factor(batch), class=class)

plots[[3]] <- ggplot(data=plotdata_corr, aes(x=PC1, y=PC2, col=batch)) +
    geom_point(size=2) +
    theme(panel.background=element_blank()) +
    scale_color_manual(values=manual_color) +
    ggtitle("PCA scores, after correction") +
    xlab(paste0("PC1 (", exp_var_pca_corrected[1] ," %)")) +
    ylab(paste0("PC2 (", exp_var_pca_corrected[2] ," %)"))

plots[[4]] <- ggplot(data=plotdata_corr, aes(x=PC1, y=PC2, col=class)) +
    geom_point(size=2) +
    theme(panel.background=element_blank()) +
    scale_color_manual(values=manual_color) +
    ggtitle("PCA scores, after correction") +
    xlab(paste0("PC1 (", exp_var_pca_corrected[1] ," %)")) +
    ylab(paste0("PC2 (", exp_var_pca_corrected[2] ," %)"))

grid.arrange(ncol=2, plots[[1]], plots[[2]], plots[[3]], plots[[4]])

## -----------------------------------------------------------------------------
sessionInfo()

#' Internal data set for testhat unit tests
#'
#' Subset of DIMS data set to test package functions
#'
#' @source https://www.ebi.ac.uk/metabolights/MTBLS79
#' @noRd
"testData"

#' Direct-infusion mass spectrometry (DIMS) data set
#'
#' Data set of 20 biological (cow vs sheep) serum samples that were analysed
#' repeatedly, in 8 batches across 7 days.
#'
#' Code below includes all commands used to generate MTBLS79 object. \cr
#' 
#' \preformatted{
#' library (openxlsx)
#' library (SummarizedExperiment)
#' 
#' download.file(destfile = "MTBLS79.xlsx", mode="wb",
#' url = "ftp://ftp.ebi.ac.uk/pub/databases/metabolights/studies/public/ 
#'    MTBLS79/Dataset07__SFPM.xlsx") 
#' wb <- openxlsx::loadWorkbook(xlsxFile="MTBLS79.xlsx")
#'
#' MTBLS79 <- list()
#'
#' MTBLS79$assay <- openxlsx::readWorkbook(wb, "data", colNames=T, rowNames=T)
#' 
#' # Last row of the peak matrix represent mean intensities across all samples.
#' MTBLS79$assay <- MTBLS79$assay[-c(nrow(MTBLS79$assay)), ]
#'
#' # Transpose peak matrix, so that features are in rows and samples in columns.
#' MTBLS79$assay <- as.matrix(t(MTBLS79$assay))
#'
#' # Missing values in the input data are stored as 0, replace with NA
#' MTBLS79$assay[MTBLS79$assay == 0] <- NA
#' 
#' rownames(MTBLS79$assay) <- round(as.numeric(rownames(MTBLS79$assay)), 5)
#' 
#' MTBLS79$colData <- openxlsx::readWorkbook(wb, "meta", colNames=T, rowNames=F)
#' MTBLS79$colData <- MTBLS79$colData[-c(nrow(MTBLS79$colData)), 1:4]
#' 
#' MTBLS79 <- SummarizedExperiment(assays=list(MTBLS79$assay), 
#' colData=DataFrame(MTBLS79$colData))
#' }
#'
#' @references Kirwan et al, Scientific Data volume 1,
#'Article number: 140012 (2014) https://www.nature.com/articles/sdata201412
#'
#' @format A \link[SummarizedExperiment]{RangedSummarizedExperiment-class} 
#' object. \cr
#' \code{assay(MTBLS79)} Peak intensities of the DIMS data set. 
#' Contains 172 samples and 2488 features. \cr
#' \code{colData(MTBLS79)} Sample meta data containing 4 columns. \cr
#'     \code{Batch} - \code{character()}, sample batch name. \cr
#'     \code{Sample_Rep} - \code{character()}, sample replicate code. \cr
#'     \code{Class} - \code{character()}, sample class labels. \cr
#'     \code{Class2} - \code{character()}, alternative sample class labels
#'     grouping together replicate samples. \cr
# 
#' @source https://www.ebi.ac.uk/metabolights/MTBLS79
#' 
#' @docType data
#' 
"MTBLS79"

#' @import ggplot2
#' @importFrom grDevices pdf dev.off
#' @importFrom reshape2 melt
NULL

#' Plot QCRSC corrected outputs
#' 
#' Plot the output from signal batch correction for the selected or
#' the first 100 features.
#'
#' @inheritParams filter_peaks_by_blank
#' @param corrected_df Output from \link[pmp]{QCRSC} function.
#' @param batch \code{numeric()} or \code{character()}, a vector indicating
#' the batch each sample was measured in. If only one batch was measured then
#' all values should be set to 1
#' @param indexes \code{numeric()}, a vector defining which features to plot.
#' If set to \code{NULL} will plot the first 100.
#' @param output \code{character()}, a filename of the output pdf file. 
#' Can include the path. If set to \code{NULL} output will be list object 
#' containing class \code{ggplot} plots.
#' 
#' @return Pdf file or \code{list()} object \code{ggplot} class showing data 
#' before and after signal correction.
#' 
#' @examples 
#' 
#' order <- c(1:ncol(MTBLS79))
#' data <- MTBLS79[1:10, ]
#' 
#' out <- QCRSC(df =data, order=order, batch=MTBLS79$Batch, 
#'    classes=MTBLS79$Class, spar=0, minQC=4)
#' plots <- sbc_plot (df=data, corrected_df=out, classes=MTBLS79$Class,
#'    batch=MTBLS79$Batch, output=NULL) 
#'
#' @export

sbc_plot <- function(df, corrected_df, classes, batch, indexes = NULL,
    qc_label="QC", output = "sbcms_plots.pdf") {
    df <- check_input_data(df=df, classes=classes)
    corrected_df <- check_input_data(df=corrected_df, classes=classes)
    shapes <- rep(19, length(classes))
    shapes[classes == qc_label] <- 3
    manual_color <- c("#386cb0", "#ef3b2c", "#7fc97f", "#fdb462", "#984ea3", 
        "#a6cee3", "#778899", "#fb9a99", "#ffff33")
    gg_THEME <- theme(panel.background = element_blank(), 
        panel.grid.major = element_line(color = "gray80", linewidth = 0.3), 
        axis.line = element_line(color = "black"), 
        axis.text = element_text(color = "black"), 
        axis.title = element_text(color = "black"), 
        panel.grid.minor.x = element_line(color = "gray80", 
            linewidth = 0.3, linetype = "dashed"), 
        panel.grid.minor.y = element_line(color = "gray80", linewidth = 0.3))
    plots <- list()
    if (is.null(indexes) & nrow(df) >= 100) {
        indexes <- seq_len(100)
    } else if (nrow(df) < 100 & is.null(indexes)) {
        indexes <- seq_len(nrow(df))
    }
    for (peakn in indexes) {
        A <- data.frame(x = c(seq_len(ncol(df))), 
            original = as.vector(log(assay(df[peakn, ]), 10)),
            corrected = as.vector(log(assay(corrected_df[peakn, ]), 10)), 
            batch = as.factor(batch), shapes = shapes)
        A <- melt(A, id.vars = c("x", "batch", "shapes"))
        
        plots[[peakn]] <- ggplot(A, 
            aes(x = .data[['x']], 
                y = .data[['value']], 
                col = .data[['batch']], 
                shape = .data[['shapes']])) + 
            facet_grid(variable ~ .) + geom_point() + 
            scale_shape_identity() + geom_point(size=2) +
            scale_colour_manual(values = manual_color) + 
            ggtitle(row.names(df)[peakn]) + ylab("log10(intensity)") + 
            xlab("injection order") + gg_THEME
    }
    # remove lists with NULL values
    plots <- plots[lengths(plots) != 0]
    if (!is.null(output)) {
        pdf(output)
        invisible(lapply(plots, print))
        dev.off()
    } else {
        return(plots)
    }
}

context ("Test sbc_plot function")

classes <- MTBLS79$Class
batch <- MTBLS79$Batch
order <- c(1 : ncol(MTBLS79))
data <- assay(MTBLS79)[1:10, ]

test_that ("sbc_plot returns ggplot output", {

  out <- QCRSC(df = data, order = order, batch = batch, classes = classes,
    spar = 0, minQC = 4)

  plots <- sbc_plot(df = data, corrected_df = out, 
    classes, batch, output=NULL, indexes=c(1,5,7))
  
  testthat::expect_s3_class(plots[[1]], "ggplot")
  # Empty plots are getting removed from the list object
  testthat::expect_true(length(plots) == 3)

})

test_that ("sbcmsPlot returns ggplot output, with different QC sample label", {
  
  classes2 <- classes
  classes2[classes == "QC"] <- "Quality"

  out <- QCRSC(df = data, order = order, batch = batch, classes = classes2,
               spar = 0, minQC = 4, qc_label="Quality")
  
  plots <- sbc_plot(df = data, corrected_df = out, 
                     classes, batch, output=NULL, indexes=c(1,5,7), qc_label="Quality")
  
  testthat::expect_s3_class(plots[[1]], "ggplot")
  # Empty plots are getting removed from the list object
  testthat::expect_true(length(plots) == 3)
  
})

test_that ("sbcmsPlot returns pdf output", {
  
  data <- data[1:2, ]
  out <- QCRSC(df = data, order = order, batch = batch, classes = classes,
    spar = 0, minQC = 4)
  pdf_output <- tempfile()
  testthat::expect_warning(sbc_plot (df = data, corrected_df = out, classes, 
    batch, output=pdf_output))
  
  testthat::expect_true(file.exists(pdf_output))
  testthat::expect_true(file.remove(pdf_output))
})

test_that ("sbcmsPlot returns output for the first 100 features if
  indexes=NULL", {
  
  data <- assay(MTBLS79)[1:101, ]
  
  out <- QCRSC(df = data, order = order, batch = batch, classes = classes,
               spar = 0, minQC = 4)
  
  plots <- sbc_plot (df = data, corrected_df = out, classes, batch, output=NULL)
  
  testthat::expect_true(length(plots) == 100)
})

#' @importFrom matrixStats rowAnyMissings

NULL

#' Normalisation by total sum of the features per sample
#' 
#' For each sample, every feature intensity value is divided by the total sum of
#' all feature intensity values measured in that sample (\code{NA} values
#' ignored by default), before multiplication by 100; the unit is \%.
#' 
#' @inheritParams mv_imputation
#' 
#' @return Object of class \code{SummarizedExperiment}. If input data are a 
#' matrix-like (e.g. an ordinary matrix, a data frame) object, function returns 
#' the same R data structure as input with all value of data type 
#' \code{numeric()}.
#' 
#' @examples 
#' df <- MTBLS79[ ,MTBLS79$Batch == 1]
#' out <- normalise_to_sum (df=df)
#'
#' @export

normalise_to_sum <- function(df, check_df=TRUE) {
    if (check_df == TRUE) {
        df <- check_input_data(df)
    } else {
        # normalise_to_sum doesn't need class labels
        # Create generic class label vector to avoid DF to be transposed
        df <- check_input_data(df=df, classes=rep("S", ncol(df)))
    }
    assay(df) <- (sweep(assay(df), 2, colSums(assay(df), na.rm=TRUE)/100, 
        FUN="/"))
    meta_data <- metadata(df)
    meta_data$processing_history$normalise_to_sum <- 
        return_function_args()
        #list (check_df=check_df)
    metadata(df) <- meta_data
    df <- return_original_data_structure(df)
    df
}

#' Calculate reference mean of samples
#' 
#' @param df_qc \code{numeric()}, peak matrix of QC samples.
#' 
#' @return vector of reference mean values
#' @noRd
calculate_ref_mean <- function(df_qc,ref_method='mean'){
    
    if (ref_method=='mean') {
        ref_mean <- rowMeans(df_qc, na.rm=TRUE)
    } else if (ref_method=='median') {
        ref_mean <- rowMedians(df_qc,na.rm=TRUE)
    } else {
        stop(paste0('Unknown ref_method for calculate_ref_mean: "',ref_method,
            '". Valid options are ',
            '"mean" or "median".'))
    }
    names(ref_mean)=rownames(df_qc)
    return(ref_mean)
}

#' Probabilistic quotient normalisation (PQN)
#' 
#' For every feature the mean response is calculated across all QC samples. A 
#' reference vector is then generated. The median between the reference vector
#' and every sample is computed obtaining a vector of coefficients related to
#' each sample. Each sample is then divided by the median value of the vector
#' of coefficients; this median value is different for each sample. This 
#' method was adapted by Dieterle et al. (2006) (see references). Its purpose 
#' is to take into account the concentration changes of some metabolite 
#' features that affect limited regions of the data.
#'
#' @references Dieterle F. et al., Anal. Chem., 78(13), 2006. 
#' http://dx.doi.org/10.1021/ac051632c
#'
#' @inheritParams filter_peaks_by_blank 
#' @param ref_mean \code{numeric()} or \code{NULL}, Vector of reference mean
#' values to use instead of calculating from QC sample group. If set to 
#' \code{NULL}, QC sample data will be used.
#' @param qc_frac \code{numeric()} A value between 0 and 1 to indicate the 
#' minimum proportion of QC samples a feature must be present in for it to be included 
#' when computing the reference. Default \code{qc_frac = 0}.
#' @param sample_frac \code{numeric()} A value between 0 and 1 to indicate the 
#' minimum proportion of samples a feature must be present in for it to be considered 
#' when computing the normalisation coefficients. Default \code{sample_frac = 0}.
#' @param ref_method \code{character()} Method used to compute the reference from
#' the QC samples. Default \code{ref_method = 'mean'}. Allowed
#' values are "mean" or "median".
#' @return Object of class \code{SummarizedExperiment}. If input data are 
#' matrix-like (e.g. an ordinary matrix, a data frame) object, the same R data 
#' structure as the input will be returned with all values of the data type.
#' 
#' \code{numeric()}.
#' 
#' @examples 
#' df <- MTBLS79[ , MTBLS79$Batch==1]
#' pqn_normalisation(df=df,
#'     classes=df$Class, qc_label='QC')
#' 
#' @export

pqn_normalisation <- function(df, classes, qc_label, ref_mean=NULL, qc_frac=0,
    sample_frac=0,ref_method='mean') {
    
    # check df for issues
    df <- check_input_data(df=df, classes=classes)
    
    # add some rownames if not present
    rm_rownames=FALSE
    if (is.null(rownames(assay(df)))) {
        rownames(df)=as.character(1:nrow(df))
        rm_rownames=TRUE
    }
    
    # if no reference then create one
    if (is.null(ref_mean)){
        if (qc_label == "all") {
            # use all samples
            ref <- df
            ref_class = classes
        } else {
            # use only samples with the provided QC label
            ref <- df[, classes == qc_label]
            ref_class = classes[classes == qc_label]
        }
        ## apply missing value filter to reference
        # only keep features present in qc_frac*100% samples
        ref = filter_peaks_by_fraction(
            df=ref,
            min_frac = qc_frac,
            classes = ref_class,
            method='across',
            qc_label=qc_label,
            remove_peaks = TRUE
        )
        # check we have some features left
        if (nrow(assay(ref))<1) {
            stop(paste0('QC filtering was too strict and none of the selected ',
            'reference features survived the filtering. Try reducing the value',
            ' of qc_frac.')
            )
        }
        # record the number of reference samples used
        n_ref = nrow(assay(ref))
        
        # average the reference samples
        ref_mean <- calculate_ref_mean(df_qc=assay(ref),ref_method)
    } else {
        ref = df
        n_ref = NA
    }
    
    # filter the samples before calculating coefficient
    df_filt = filter_peaks_by_fraction(
        df=df,
        min_frac = sample_frac,
        classes = classes,
        method='across',
        qc_label=qc_label,
        remove_peaks = TRUE
    )
    
    # check we have some features left
    if (nrow(assay(df_filt))<1) {
        stop(paste0('Sample filtering was too strict and none of the',
        ' features survived the filtering. Try reducing the ',
        'value of sample_frac.')
        )
    }

    ## only keep features that survive both filters
    U=intersect(rownames(assay(df_filt)),rownames(assay(ref)))
    ref_mean=ref_mean[U]
    df_filt=df_filt[U,]
    
    # check we have some features left
    if (nrow(assay(df_filt))<1) {
        stop(paste0('No features remain after filtering that are common to ',
        'both the reference and the sample matrix. Try again with lower ',
        'values for QC_frac and/or sample_frac.')
        )
    }
    # record number of features
    n_samp=nrow(df_filt)
    
    ## calculate coefficients
    # divide each sample by reference
    coef = apply(assay(df_filt),2,function(x) {
            return(x / ref_mean)
        }
    )
    # count number of coefficients per sample
    coef_count = apply(coef,2,function(x){
        return(sum(!is.na(x)))
    })
    # median coefficient for each sample
    coef_med = matrixStats::colMedians(coef,na.rm=TRUE)
    # index of coefficient used for normalisation
    # (lowest index of, in event of ties)
    coef_idx = apply(coef,2,function(x) {
        return(which.min(abs(x - median(x,na.rm=TRUE)))[1])
    })
    # name of featured used for scaling
    coef_idx_name=rownames(df_filt)[coef_idx] 
    # convert to matrix
    coef_med=matrix(coef_med,nrow=nrow(df),ncol=length(coef_med),byrow = TRUE)
    
    if (any(is.na(coef_med))) {
        warning(paste0('A normlisation coefficient could not be computed for ',
        'one or more samples.'))
    }
    
    # apply normalisation
    assay(df) <- assay(df) / coef_med 
    col_data <- DataFrame(pqn_coef=coef_med[1,])
    colData(df) <- cbind(colData(df), col_data)
    meta_data <- metadata(df)
    meta_data$processing_history$pqn_normalisation <- c(
        return_function_args(),
        list('reference_feature_count'=n_ref,
             'sample_feature_count'=n_samp,
             'coef_feature_count'=coef_count,
             'coef_idx'=coef_idx,
             'coef_idx_name'=coef_idx_name,
             'computed_ref'=ref_mean
        )
    )
    metadata(df) <- meta_data
    df <- return_original_data_structure(df)
    if (!is(df, "SummarizedExperiment")){
        attributes(df)$flags <- as.matrix(col_data)
    }
    
    # remove the temporary rownames if we added them earlier
    if (rm_rownames) {
        rownames(df)=NULL
    }
    
    return(df)
}

context("test-normalise_to_total_sum")

test_that("normalise to sum returns correct output", {
  out <- normalise_to_sum (testData$data)
  attributes(out)$processing_history <- NULL
  expect_equal(out, testData$normalise_to_sum)
})

test_that("normalise to sum returns correct output when matrix needs to be transposed", {
  expect_warning (out <- normalise_to_sum(t(testData$data)))
  attributes(out)$processing_history <- NULL
  expect_equal(out, testData$normalise_to_sum)
})

test_that("normalise to sum returns correct output when there are less features than samples", {
  out <- normalise_to_sum(testData$data[1:8,], check_df=FALSE)
  expect_true(all(round(apply (out, 2, sum, na.rm=TRUE), 0) == 100L))
})

context("test-pqn_normalisation")

test_that("PQN normalisation returns correct output", {
  out <- pqn_normalisation(df=testData$data, classes=testData$class, 
    qc_label="QC")
  attributes(out)$processing_history <- NULL
  expect_equal(as.vector(attributes(out)$flags),
    testData$pqn_normalisation$coef)
  attributes(out)$flags <- NULL
  expect_equal(out, testData$pqn_normalisation$df)
})

test_that("PQN normalisation returns correct output, when all samples are used to calculate correction factor", {
  out <- pqn_normalisation(df=testData$data, classes=testData$class, 
    qc_label="all")
  attributes(out)$processing_history <- NULL
  expect_equal(as.vector(attributes(out)$flags),
               testData$pqn_normalisation_all$coef)
  attributes(out)$flags <- NULL
  expect_equal(out, testData$pqn_normalisation_all$df)
})

test_that("PQN normalisation returns correct output when matrix needs to be transposed", {
  out <- pqn_normalisation(df=t(testData$data), classes=testData$class, 
    qc_label="QC")
  attributes(out)$processing_history <- NULL
  expect_equal(as.vector(attributes(out)$flags),
               testData$pqn_normalisation$coef)
  attributes(out)$flags <- NULL
  expect_equal(out, testData$pqn_normalisation$df)
})

test_that("PQN normalisation doesn't crash if only one feature is selected for normalisation", {
  out <- matrix(nrow=3, ncol=9)
  out <- rbind(out, testData$data[1, ])
  expect_warning(out <- pqn_normalisation(df=out, classes=testData$class, qc_label="QC"))
  expect_true(nrow(out) == 4)
})


test_that("PQN computation of reference works as expected for mean and median",{
  out <- pqn_normalisation(df=testData$data, classes=testData$class, 
    qc_label="QC",ref_method = 'mean')
  mean_ref=attributes(out)$processing_history$pqn_normalisation$computed_ref
  expect_equal(mean_ref[[1]],106308.238,tolerance = 0.0005)
  
  out <- pqn_normalisation(df=testData$data, classes=testData$class, 
    qc_label="QC",ref_method = 'median')
  median_ref=attributes(out)$processing_history$pqn_normalisation$computed_ref
  expect_equal(median_ref[[1]],88597.333,tolerance = 0.0005)
})


test_that("PQN check reference when appling additional filtering",{
  out <- pqn_normalisation(df=testData$data, classes=testData$class, 
    qc_label="QC",qc_frac = 1,sample_frac=1,ref_method = 'mean')
  mean_ref=attributes(out)$processing_history$pqn_normalisation$computed_ref
  expect_equal(mean_ref[[1]],19855.36,tolerance = 0.0005)
  
  out <- pqn_normalisation(df=testData$data, classes=testData$class, 
    qc_label="QC",qc_frac = 1,sample_frac=1,ref_method = 'median')
  median_ref=attributes(out)$processing_history$pqn_normalisation$computed_ref
  expect_equal(median_ref[[1]],10773.31,tolerance = 0.0005)
})

#' @importFrom matrixStats rowMedians
NULL

#' Quality Control-Robust Spline Correction (QC-RSC)
#' 
#' Implementation of Quality QC-RSC algorithm for signal drift and batch 
#' effect correction within/across a multi-batch direct infusion mass
#' spectrometry (DIMS) and liquid chromatography mass spectrometry (LCMS)
#' datasets.
#' This version supports missing values, but requires at least 4 data point 
#' for quality control (QC) samples measured within each analytical batch.
#' The smoothing parameter (spar) can be optimised using leave-one-out cross
#' validation to avoid overfitting.
#' 
#' @author Andris Jankevics \email{a.jankevics@bham.ac.uk}
#' @references Kirwan et al, Anal. Bioanal. Chem., 405 (15), 2013
#'\url{https://dx.doi.org/10.1007/s00216-013-6856-7}
#' 
#' @inheritParams sbc_plot
#' @param order \code{numeric()}, A numeric vector indicating the order in 
#' which samples were measured.
#' @param spar \code{numeric(1)}, Spline smoothing parameter. Should be in the 
#' range 0 to 1. If set to 0 it will be estimated using leave-one-out 
#' cross-validation.
#' @param log \code{logical(1)}, to perform the signal correction fit on the
#' log scaled data. Default is TRUE.
#' @param minQC \code{integer(1)}, Minimum number of measured quality control
#' (QC) samples required for signal correction within feature per batch. For 
#' features where signal was measured in less QC samples than threshold signal
#' correction won't be applied.
#' @param spar_lim A 2 element numeric vector containing the min and max
#'values of spar when searching for an optimum. Default \code{spar_lim = c(-1.5,1.5)}
#' @return Object of class \code{SummarizedExperiment}. If input data are a 
#' matrix-like (e.g. an ordinary matrix, a data frame) object, function returns 
#' the same R data structure as input with all value of data type 
#' \code{numeric()}.
#'
#' @examples 
#' classes <- MTBLS79$Class
#' batch <- MTBLS79$Batch
#' order <- c(1:ncol(MTBLS79))
#' 
#' out <- QCRSC(df = MTBLS79[1:10, ], order = order, batch = MTBLS79$Batch,
#' classes = MTBLS79$Class, spar = 0, minQC = 4)
#' 
#' @export

QCRSC <- function(df, order, batch, classes, spar = 0, log = TRUE,
    minQC = 5, qc_label="QC", spar_lim = c(-1.5,1.5)) {
    
    df <- check_input_data(df=df, classes=classes)
    
    if (length(which(classes == qc_label)) <= 0) {
        message("QC samples are not defined! Please see help page for
            parameter qc_label.")
        stop()
    }
    
    message("The number of NA and <= 0 values in peaksData before QC-RSC: ",
        sum(is.na(assay(df)) | assay(df) <= 0))
    
    qcData <- df[, classes == qc_label]
    qc_batch <- batch[classes == qc_label]
    qc_order <- order[classes == qc_label]
    QC_fit <- lapply(seq_len(nrow(df)), sbcWrapper, qcData = assay(qcData), 
        order = order, qcBatch = qc_batch, qcOrder = qc_order, 
        log = log, spar = spar, batch = batch, minQC = minQC,
        spar_lim = spar_lim)
    QC_fit <- do.call(rbind, QC_fit)
    
    meta_data <- metadata(df)
    meta_data$processing_history$QCRSC <- return_function_args()
    meta_data$processing_history$QCRSC$QC_fit=QC_fit
    
    # Median value for each feature, and divide it by predicted value
    mpa <- matrixStats::rowMedians(assay(df), na.rm=TRUE)
    QC_fit <- QC_fit/mpa
    
    # Divide measured value by correction factor
    assay(df) <- assay(df)/QC_fit
    assay(df)[assay(df) <= 0] <- NA
    
    
    metadata(df) <- meta_data
    df <- return_original_data_structure(df)
    
    return (df)
}

context ("Test QCRSC function")

classes <- MTBLS79$Class
batch <- MTBLS79$Batch
order <- c(1:ncol(MTBLS79))
features_to_select <- c("70.03364", "70.03375", "70.03413", "70.03427", 
  "73.0584", "73.05882", "73.53802", "73.53822", "73.53838", "75.48804",
  "75.48817", "75.48845", "76.8677", "76.86788", "76.86806", "78.02111", 
  "97.00463", "97.005", "98.99486", "98.99512")
data <- assay(MTBLS79[features_to_select, ])

test_that ("QC-RSC returns expected output", {
  out <- QCRSC(df = data, order = order, batch = batch, classes = classes,
                  spar = 0, minQC = 4)
  attributes(out)$processing_history <- NULL
  expect_equal (out, testData$QCRSC, tolerance = 10^-5)
})

test_that ("QC-RSC returns error if QC sample label is wrong", {
  classes[classes == "QC"] <- "Quality"
  expect_error(QCRSC(df = data, order = order, batch = batch, classes = classes,
               spar = 0, minQC = 4))
})

#' @importFrom stats median sd var 
#' @importFrom S4Vectors DataFrame
#' @import SummarizedExperiment
#' @importFrom matrixStats rowMedians
#' @importFrom matrixStats rowCounts
#' @importFrom matrixStats rowSds

NULL

#' Filter features by blank samples
#'
#' Metabolomics datasets often contain many features of non-biological 
#' origin e.g. those associated with extraction and analysis solvents. 
#' This tool facilitates the removal of such features from the data matrix, 
#' as defined using an appropriate blank sample.
#' 
#' If parameter \code{qc_label} is not \code{NULL}, QC samples which will be 
#' used to calculate the median signal intensity.
#'
#' @param df A matrix-like (e.g. an ordinary matrix, a data frame) or 
#' \link[SummarizedExperiment]{RangedSummarizedExperiment-class} object with 
#' all values of class \code{numeric()} or \code{integer()} of peak 
#' intensities, areas or other quantitative characteristic.
#' @param fold_change \code{numeric(1)}, fold_change minimum fold change 
#' between analytical and blank samples.
#' @param classes \code{character()}, vector of class labels. Must be the same 
#' length as the number of sample in the input peak table. If input is 
#' \code{SummarizedExperiment} object, use 
#' \code{SummarizedExperiment_object$meta_data_column_name}.
#' @param blank_label \code{character(1)}, class label used to identify blank
#' samples.
#' @param qc_label \code{character(1)} or \code{NULL}, class label used to
#' identify QC samples.
#' @param remove_samples \code{logical(1)}, remove blank samples from peak 
#' matrix or not.
#' @param remove_peaks \code{logical(1)}, remove filtered features from peak 
#' matrix or not.
#' @param fraction_in_blank \code{numeric(1)}, value between 0 to 1 to specify 
#' fraction in how many blanks peaks should be present.
#' 
#' @return Object of class \code{SummarizedExperiment}. If input data are a 
#' matrix-like (e.g. an ordinary matrix, a data frame) object, function returns 
#' \code{numeric()} matrix-like object of filtered data set. Function 
#' \code{flags} are added to the object \code{attributes} and is a 
#' \link[S4Vectors]{DataFrame-class} with five columns. The same 
#' \code{DataFrame} object containing flags is added to \code{rowData()} 
#' element of \code{SummarizedExperiment} object as well.\cr
#' \cr
#' Columns in \code{rowData()} or \code{flags} element contain: \cr
#' \code{median_non_blanks} median intensities of features of non-blank
#' samples; \cr
#' \code{median_blanks} median intensities of features of blank samples; \cr
#' \code{fold_change} fold change between analytical and blank samples; \cr
#' \code{blank_flags} \code{integer()}, if 0 feature is flagged to be 
#' removed; \cr
#' \code{blank_fraction_flags} \code{numeric()}, fraction in how many blank
#' samples peaks is present. \cr
#' 
#' @examples
#' df <- MTBLS79[ ,MTBLS79$Batch == 1]
#' df$Class[1:2] <- "Blank"
#' out <- filter_peaks_by_blank(df=df, fold_change=1.2, 
#'    classes=df$Class, blank_label="Blank", qc_label=NULL, 
#'    remove_samples=FALSE, remove_peaks=TRUE, fraction_in_blank=0)
#' 
#' @export

filter_peaks_by_blank <- function(df, fold_change, classes, blank_label, 
        qc_label=NULL, remove_samples=TRUE, remove_peaks=TRUE, 
        fraction_in_blank=0) {
    input_df_class <- class(df)[1]
    df <- check_input_data(df=df, classes=classes)
    M_blanks <- df[, classes == blank_label]
    if (!is.null(qc_label)) {
        M_non_blanks <- df[, classes == qc_label,
            drop=FALSE]
    } else {
        M_non_blanks <- df[, classes != blank_label,
            drop=FALSE]
    }
    median_intensity_blanks <- 
        matrixStats::rowMedians(assay(M_blanks), na.rm=TRUE)
    median_intensity_non_blanks <- 
        matrixStats::rowMedians(assay(M_non_blanks), na.rm=TRUE)
    fold_change_exp <- median_intensity_non_blanks / median_intensity_blanks
    blank_fraction <- 1 - (rowCounts(assay(M_blanks), value=NA) / 
        ncol(assay(M_blanks)))
    idxs <- fold_change_exp >= fold_change & blank_fraction >= fraction_in_blank
    idxs[which(is.na(median_intensity_non_blanks))] <- FALSE
    idxs[which(is.na(median_intensity_blanks))] <- TRUE
    if (is.null(qc_label)) {
        row_data <- DataFrame(median_non_blanks=median_intensity_non_blanks)
    } else {
        row_data <- DataFrame(median_QCs=median_intensity_non_blanks)
    }
    row_data$median_blanks <- median_intensity_blanks
    row_data$fold_change <- fold_change_exp
    row_data$blank_flags <- as.numeric(idxs)
    row_data$blank_fraction_flags <- as.numeric(blank_fraction)
    rowData(df) <- cbind(rowData(df), row_data)
    if (remove_peaks){
        df <- df[idxs, ]
    }
    if (remove_samples) {
        df <- df[, classes != blank_label]
    }
    meta_data <- metadata(df)
    meta_data$processing_history$filter_peaks_by_blank <- return_function_args()
    metadata(df) <- meta_data
    df <- return_original_data_structure(df)
    return(df)
}

#' Filter features by fraction of missing values
#' 
#' Metabolomics datasets often contain 'features' with irreproducible peak 
#' intensity values, or with large numbers of missing values. This tool 
#' facilitates the remove of such features from a data matrix, based upon the 
#' relative proportion (minimum fraction) of samples containing non-missing 
#' values.
#'
#' @inheritParams filter_peaks_by_blank
#' @param min_frac \code{numeric(1)}, value between 0 and 1, a threshold of 
#' fraction of detection.
#' @param method \code{character(1)}, method to use. \code{QC} - within 
#' QC samples, \code{within} - within each sample class or \code{across} - 
#' across all samples.
#'
#' @return Object of class \code{SummarizedExperiment}. If input data are a 
#' matrix-like (e.g. an ordinary matrix, a data frame) object, function returns 
#' \code{numeric()} matrix-like object of filtered data set. Function 
#' \code{flags} are added to the object \code{attributes} and is a 
#' \link[S4Vectors]{DataFrame-class} with five columns. The same 
#' \code{DataFrame} object containing flags is added to \code{rowData()} 
#' element of \code{SummarizedExperiment} object as well.\cr
#' \cr
#' 
#' Columns in \code{rowData()} or \code{flags} element contain \code{fractions} 
#' of missing values per feature within QC samples (mehtod \code{QC}), 
#' across (method \code{across}) or within (mehtod \code{within}) each sample 
#' group.
#' 
#' @examples 
#' df <- MTBLS79[ ,MTBLS79$Batch == 1]
#' out <- filter_peaks_by_fraction(df=df, min_frac=1, 
#'     classes=df$Class, method='QC', qc_label='QC')
#'     
#' out <- filter_peaks_by_fraction(df=df, min_frac=1, 
#'     classes=df$Class, method='across', qc_label='QC')
#' 
#' out <- filter_peaks_by_fraction(df=df, min_frac=1, 
#'     classes=df$Class, method='within', qc_label='QC')
#' 
#' @export

filter_peaks_by_fraction <- function(df, min_frac, classes=NULL,
    method="QC", qc_label="QC", remove_peaks=TRUE) {
    df <- check_input_data(df=df, classes=classes)
    if (method == "within" || method == "QC") {
        fracs <- data.frame(matrix(ncol=dim(df)[1], nrow=0))
        fracs_flags <- data.frame(matrix(ncol=dim(df)[1], nrow=0))
        rn_fracs <- c()
        rn_fracs_flags <- c()
        if (method == "within") {
            cls <- unique(classes)
        } else {
            cls <- c(qc_label)
        }
        for (cl in cls) {
            idxs <- cl == classes
            subset_cl <- df[, idxs, drop=FALSE]
            frac <- rowSums(!is.na(assay(subset_cl)))/ncol(subset_cl)
            fracs <- rbind(fracs, round(frac, 2))
            fracs_flags <- rbind(fracs_flags, as.numeric(frac >= min_frac))
            rn_fracs <- c(rn_fracs, paste("fraction_", cl, sep=""))
            rn_fracs_flags <- c(rn_fracs_flags, paste("fraction_flag_", cl,
                sep=""))
        }
        colnames(fracs) <- rownames(df)
        colnames(fracs_flags) <- rownames(df)
        rownames(fracs) <- rn_fracs
        rownames(fracs_flags) <- rn_fracs_flags
        flags <- t(rbind(fracs, fracs_flags))
        idxs <- colSums(fracs_flags) > 0
    } else if (method == "across") {
        frac <- rowSums(!is.na(assay(df)))/ncol(df)
        idxs <- frac >= min_frac
        flags <- cbind(fraction=round(frac, 2),
            fraction_flags=as.numeric(idxs))
    }
    rowData(df) <- cbind(rowData(df), DataFrame(flags))
    if (remove_peaks){
        df <- df[idxs, ]
    }
    meta_data <- metadata(df)
    function_arguments <- return_function_args()
    filter_name <- paste("filter_peaks_by_fraction", 
        function_arguments$method, sep="_")
    meta_data$processing_history[[filter_name]] <- 
        function_arguments
    metadata(df) <- meta_data
    df <- return_original_data_structure(df)
    return(df)
}

#' Remove features from peak intensity matrix
#'
#' Filter to remove features.
#'
#' @inheritParams filter_peaks_by_blank
#' @param rem_index \code{logical()}, vector containing \code{TRUE} vales for 
#' features to remove. Should be the same length as number of features in input 
#' data.
#' 
#' @return Object of class \code{SummarizedExperiment}. If input data are a 
#' matrix-like (e.g. an ordinary matrix, a data frame) object, function returns 
#' the same R data structure as input with all value of data type 
#' \code{numeric()}.
#' 
#' @examples
#' df <- MTBLS79[ ,MTBLS79$Batch == 1]
#' rem_index <- vector(mode="logical", 
#'     length=nrow(SummarizedExperiment::assay(df)))
#' rem_index[c(1, 20, 456, 789)] <- TRUE
#' out <- remove_peaks(df=df, rem_index=rem_index)
#' 
#' @export

remove_peaks <- function(df, rem_index) {
    df <- check_input_data(df=df)
    if (is.logical(rem_index)) {
        df <- df[!rem_index, ]
        meta_data <- metadata(df)
        meta_data$processing_history$remove_peaks <- 
            return_function_args()
        metadata(df) <- meta_data
        df <- return_original_data_structure(df)
        return(df)
    } else {
        stop("Vector of indexes to remove from peak matrix should be logical
    vector of TRUE/FALSE vaues.")
    }
}

#' Filter features by RSD\% of QC samples
#'
#' Metabolomics datasets often contain 'features' with irreproducible peak 
#' intensity values, or with large numbers of missing values. This tool 
#' facilitates the remove of such features from a data matrix, based upon 
#' relative standard deviation of intensity values for a given feature within 
#' specified QC samples.
#'
#' @inheritParams filter_peaks_by_blank
#' @param max_rsd \code{numeric()}, threshold of QC RSD\% value
#' 
#' @return Object of class \code{SummarizedExperiment}. If input data are a 
#' matrix-like (e.g. an ordinary matrix, a data frame) object, function returns 
#' \code{numeric()} matrix-like object of filtered data set. Function 
#' \code{flags} are added to the object \code{attributes} and is a 
#' \link[S4Vectors]{DataFrame-class} with five columns. The same 
#' \code{DataFrame-class} object containing flags is added to \code{rowData()} 
#' element of \code{SummarizedExperiment} object as well.\cr
#' \cr
#' Columns in \code{rowData()} or \code{flags} element contain: \cr
#' \code{rsd_QC} \code{numeric()}, RSD\% value of QC samples per feature; \cr
#' \code{rsd_flags} \code{integer()},if 0 feature is flagged to be removed. \cr
#' 
#' @examples 
#' 
#' df <- MTBLS79[ ,MTBLS79$Batch == 1]
#' out <- filter_peaks_by_rsd(df=df, max_rsd=20,
#'     classes=df$Class, qc_label='QC')
#' 
#' @export

filter_peaks_by_rsd <- function(df, max_rsd, classes, qc_label,
    remove_peaks=TRUE) {
    df <- check_input_data(df=df, classes=classes)
    df_qcs <- df[, classes == qc_label, drop=FALSE]
    rsd_values <- rowSds(assay(df_qcs), na.rm=TRUE) / 
        rowMeans(assay(df_qcs), na.rm=TRUE) * 100
    idxs <- rsd_values < max_rsd
    idxs[is.na(idxs)] <- FALSE
    row_data <- DataFrame(rsd_QC=round(rsd_values, 2),
        rsd_flags=as.numeric(idxs))
    rowData(df) <- cbind(rowData(df), row_data)
    if (remove_peaks){
        df <- df[idxs, ]
    }
    meta_data <- metadata(df)
    meta_data$processing_history$filter_peaks_by_rsd <- return_function_args()
    metadata(df) <- meta_data
    df <- return_original_data_structure(df)
    return(df)
}

#' Filter samples by missing values
#'
#' Missing values in mass spectrometry metabolomic datasets occur widely and 
#' can originate from a number of sources, including for both technical and 
#' biological reasons. In order for robust conclusions to be drawn from 
#' down-stream statistical testing procedures, the issue of missing values 
#' must first be addressed. This tool facilitates the removal of samples 
#' containing a user-defined maximum percentage of missing values.
#'
#' @inheritParams filter_peaks_by_blank
#' @param max_perc_mv \code{numeric(1)}, Value between 0 and 1 of threshold 
#' of missing value percentage in sample.
#' 
#' @return Object of class \code{SummarizedExperiment}. If input data are a 
#' matrix-like (e.g. an ordinary matrix, a data frame) object, function returns 
#' \code{numeric()} matrix-like object of filtered data set. Function 
#' \code{flags} are added to the object \code{attributes} and is a 
#' \link[S4Vectors]{DataFrame-class} with five columns. The same 
#' \code{DataFrame} object containing flags is added to \code{rowData()} 
#' element of \code{SummarizedExperiment} object as well. If element 
#' \code{colData()} already exists flags are appended to existing values.\cr
#' \cr
#' 
#' Columns in \code{colData()} or \code{flags} element contain: \cr
#' \code{perc_mv} \code{numeric()}, fraction of missing values per sample; \cr
#' \code{flags} \code{integer()},if 0 feature is flagged to be removed. \cr
#' 
#' @examples 
#' 
#' df <- MTBLS79
#' out <- filter_samples_by_mv (df=df, max_perc_mv=0.8)
#' 
#' @export

filter_samples_by_mv <- function(df, max_perc_mv, classes=NULL, 
    remove_samples=TRUE) {
    df <- check_input_data(df=df, classes=classes)
    perc_mv <- colSums(is.na(assay(df)))/nrow(df)
    idxs <- perc_mv <= max_perc_mv
    col_data <- DataFrame(filter_samples_by_mv_perc=round(perc_mv, 2),
        filter_samples_by_mv_flags=as.numeric(idxs))
    colData(df) <- cbind(colData(df), col_data)
    if (remove_samples){
        df <- df[ ,idxs]
    }
    meta_data <- metadata(df)
    meta_data$processing_history$filter_samples_by_mv <- return_function_args()
    metadata(df) <- meta_data
    df <- return_original_data_structure(df)
    if (!is(df, "SummarizedExperiment")){
        attributes(df)$flags <- as.matrix(col_data)
    }
    return(df)
}

context ("test-filter_peaks_by_blank")

test_that ("Test that filter_peaks_by_blank returns expected output", {
  out <- filter_peaks_by_blank(df=testData$data, fold_change=1.2, 
    classes=testData$class, blank_label="Blank", qc_label=NULL, 
    remove_samples=FALSE, remove_peaks=TRUE, fraction_in_blank=0)
  attributes(out)$processing_history <- NULL
  expect_equal(as.matrix(attributes(out)$flags),
    as.matrix(testData$filter_peaks_by_blank$flags))
  attributes(out)$flags <- NULL
  expect_equal(as.matrix(out), testData$filter_peaks_by_blank$df)
  
})

test_that ("Test that filter_peaks_by_blank returns expected output with QC samples specified", {
  out <- filter_peaks_by_blank(df=testData$data, fold_change=1.2, 
    classes=testData$class, blank_label="Blank", qc_label="QC",
    remove_samples=FALSE, remove_peaks=TRUE, fraction_in_blank=0)
  expect_equal(processing_history(out)[[1]][[1]], 1.2)
    attributes(out)$processing_history <- NULL
    attributes(out)$flags <- NULL
  expect_equal(out, testData$filter_peaks_by_blank_qc$df)
})

test_that ("Test that filter_peaks_by_blank removes blank samples, but keeps features", {
  out <- filter_peaks_by_blank(df=testData$data, fold_change=1.2, 
    classes=testData$class, blank_label="Blank", qc_label=NULL, 
    remove_samples=TRUE, remove_peaks=FALSE,fraction_in_blank=0)
  expect_equal(as.matrix(attributes(out)$flags), 
    as.matrix(testData$filter_peaks_by_blank_remove_blanks$flags))
    attributes(out)$flags <- NULL
  expect_equal(out, testData$filter_peaks_by_blank_remove_blanks$df)
  
})

context ("test-filter_peaks_by_fraction")

test_that ("Test that filter_peaks_by_fraction returns expected output, method=QC", {
  out <- filter_peaks_by_fraction(df=testData$data, min_frac=1, 
  classes=testData$class, method="QC", qc_label="QC")
  attributes(out)$processing_history <- NULL
  expect_equal(attributes(out)$flags, testData$filter_peaks_by_fraction$flags)
  attributes(out)$flags <- NULL
  expect_equal(out, testData$filter_peaks_by_fraction$df)
  
})

test_that ("Test that filter_peaks_by_fraction returns expected output, method=within", {
  out <- filter_peaks_by_fraction(df=testData$data, min_frac=1, 
  classes=testData$class, method="within", qc_label="QC")
  attributes(out)$processing_history <- NULL
  expect_equal(attributes(out)$flags, testData$filter_peaks_by_fraction_within$flags)
  attributes(out)$flags <- NULL
  expect_equal(out, testData$filter_peaks_by_fraction_within$df)
  
})

test_that ("Test that filter_peaks_by_fraction returns expected output, method=across", {
  out <- filter_peaks_by_fraction(df=t(testData$data), min_frac=0.6, 
    classes=testData$class, method="across", qc_label="QC")
  attributes(out)$processing_history <- NULL
  expect_equal(attributes(out)$flags, testData$filter_peaks_by_fraction_across$flags)
  attributes(out)$flags <- NULL
  expect_equal(out, testData$filter_peaks_by_fraction_across$df)
})

context ("test-filter_peaks_by_rsd")

test_that("Test that filter_peaks_by_rsd returns expected output", {
  out <- filter_peaks_by_rsd(df=t(testData$data), max_rsd=20, 
    classes=testData$class, qc_label="QC")
  attributes(out)$processing_history <- NULL
  expect_equal (attributes(out)$flags, testData$filter_peaks_by_rsd$flags)
  attributes(out)$flags <- NULL
  expect_equal (out, testData$filter_peaks_by_rsd$df)
  
})

context ("test-filter_samples_by_mv")

test_that("Test that filter_samples_by_mv returns expected output", {
  
  out <- testData$data
  out[c(1:25),c(2,9)] <- NA
  expect_warning(out <- filter_samples_by_mv (df=t(out), max_perc_mv=0.8))
  attributes(out)$processing_history <- NULL
  flags <- attributes(out)$flags
  colnames(flags) <- c("perc_mv", "flags")
  expect_equal (flags, testData$filter_samples_by_mv$flags)
  attributes(out)$flags <- NULL
  expect_equal (out, testData$filter_samples_by_mv$df)
  
})

test_that("Remove peaks filter returns expecteed output", {
  
  rem_index <-testData$remove_peaks$rem_index
  out <- remove_peaks(df=testData$data, rem_index=rem_index)
  attributes(out)$processing_history <- NULL
  expect_equal (out, testData$remove_peaks$rem_output)
  
})

test_that("Remove peaks filter fails if input is not logical vector", {
  
  rem_index <- as.factor(testData$remove_peaks$rem_index)
  expect_error(out <- remove_peaks(df=testData$data, rem_index=rem_index))
  
})

test_that("Blank filter fails if one column is a factor", {
  data <- as.data.frame(testData$data)
  data$QC_2 <- as.factor(data$QC_2)
  testthat::expect_error(filter_peaks_by_blank(df=data, 
    fold_change=1.2, classes=testData$class, 
    blank_label="Blank", qc_label=NULL, remove=FALSE, 
    fraction_in_blank=0))
})

test_that("Blank filter fails if all data are data type 'character'", {
  data <- as.character(testData$data)
  testthat::expect_error(filter_peaks_by_blank(df=data, 
    fold_change=1.2, classes=testData$class, blank_label="Blank", 
    qc_label=NULL, remove=FALSE, fraction_in_blank=0))
})

---
title: "Signal drift and batch effect correction and mass spectral quality assessment"
author: 
- name: "Andris Jankevics"
  affiliation: Phenome Centre Birmingham, University of Birmingham, UK
  email: a.jankevics@bham.ac.uk
- name: Gavin Rhys Lloyd
  affiliation: Phenome Centre Birmingham, University of Birmingham, UK
  email: g.r.lloyd@bham.ac.uk
- name: Ralf Johannes Maria Weber
  affiliation: Phenome Centre Birmingham, University of Birmingham, UK
  email: r.j.weber@bham.ac.uk
package: pmp
date: "`r Sys.Date()`"
output: 
    BiocStyle::html_document:
        toc: true
        toc_depth: 2  
        number_sections: true  
        toc_float: true

bibliography: pmp.bib
csl: harvard-university-of-birmingham.csl

vignette: >
    %\VignetteIndexEntry{Signal drift and batch effect correction and mass spectral quality assessment}
    %\VignetteEncoding{UTF-8}
    %\VignetteEngine{knitr::rmarkdown}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
    collapse = TRUE,
    comment = "#>",
    fig.width=5,
    fig.height=5
)
```

# Introduction

This is a step by step tutorial on how to assess and/or correct signal drift and
batch effects within/across a multi-batch direct infusion mass spectrometry
(DIMS) dataset. The same approach can be used on liquid chromatography mass
spectrometry (LCMS) peak table as well.

Deeper details on how the algorithm works are detailed in \@ref(algorithm)

# Installation

You should have R version 4.0.0 or above and Rstudio installed to be able to
run this notebook.

Execute following commands from the R terminal.

```{r echo=TRUE, message=FALSE, warning=FALSE, eval=FALSE}
install.packages("gridExtra")

if (!requireNamespace("BiocManager", quietly = TRUE))
    install.packages("BiocManager")
BiocManager::install("pmp")
```

Load the required libraries into the R environment
```{r, message=FALSE, warning=FALSE}
library(S4Vectors)
library(SummarizedExperiment)
library(pmp)
library(ggplot2)
library(reshape2)
library(gridExtra)
```

# Dataset

In this tutorial we will be using  a direct infusion mass spectrometry (DIMS) 
dataset consisting of 172 samples measured across 8 batches and is included in
`pmp` package as `SummarizedExperiemnt` class object `MTBLS79`. 
More detailed description of the dataset is available from  @kirwan2014, 
[MTBLS79](https://www.ebi.ac.uk/metabolights/MTBLS79) and R man page.

```{r, eval=FALSE}
help ("MTBLS79")
```

```{r}
feature_names <- c("70.03364", "133.07379", "146.16519", "163.04515", 
    "174.89483", "200.03196", "207.07818", "221.05062", "240.02445",
    "251.03658", "266.01793", "304.99115", "321.07923", "338.98131", 
    "376.03962", "393.35878", "409.05716", "430.24353", "451.01086",
    "465.14937")
summary(t(SummarizedExperiment::assay(MTBLS79[feature_names, ])))
```

```{r}
#number of samples:
ncol(MTBLS79)
```

```{r}
#Batches:
unique(MTBLS79$Batch)
```

```{r}
#Sample classes:
unique(MTBLS79$Class)
```

# Exploratory data analysis

A more detailed overview and guidelines on strategies for quality control of
mass spectrometry assays is detailed in recent work by @broadhurst2018.

To evaluate if the data needs correction, it is common practice to examine the
relative standard deviation (RSD) of the quality control (QC) samples and
biological samples. RSD% is also sometimes referred to as the coefficient of
variation (CV). An RSD% for the QC samples below 20-30% is commonly used as an
acceptable level of technical variation where signal correction is not required.

The following code calculates and plots the RSD% values of the features within
the dataset.

```{r fig.height=5, fig.width=4, message=FALSE, warning=FALSE}
#  separate the LCMS data from the meta data
data(MTBLS79)
data <- SummarizedExperiment::assay(MTBLS79[feature_names, ])
class <- SummarizedExperiment::colData(MTBLS79)$Class
batch <- SummarizedExperiment::colData(MTBLS79)$Batch
order <- c(1:ncol(data))

# get index of QC samples
QChits <- which(class == "QC")

# small function to calculate RSD%
FUN <- function(x) sd(x, na.rm=TRUE) / mean(x, na.rm=TRUE) * 100

# RSD% of biological and QC samples within all 8 batches:
out <- matrix(ncol=2, nrow=nrow(data))
colnames(out) <- c("Sample","QC")
rownames(out) <- rownames(data)

# for each feature calculate RSD% for the samples and the QCs
for (i in 1:nrow(data)) {
    out[i, 1] <- FUN(data[i, -QChits]) # for samples
    out[i, 2] <- FUN(data[i, QChits]) # for QCs
}

# prepare data for plotting
plotdata <- melt(data.frame(out), variable.name="Class", value.name="RSD")
plotdata$feature <- rownames(data)

plotdata$RSD <- round(plotdata$RSD,0)
plotdata$feature <- factor(plotdata$feature, ordered=TRUE,
    levels=unique(plotdata$feature))

# plot
ggplot(data=plotdata, aes(x=Class, y=feature, fill=RSD)) + 
    geom_tile() + 
    geom_text(aes(label=RSD)) +
    scale_fill_gradient2(low="black", mid="white", high="red")
```

A violin plot is a useful way of summarising the RSD% over all samples/QCs in
the data set. Note a very high QC sample RSD% value for feature '409.05716'.
```{r}
ggplot(data=plotdata, aes(x=Class, y=RSD, fill=Class)) +
    geom_violin(draw_quantiles=c(0.25,0.5,0.75)) +
    ylab("RSD%") + 
    guides(fill=FALSE) +
    theme(panel.background=element_blank())
```

The plots indicates that most features have a QC RSD% lower than 30%, which is
a commonly accepted threshold, but for some features the QC RSD% exceeds 30%
and is more similar to the signal variation of the biological samples. We can
calculate similar statistics per batch and visualise the results with a box
plot.

```{r message=FALSE, warning=FALSE, fig.height=6}
# prepare some matrices to store the results
RSDQC <- matrix(ncol=8, nrow=nrow(data))
RSDsample <- matrix(ncol=8, nrow=nrow(data))
colnames(RSDQC) <- unique(batch)
colnames(RSDsample) <- unique(batch)

rownames(RSDQC) <- rownames(data)
rownames(RSDsample) <- rownames(data)

# for each feature
for (i in 1:nrow(data)) {
    # for each batch
    for (nb in 1:8) {
        # RSD% of QCs in this batch
        RSDQC[i, nb] <- FUN(data[i, which(class == "QC" & batch == nb)])
        # RSD% of samples in this batch
        RSDsample[i, nb] <- FUN(data[i, which(!class == "QC" & batch == nb)])
    }
}

# prepare results for plotting
plotdataQC <- melt(as.data.frame(RSDQC), variable.name="batch",
    value.name="RSD")
plotdataQC$Class <- "QC"

plotdataBio <- melt(as.data.frame(RSDsample), variable.name="batch",
    value.name="RSD")
plotdataBio$Class <- "Sample"

plotdata <- rbind(plotdataQC, plotdataBio)

plotdata$Class <- as.factor(plotdata$Class)

# plot
ggplot(data=plotdata, aes(x=Class, y=RSD, fill=Class)) + geom_boxplot() +
    facet_wrap(~ batch, ncol=3) +
    ylab("RSD%") +
    xlab("") +
    scale_x_discrete(labels=NULL) +
    theme(panel.background=element_blank(), axis.text.x=element_blank(),
        axis.ticks.x=element_blank())
```

**Summary of RSD% of QC samples**
```{r}
summary(RSDQC)
```

**Summary of RSD% of biological samples**
```{r}
summary(RSDsample)
```

From the above we can conclude that for every analytical batch RSD% tends to be
higher in the analytical samples than it is in the QC samples for all 20
measured features. A few outlier QC samples can be observed.

An alternative measure of QC and biological sample variability is the so called
D-ratio, which indicates if the technical variation within the QC samples
exceeds the biological variation within biological samples.

```{r message=FALSE, warning=FALSE}

# prepare a list of colours for plotting
manual_color = c("#386cb0", "#ef3b2c", "#7fc97f", "#fdb462", "#984ea3", 
    "#a6cee3", "#778899", "#fb9a99", "#ffff33")

# Function to calculate median absolute deviation
DRatfun <- function(samples, qcs) mad(qcs) / mad(samples)

# prepare matrix for dratio output
dratio <- matrix(ncol=8, nrow=nrow(data))
colnames(dratio) <- unique(batch)
rownames(dratio) <- rownames(data)

# calculate dratio for each feature, per batch
for (i in 1:nrow(dratio)){
    for (nb in 1:8) {
        dratio[i, nb] <- DRatfun(samples=data[i, which(!class == "QC" &
        batch == nb)], qcs=data[i, which(class == "QC" & batch == nb)])
    }
}

# prepare data for plotting
dratio <- as.data.frame(round(dratio, 2))

plotdata2 <- melt(dratio, variable.name="batch")
plotdata2$index <- rownames(data)
plotdata2$index <- factor(plotdata2$index, ordered=TRUE,
    levels=unique(plotdata2$index))

ggplot(data=plotdata2, aes(x=index, y=value, color=batch)) +
    geom_point(size=2) +
    xlab("index") + ylab("D-ratio") +
    geom_hline(yintercept=1) + theme(panel.background=element_blank()) +
    scale_color_manual(values=manual_color) +
    theme(axis.text.x=element_text(angle=90))
```

The D-ratio is a convenient measure to assess if technical variation in the QC
samples (MAD QC) is higher than the variation within the biological samples
(MAD sample). Ratio values close to, or higher than 1  indicate that technical
variation of the measured feature is higher than the biological variation and
therefore should be treated carefully during interperation of the dataset.
The colors in the figure above indicate different analytical batches. In the
example above we can see that feature '409.05716' has a D-ratio value above 1
in four batches, while for features '70.0336' and '393.35878' the D-ratio
is reproducibly low within all eight batches.

Principal components analysis (PCA) can be used to check common trends in  the
data. Let's inspect the scores of the first two principal components and
samples colored by batch and class. For PCA model data should be normalised and 
missing values should be replaced using imputation, followed by data scaling.
We will use the probabilistic quotient normalisation (PQN) method to normalise
the data, k-nearest neighbours (KNN) for missing value imputation and finally
the glog method to stabilise the variance across low and high intensity mass
spectral features.
See @guida2016 for a more detailed review on common pre-processing steps and 
methods.

```{r, fig.width=6.5, fig.height=5}
pca_data <- MTBLS79[feature_names, ]

pca_data <- pqn_normalisation(pca_data, classes=class, qc_label="QC")
pca_data <- mv_imputation(pca_data, method="KNN", k=5, rowmax=0.5,
    colmax=0.5, maxp=NULL, check_df=FALSE)
pca_data <- glog_transformation(pca_data, classes=class, qc_label="QC")

pca_data <- prcomp(t(SummarizedExperiment::assay(pca_data)), center=TRUE, 
    scale=FALSE)
exp_var_pca <- round(((pca_data$sdev^2)/sum(pca_data$sdev^2)*100)[1:2], 2)

plots <- list()

plotdata <- data.frame(PC1=pca_data$x[, 1], PC2=pca_data$x[, 2],
    batch=as.factor(batch), class=class)

plots[[1]] <- ggplot(data=plotdata, aes(x=PC1, y=PC2, col=batch)) +
    geom_point(size=2) +
    theme(panel.background=element_blank()) +
    scale_color_manual(values=manual_color) +
    ggtitle("PCA scores, before correction") +
    xlab(paste0("PC1 (", exp_var_pca[1] ," %)")) +
    ylab(paste0("PC2 (", exp_var_pca[2] ," %)"))

plots[[2]] <- ggplot(data=plotdata, aes(x=PC1, y=PC2, col=class)) +
    geom_point(size=2) +
    theme(panel.background=element_blank()) +
    scale_color_manual(values=manual_color) +
    ggtitle("PCA scores plot, before correction") +
    xlab(paste0("PC1 (", exp_var_pca[1] ," %)")) +
    ylab(paste0("PC2 (", exp_var_pca[2] ," %)"))

grid.arrange(ncol=2, plots[[1]], plots[[2]])
```

Left side plot above clearly shows that samples measured in batches 7 and 8 are
differentiating from bathes 1 to 6. On the right hand side plot the seperation
between samples classes is still visible, but also seperation between
measurement batches is clearly visible across PC2 axis.

## Correlation between signal intensity and injection order of QC samples

Alternatively, trends in measured signal related to injection order could
indicate if signal drif and/or batch effect correction is required. The plot
below illustrates the measured signal of QC samples across all 8 batches. To be
able to compare all 20 features measured at different signal ranges, the data
will be scaled to unit variance (UV).

```{r message=FALSE, warning=FALSE, fig.height=10}

# autoscale the QC data
QCdata <- data[ ,QChits]
QCdata2 <- as.data.frame(scale(t(QCdata), scale=TRUE, center=TRUE))

# prepare the data for plotting
plotdata <- melt(QCdata2, value.name="intensity")
plotdata$index <- rep(1:nrow(QCdata2), ncol(QCdata2))

plotdata$batch <- as.factor(batch[QChits])

# plot
ggplot(data=plotdata, aes(x=index, y=intensity, col=batch)) +
    geom_point(size=2) +
    facet_wrap(~ variable, ncol=4) +
    theme(panel.background=element_blank()) +
    scale_color_manual(values=manual_color)
```

This figure indicates that there is some fluctuation in the measured signal
across the eight batches, and that some features are following a similar
pattern, i.e. they are correlated. We can create a similar plot to the one
above including linear regression fit between measured data points.

```{r, warning=FALSE, fig.height=10}
ggplot(data=plotdata, aes(x=index, y=intensity, col=batch)) + 
    geom_point(size=2) +
    facet_wrap(~ variable, ncol=4) +
    geom_smooth(method="lm", se=TRUE, colour="black") +
    theme(panel.background=element_blank()) +
    scale_color_manual(values=manual_color)
```

The plot above indicates that some trends can be observed. It is possible to
calculate actual correlation values within QC samples for each measured
feature, and we will use Kendall's *tau* statistic to estimate a rank-based
measure of association.

```{r message=FALSE, warning=FALSE, fig.height=7.5}
sampleorder <- c(1:ncol(QCdata))

correlations <- matrix(ncol=2, nrow=nrow(data))
rownames(correlations) <- rownames(data)
colnames(correlations) <- c("tau","p.value")
correlations <- as.data.frame(correlations)

for (coln in 1:nrow(data)) {
    stat <- cor.test(sampleorder, QCdata[coln, ], method="kendall")
    correlations$tau[coln] <- stat$estimate
    correlations$p.value[coln] <- stat$p.value
}

correlations

```

While most of the calculated *tau* values and corresponding *p-values* indicate
that there is not a strong trend between injection order and the measured QC
sample signal, some of the values don't match the trends we can observe in
figure above. For features "133.07379" and "376.03962" calculated correlation
values are aroun 0.6 and in figure above intensity increasing tren can be
observed. For other features correlation values are relatively low, but in
figure above clear trends in signal changes in batches 7 and 8 can be osberved.

Alternatively it is possible to calculate correlation statistics per batch and
visualise the results.

```{r, warning=FALSE, message=FALSE}

correlations <- matrix(ncol=8, nrow=nrow(data))
rownames(correlations) <- rownames(data)
colnames(correlations) <- unique(batch)

QCbatch <- batch[QChits]

for (coln in 1:nrow(data)) {
    for (bch in 1:8) {
        sampleorder <- scale(c(1:length(which(QCbatch==bch))),
        center=TRUE, scale=TRUE)

        if ((length(sampleorder) - 
        length(which(is.na(QCdata[coln, which(QCbatch==bch)])))) >= 3){
            correlations[coln, bch] <- cor.test(sampleorder,
            QCdata[coln, which(QCbatch==bch)], method="kendall")$estimate
        }
    }
}

round(correlations, 2)
```

```{r, fig.wide=TRUE}
plotdata <- as.data.frame(correlations)
plotdata$feature <- rownames(plotdata)

plotdata <- melt(plotdata, variable.name="batch")
plotdata$feature <- factor(plotdata$feature, ordered=TRUE,
    levels = unique(plotdata$feature))

ggplot(data=plotdata, aes(x=batch, y=feature, fill=value)) + 
    geom_tile() + scale_fill_gradient2()

```

Figure above indicates that there are significant acquisition order related
trends within some batches. Fore example, bathes 4 and 6 for features
'451.01081' and '409.05716'.

## Using regression to estimate signal trends and variability across QC sample

It is possible to apply univariate regression to the QC sample injection
order and signal intensity, to estimate correlation and spread (R2) of the
measured data points.

```{r}
sampleorder <- c(1:ncol(QCdata))

regressionout <- matrix(ncol=3, nrow=nrow(data))
rownames(regressionout) <- rownames(data)
colnames(regressionout) <- c("R2.adj","coefficient","p.value")
regressionout <- as.data.frame(regressionout)

for (coln in 1:nrow(data)) {
    tempdat <- data.frame(x=sampleorder, y=QCdata[coln, ])
    stat <- lm(x ~ y, data=tempdat)
    stat <- summary(stat)

    regressionout$R2.adj[coln] <- stat$adj.r.squared  
    regressionout$coefficient[coln] <- stat$coefficients[2,1]
    regressionout$p.value[coln] <- stat$coefficients[2,4]
}

regressionout
```

And regression statistics per batch.

```{r}
regPerBatch <- matrix(ncol=8, nrow=nrow(data))
rownames(regPerBatch) <- rownames(data)
colnames(regPerBatch) <- unique(batch)

QCbatch <- MTBLS79$Batch[QChits]

for (coln in 1:nrow(data)) {
    for (bch in 1:8) {
        sampleorder <- c(1:length(which(QCbatch == bch)))
        tempdat <- data.frame(x=sampleorder, y=QCdata[coln, 
            which(QCbatch==bch)])
        stat <- lm(x ~ y, data=tempdat)
        stat <- summary(stat)
        regPerBatch[coln,bch] <- stat$adj.r.squared
    }
}

round(regPerBatch,2)
```

```{r, fig.wide=TRUE}
plotdata <- as.data.frame(regPerBatch)
plotdata$feature <- rownames(plotdata)

plotdata <- melt(plotdata, variable.name="batch")
plotdata$feature <- factor(plotdata$feature, ordered=TRUE, 
    levels=unique(plotdata$feature))

ggplot(data=plotdata, aes(x=batch, y=feature, fill=value)) + 
    geom_tile() + scale_fill_gradient2()
```

## Example of signal drift and batch effect correction for a single feature{#algorithm}

Let's have a closer look to '451.01086' measured feature and how signal
correction can be applied.

```{r, warning=FALSE, message=FALSE}
data <- data.frame(data=
    as.vector(SummarizedExperiment::assay(MTBLS79["451.01086", ])), batch=batch,
    class=factor(class, ordered=TRUE))
data$order <- c(1:nrow(data))
data$batch <- as.factor(data$batch)

ggplot(data=data, aes(x=order, y=log(data,10), col=batch, shape=class)) +
    geom_point(size=2) + theme(panel.background=element_blank()) +
    scale_color_manual(values=manual_color)
```

In this plot circles and squares represent biological samples and triangles are
the QC samples. Analytical batches are represented by colours. Differences in
measured intensities can be observed between analytical batches.

Similar plot for QC samples only

```{r}

QCdata <- data[data$class == "QC",]

ggplot(data=QCdata, aes(x=order, y=log(data,10), col=batch, shape=class)) +
    geom_point(size=2) + theme(panel.background=element_blank()) +
    scale_color_manual(values=manual_color, drop=FALSE) +
    scale_shape_manual(values=c(16, 17, 15), drop=FALSE)
```

This figure indicates that there is signal drift present within each analytical
batch and between analytical batches for this feature. Let's have a look at the
RSD% for all QC samples and QC samples within each analytical batch.

```{r}
FUN <- function(x) sd(x, na.rm=TRUE)/mean(x, na.rm=TRUE) * 100

# RSD% of biological and QC samples within all 6 batches:
out <- c(NA,NA)
names(out) <- c("Biological","QC")
out[1] <-FUN(data$data[-QChits])
out[2] <-FUN(data$data[QChits])
out
```

```{r}
# RSD% per batch:
out <- matrix(ncol=8,nrow=2)
colnames(out) <- unique(batch)
rownames(out) <- c("Biological","QC")
for (i in 1:8) {
    out[1, i] <- FUN(data$data[which(!class=="QC" & batch==i)])
    out[2, i] <- FUN(data$data[which(class=="QC" & batch==i)])
}

out
```

From the outputs above it's clear that variance of measured QC sample
intensities between batches is high (RSD% = 50), and is relatively high within
batches 1 to 5. For batch 3 QC variation exceeds that of the biological samples.

# Signal drift and batch effect correction using smoothed spline fitting

We will apply QC-RSC signal correction method as it is described in @kirwan2013.

The first step involves extracting QC sample data

```{r}
qcData <- data$data[class == "QC"]
qc_batch <- batch[class == "QC"]
qc_order <- order[class == "QC"]

qcData

```

Note that the QC data has 1 missing value. Smoothed spline regression doesn't
support missing values, so the workaround is to apply missing value imputation
or remove the NA values from input to the smoothed spline fit function
(which we will do here). We recommend at least 4 QC values be present per batch
for the fit to be reliable.

The next step involves applying the smoothed spline fit function to the QC
sample data within each batch. We will look at the data for batch 6 in detail.

```{r}
nbatch <- unique(qc_batch)

nb <- 6

# Sample injection order
x <- qc_order[qc_batch==nbatch[nb]]

# Measured peak intensity or area
y <- qcData[qc_batch==nbatch[nb]]
y
```

In this example, signal for 1 QC sample wasn't measured, so these samples need
to be removed. The smoothed spline regression input will look like this:

```{r}
NAhits <- which(is.na(y))
if (length(NAhits)>0) {
    x <- x[-c(NAhits)]
    y <- y[-c(NAhits)]
    rbind(x,y)
}
```

We will apply a log transformation to the data before fitting
```{r}
y <- log((y + sqrt(y^2)) / 2)
y
```

Fit a smoothed cubic spline using internal cross-validation for parameter
estimation
```{r}
sp.obj <- smooth.spline(x, y, cv=TRUE) 
sp.obj

out <- rbind(y,sp.obj$y)
row.names(out) <- c("measured","fitted")
out
```

Now the smoothed spline fit is used to predict values for  the biological
sample for the current batch.

```{r}
valuePredict=predict(sp.obj, order[batch==nb])

plotchr <- as.numeric(data$class)

# reverse the log transformation to convert the predictions back to the
# original scale
valuePredict$y <- exp(valuePredict$y)

plotdata <- data.frame(measured=data$data[batch==nb], fitted=valuePredict$y,
    Class=class[batch==nb], order=order[batch==nb])
plotdata2 <- melt(plotdata, id.vars=c("Class","order"), value.name="intensity",
    variable.name="data")

ggplot(data=plotdata2, aes(x=order, y=log(intensity,10), color=data,
    shape=Class)) + geom_point(size=2) +
    theme(panel.background=element_blank()) +
    scale_color_manual(values=manual_color) +
    scale_shape_manual(values=c(16, 17, 15), drop=FALSE)
```

The figure above shows the original data points in blue and the fitted data in
red. Triangles represent QC samples.

The next step in signal correction is to "flatten" the fitted curve to correct
for signal drift. This can usually be done by subtracting the fitted values
from the actual measured values for each feature. To avoid getting negative
values we will add the median value of the feature to the corrected data.

```{r, warning=FALSE}
fitmedian <- median(plotdata$measured, na.rm=TRUE)
plotdata$corrected_subt <- (plotdata$measured - plotdata$fitted) + fitmedian

plotdata2 <- melt(plotdata, id.vars=c("Class","order"), 
    value.name="intensity", variable.name="data")

plotdata_class <- as.character(plotdata2$Class)
plotdata_class[plotdata_class == "S"] <- "Sample"
plotdata_class[plotdata_class == "C"] <- "Sample"
plotdata2$Class <- factor(plotdata_class)

ggplot(data=plotdata2, aes(x=order, y=intensity, color=data, shape=Class)) +
    geom_point(size=2) + theme(panel.background=element_blank()) + 
    scale_color_manual(values=manual_color) +
    facet_grid(Class ~ .) +
    scale_shape_manual(values=c(17, 16), drop=FALSE)
```

An alternative to subtraction of the fitted values is to divide them by the
median of the fit and use the resulting coefficients to correct the data points.
The same general relative trends should be observed in either case.

```{r, warning=FALSE}
plotdata$corrected_div <- plotdata$measured/(plotdata$fitted/fitmedian)

plotdata3 <- plotdata[,c("Class", "order", "corrected_subt", "corrected_div")]

plotdata3 <- melt(plotdata3, id.vars=c("Class","order"), 
    value.name="intensity", variable.name="data")

plotdata_class <- as.character(plotdata3$Class)
plotdata_class[plotdata_class=="S"] <- "Sample"
plotdata_class[plotdata_class=="C"] <- "Sample"
plotdata3$Class <- factor(plotdata_class)

ggplot(data=plotdata3, aes(x=order, y=intensity, color=data, shape=Class)) +
    geom_point(size=2) + theme(panel.background=element_blank()) +
    scale_color_manual(values=manual_color) +
    geom_smooth(se=FALSE) + facet_grid(Class ~ .)

```

So far we have applied signal correction for data points within one analytical 
batch. The code below will perform the same steps for each of the 8 batches.

```{r, warning=FALSE}

outl <- rep(NA, nrow(data))

for (nb in 1:length(nbatch)){
    # assigning sample injection order for a batch to 'x', and corresponding 
    # intensities to 'y'
    x <- qc_order[qc_batch == nbatch[nb]]
    y <- qcData[qc_batch == nbatch[nb]]

    # remove measurements with missing values
    NAhits <- which(is.na(y))
    if (length(NAhits) > 0) {
        x <- x[-c(NAhits)]
        y <- y[-c(NAhits)]
    }

    # require at least 4 data points for QC fit
    if (length(y) >= 4) {
        range <- c(batch == nbatch[nb])
        # Order is a vector of sample indices for the current batch
        outl[range] <- pmp:::splineSmoother(x=x, y=y, newX=order[range], 
        log=TRUE, a=1, spar=0)

        # If less than 5 data points are present, return empty values   
    } else {
        range <- c(batch == nbatch[nb])
        outl[range] <- rep(NA, nrow(data))[range]
    }
}

plotdata <- data.frame(measured=data$data, fitted=outl, Class=class, 
    batch=batch, order=c(1:nrow(data)))
plotdata2 <- melt(plotdata, id.vars=c("batch","Class","order"),
    value.name="intensity", variable.name="data")

ggplot(data=plotdata2, aes(x=order, y=log(intensity,10), 
    color=data, shape=Class)) + geom_point(alpha=0.5, size=2) +
    theme(panel.background=element_blank()) + 
    scale_color_manual(values=manual_color)

```

After smoothed spline fit per each batch is calculated, we can apply signal 
correction within each batch.

```{r, warning=FALSE}

# median intensity value is used to adjust batch effect

mpa <- rep(NA, nrow(data))

for (bch in 1:8) {
    mpa[batch==bch] <- median(data$data[batch==bch], na.rm=TRUE)
}


QC_fit <- outl/mpa

# and correct actual data
res <- data$data/QC_fit

# correct data using subtratcion
res2 <- (data$data -outl) +mpa

plotdata <- data.frame(measured=data$data, corrected_subt=res2, 
    corrected_div=res, Class=class, batch=batch, order=c(1:nrow(data)))
plotdata2 <- melt(plotdata, id.vars=c("batch","Class","order"), 
    value.name="intensity", variable.name="data")

ggplot(data=plotdata2, aes(x=order, y=log(intensity,10), 
    color=data, shape=Class)) + geom_point(alpha=0.2, size=2) +
    theme(panel.background=element_blank()) +
    scale_color_manual(values=manual_color) +
    geom_smooth(se=FALSE) +
    facet_grid(Class ~ .)
```

The figure above shows the measured data points in blue and the corrected values
using subtraction (red) or division(green). Fitted smoothed spline curves of
the corrected data over all batches still indicates that there is batch related
signal drift in the data. This can be corrected using the "grand median".

First, a grand median is calculated across all batches, and then difference
between each batch median and the grand median is subtracted from all the
samples in that batch, to remove the difference.

```{r, warning=FALSE}
mpa <- rep(NA, nrow(data))

for (bch in 1:8) {
    mpa[batch == bch] <- median(res2[batch == bch], na.rm=TRUE)
}

grandMedian <- median(res2, na.rm=TRUE)

mpa <- mpa - grandMedian

plotdata$corrected_subt <- plotdata$corrected_subt - mpa

mpa <- rep(NA, nrow(data))

for (bch in 1:8) {
    mpa[batch == bch] <- median(res[batch == bch], na.rm=TRUE)
}

grandMedian <- median(res, na.rm=TRUE)

mpa <- mpa - grandMedian

plotdata$corrected_div <- plotdata$corrected_div - mpa

plotdata2 <- melt(plotdata, id.vars=c("batch","Class","order"),
    value.name="intensity", variable.name="data")

ggplot(data=plotdata2, aes(x=order, y=log(intensity,10),
    color=data, shape=Class)) + geom_point(alpha=0.2, size=2) +
    theme(panel.background=element_blank()) +
    scale_color_manual(values=manual_color) +
    geom_smooth(se=FALSE) +
    facet_grid(Class ~ .)
```

We can calculate RSD% before and after correction.

```{r, warning=FALSE}
FUN <- function(x) sd(x, na.rm=TRUE)/mean(x, na.rm=TRUE) * 100

# RSD% of biological and QC samples within all 6 batches:
out <- matrix(nrow=2, ncol=2)
colnames(out) <- c("Biological","QC")
rownames(out) <- c("measured", "corrected")
out[1,1] <-FUN(data$data[-QChits])
out[1,2] <-FUN(data$data[QChits])
out[2,1] <-FUN(res[-QChits])
out[2,2] <-FUN(res[QChits])

round(out, 2)

# RSD% per batch:
out <- matrix(ncol=8,nrow=4)
colnames(out) <- unique(batch)
rownames(out) <- c("Biological","QC","Corrected biological","Corrected QC")
for(i in 1:8) {
    out[1, i] <- FUN(data$data[which(!class=="QC" & batch==i)])
    out[2, i] <- FUN(data$data[which(class=="QC" & batch==i)])
    out[3, i] <- FUN(res[which(!class=="QC" & batch==i)])
    out[4, i] <- FUN(res[which(class=="QC" & batch==i)])
}

round(out, 2)
```

The data has now been corrected for batch and signal drift effects.

All the steps from example above can be applied to all or susbet of features in
the data set using function "QCRSC".

```{r message=FALSE, warning=FALSE, include=TRUE}
data <- MTBLS79[feature_names,]

class <- MTBLS79$Class
batch <- MTBLS79$Batch
sample_order <- c(1:ncol(data))

corrected_data <- QCRSC(df=data, order=sample_order, batch=batch,
    classes=class, spar=0, minQC=4)
```

We can calculate RSD% statistics per batch before and after correction and
visualise the results with a box plot.

```{r, warning=FALSE}
data <- SummarizedExperiment::assay(data)
corrected_data <- SummarizedExperiment::assay(corrected_data)
RSDQC <- matrix(ncol=8, nrow=nrow(data))
RSDsample <- matrix(ncol=8, nrow=nrow(data))
colnames(RSDQC) <- unique(batch)
colnames(RSDsample) <- unique(batch)

RSDQC_corrected <- matrix(ncol=8, nrow=nrow(data))
RSDsample_corrected <- matrix(ncol=8, nrow=nrow(data))
colnames(RSDQC_corrected) <- unique(batch)
colnames(RSDsample_corrected) <- unique(batch)

rownames(RSDQC) <- rownames(data)
rownames(RSDsample) <- rownames(data)
rownames(RSDQC_corrected) <- rownames(data)
rownames(RSDsample_corrected) <- rownames(data)

# for each feature
for (i in 1:nrow(data)) {
    # for each batch
    for (nb in 1:8) {
        # RSD% of QCs in this batch
        RSDQC[i, nb] <- FUN(data[i, which(class == "QC" & batch == nb)])
        # RSD% of samples in this batch
        RSDsample[i, nb] <- FUN(data[i, which(!class == "QC" & batch == nb)])
        # RSD% of QCs in this batch after correction
        RSDQC_corrected[i, nb] <- FUN(corrected_data[i, which(class == "QC"
            & batch == nb)])
        # RSD% of samples in this batch after correction
        RSDsample_corrected[i, nb] <- FUN(corrected_data[i, which(!class == "QC"
            & batch == nb)]) 
    }
}

# prepare results for plotting
plotdataQC <- melt(as.data.frame(RSDQC), variable.name="batch",
    value.name="RSD")
plotdataQC$Class <- "QC"

plotdataBio <- melt(as.data.frame(RSDsample), variable.name="batch",
    value.name="RSD")
plotdataBio$Class <- "Sample"

plotdataQC_corrected <- melt(as.data.frame(RSDQC_corrected),
    variable.name="batch", value.name="RSD")
plotdataQC_corrected$Class <- "QC_corr"

plotdataBio_corrected <- melt(as.data.frame(RSDsample_corrected),
    variable.name="batch", value.name="RSD")
plotdataBio_corrected$Class <- "Sample_corr"

plotdata <- rbind(plotdataQC, plotdataQC_corrected)

plotdata$Class <- as.factor(plotdata$Class)

# plot
ggplot(data=plotdata, aes(x=Class, y=RSD, fill=Class)) + geom_boxplot() +
    facet_wrap(~ batch, ncol=3) +
    ylab("RSD%") +
    xlab("") +
    scale_x_discrete(labels=NULL) +
    theme(panel.background=element_blank(), axis.text.x=element_blank(),
        axis.ticks.x=element_blank()) +
    scale_y_continuous(limits=c(0, 50))

plotdata <- rbind(plotdataBio, plotdataBio_corrected)

plotdata$Class <- as.factor(plotdata$Class)

# plot
ggplot(data=plotdata, aes(x=Class, y=RSD, fill=Class)) + geom_boxplot() +
    facet_wrap(~ batch, ncol=3) +
    ylab("RSD%") +
    xlab("") +
    theme(panel.background=element_blank(), axis.text.x=element_blank(),
        axis.ticks.x=element_blank())

```

Or compare the scores plots of principal components analysis (PCA) before 
and after correction.

```{r, fig.width=6, fig.height=8}
# PQN used to normalise data
# KNN for missing value imputation
# glog scaling method 
# A more detailed overview is detailed in
# Di Guida et al, Metabolomics, 12:93, 2016
# https://dx.doi.org/10.1007/s11306-016-1030-9

pca_data <- pqn_normalisation(data, classes=class, qc_label="QC")
pca_data <- mv_imputation(pca_data, method="KNN", k=5, rowmax=0.5,
    colmax=0.5, maxp=NULL, check_df=FALSE)
pca_data <- glog_transformation(pca_data, classes=class, qc_label="QC")

pca_corrected_data <- pqn_normalisation(corrected_data, classes=class,
    qc_label="QC")
pca_corrected_data <- mv_imputation(pca_corrected_data, method="KNN", k=5,
    rowmax=0.5, colmax=0.5, maxp=NULL, check_df=FALSE)
pca_corrected_data <- glog_transformation(pca_corrected_data, 
    classes=class, qc_label="QC")

pca_data <- prcomp(t(pca_data), center=TRUE, scale=FALSE)
pca_corrected_data <- prcomp(t(pca_corrected_data), center=TRUE, scale=FALSE)

# Calculate percentage of explained variance of the first two PC's
exp_var_pca <- round(((pca_data$sdev^2)/sum(pca_data$sdev^2)*100)[1:2],2)
exp_var_pca_corrected <- round(((pca_corrected_data$sdev^2) /
    sum(pca_corrected_data$sdev^2)*100)[1:2],2)

plots <- list()

plotdata <- data.frame(PC1=pca_data$x[, 1], PC2=pca_data$x[, 2], 
    batch=as.factor(batch), class=class)

plots[[1]] <- ggplot(data=plotdata, aes(x=PC1, y=PC2, col=batch)) +
    geom_point(size=2) + theme(panel.background=element_blank()) +
    scale_color_manual(values=manual_color) +
    ggtitle("PCA scores, before correction") +
    xlab(paste0("PC1 (", exp_var_pca[1] ," %)")) +
    ylab(paste0("PC2 (", exp_var_pca[2] ," %)"))

plots[[2]] <- ggplot(data=plotdata, aes(x=PC1, y=PC2, col=class)) +
    geom_point(size=2) + theme(panel.background=element_blank()) +
    scale_color_manual(values=manual_color) +
    ggtitle("PCA scores, before correction") +
    xlab(paste0("PC1 (", exp_var_pca[1] ," %)")) +
    ylab(paste0("PC2 (", exp_var_pca[2] ," %)"))

plotdata_corr <- data.frame(PC1=pca_corrected_data$x[, 1], 
    PC2=pca_corrected_data$x[, 2], batch=as.factor(batch), class=class)

plots[[3]] <- ggplot(data=plotdata_corr, aes(x=PC1, y=PC2, col=batch)) +
    geom_point(size=2) +
    theme(panel.background=element_blank()) +
    scale_color_manual(values=manual_color) +
    ggtitle("PCA scores, after correction") +
    xlab(paste0("PC1 (", exp_var_pca_corrected[1] ," %)")) +
    ylab(paste0("PC2 (", exp_var_pca_corrected[2] ," %)"))

plots[[4]] <- ggplot(data=plotdata_corr, aes(x=PC1, y=PC2, col=class)) +
    geom_point(size=2) +
    theme(panel.background=element_blank()) +
    scale_color_manual(values=manual_color) +
    ggtitle("PCA scores, after correction") +
    xlab(paste0("PC1 (", exp_var_pca_corrected[1] ," %)")) +
    ylab(paste0("PC2 (", exp_var_pca_corrected[2] ," %)"))

grid.arrange(ncol=2, plots[[1]], plots[[2]], plots[[3]], plots[[4]])
```

# Session information

```{r}
sessionInfo()
```

# References

---
title: "Peak Matrix Processing for metabolomics datasets"
author: 
- name: "Andris Jankevics"
  affiliation: Phenome Centre Birmingham, University of Birmingham, UK
  email: a.jankevics@bham.ac.uk
- name: Ralf Johannes Maria Weber
  affiliation: Phenome Centre Birmingham, University of Birmingham, UK
  email: r.j.weber@bham.ac.uk

package: pmp

date: "`r Sys.Date()`"
output: 
    BiocStyle::html_document:
        toc: true
        toc_depth: 2  
        number_sections: true  
        toc_float: true

bibliography: pmp.bib
csl: harvard-university-of-birmingham.csl

vignette: >
    %\VignetteIndexEntry{Peak Matrix Processing for metabolomics datasets}
    %\VignetteEngine{knitr::rmarkdown}
    %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
    collapse = TRUE,
    comment = "#>"
)
```

# Introduction

Metabolomics data (pre-)processing workflows consist of multiple steps
including peak picking, quality assessment, missing value imputation,
normalisation and scaling. Several software solutions (commercial and
open-source) are available for raw data processing, including r-package XCMS, 
to generate processed outputs in the form of a two dimensional data matrix.

These outputs contain hundreds or thousands of so called "uninformative" or
"irreproducible" features. Such features could strongly hinder outputs of
subsequent statistical analysis, biomarker discovery or metabolic pathway
inference. Common practice is to apply peak matrix validation and filtering
procedures as described in @guida2016, @broadhurst2018 and @schiffman2019. 

Functions within the `pmp` (Peak Matrix Processing) package are designed to
help users to prepare data for further statistical data analysis in a fast,
easy to use and reproducible manner.

This vignette showcases a range of commonly applied Peak Matrix Processing
steps for metabolomics datasets.

# Installation

You should have R version 4.0.0 or above and Rstudio installed to be able to
run this notebook.

Execute following commands from the R terminal.

```{r eval=FALSE, include=TRUE}
if (!requireNamespace("BiocManager", quietly = TRUE))
    install.packages("BiocManager")
BiocManager::install("pmp")
```

```{r message=FALSE, warning=FALSE}
library(pmp)
library(SummarizedExperiment)
library(S4Vectors)
```

# Data formats

Recently a review by @stanstrup2019 reported and discussed a broad range of
heterogeneous R tools and packages that are available via `Bioconductor`,
`CRAN`, `Github` and similar public repositories. 

`pmp` package utilises `r Biocpkg("SummarizedExperiment")` class from 
Bioconductor for data input and output.

For example, outputs from widely used `r Biocpkg("xcms")` package can be 
converted to a `SummarizedExperiment` object using functions
`featureDefinitions`, `featureValues` and `pData` on the `xcms` output object.

Additionally `pmp` also supports any matrix-like `R` data 
structure (e.g. an ordinary matrix, a data frame) as an input. If the input is
a matrix-like structure `pmp` will perform several checks for data integrity.
Please see section \@ref(endomorphisms) for more details.

# Example dataset, MTBLS79

In this tutorial we will be using a direct infusion mass spectrometry (DIMS) 
dataset consisting of 172 samples measured across 8 batches and is included in
`pmp` package as `SummarizedExperiemnt` class object `MTBLS79`. 
More detailed description of the dataset is available from  @kirwan2014, 
[MTBLS79](https://www.ebi.ac.uk/metabolights/MTBLS79) and R man page.

```{r, eval=FALSE}
help ("MTBLS79")
```

```{r}
data(MTBLS79)
MTBLS79
```

This dataset before peak matrix filtering contains 172 samples, 2488 features
and 18222 missing values across all samples what is roughly around 4.2%.

```{r}
sum(is.na(assay(MTBLS79)))
sum(is.na(assay(MTBLS79))) / length(assay(MTBLS79)) * 100
```

# Filtering a dataset

Missing values in the dataset can be filtered across samples or features. The
command below will remove all samples with more than 10 % missing values.

```{r}
MTBLS79_filtered <- filter_samples_by_mv(df=MTBLS79, max_perc_mv=0.1)

MTBLS79_filtered

sum(is.na(assay(MTBLS79_filtered)))
```

Missing values sample filter has removed two samples from the dataset. 
Outputs from any `pmp` function can be used as inputs for another `pmp`
function. For example we can apply missing value filter across features on the
output of the previous call. The function call below will filter features based
on the quality control (QC) sample group only.

```{r}
MTBLS79_filtered <- filter_peaks_by_fraction(df=MTBLS79_filtered, min_frac=0.9, 
    classes=MTBLS79_filtered$Class, method="QC", qc_label="QC")

MTBLS79_filtered

sum(is.na(assay(MTBLS79_filtered)))
```

We can add another filter on top of the previous result. For this additional
filter  we will use the same function call, but this time missing values will
be calculated across all samples and not only within the “QC” group.

```{r}
MTBLS79_filtered <- filter_peaks_by_fraction(df=MTBLS79_filtered, min_frac=0.9, 
    classes=MTBLS79_filtered$Class, method="across")

MTBLS79_filtered

sum(is.na(assay(MTBLS79_filtered)))
```

Applying these 3 filters has reduced the number of missing values from 18222 to 
4779. 

Another common filter approach is to filter features by the coefficient of
variation (CV) or RSD% of QC samples. The example shown below will use a 30%
threshold.

```{r}
MTBLS79_filtered <- filter_peaks_by_rsd(df=MTBLS79_filtered, max_rsd=30, 
    classes=MTBLS79_filtered$Class, qc_label="QC")

MTBLS79_filtered

sum(is.na(assay(MTBLS79_filtered)))
```

# Processing history

Every function in `pmp` provides a history of parameter values that have been
applied. If a user has saved outputs from an R session, it’s also easy to check
what function calls were executed.

```{r}
processing_history(MTBLS79_filtered)
```

# Data normalisation

Next, we will apply probabilistic quotient normalisation (PQN).

```{r}
MTBLS79_pqn_normalised <- pqn_normalisation(df=MTBLS79_filtered, 
    classes=MTBLS79_filtered$Class, qc_label="QC")
```

# Missing value imputation

A unified function call for several commonly used missing value imputation
algorithms is also included in pmp. Supported methods are: k-nearest neighbours
(knn), random forests (rf), Bayesian PCA missing value estimator (bpca), mean
or median value of the given feature and a constant small value. In the example
below we will apply knn imputation. 

Within `mv_imputaion` interface user can easily apply different 
mehtod without worrying about input data type or tranposing dataset.

```{r}
MTBLS79_mv_imputed <- mv_imputation(df=MTBLS79_pqn_normalised,
    method="knn")
```

# Data scaling
The generalised logarithm (glog) transformation algorithm is available to
stabilise the variance across low and high intensity mass spectral features.

```{r}
MTBLS79_glog <- glog_transformation(df=MTBLS79_mv_imputed,
    classes=MTBLS79_filtered$Class, qc_label="QC")
```

`glog_transformation` function uses QC samples to optimse scaling factor 
`lambda`. Using the function `glog_plot_plot_optimised_lambda` it's possible to
visualise if the optimsation of the given parameter has converged at the
minima.

```{r plot_glog, fig.width=5}
opt_lambda <- 
    processing_history(MTBLS79_glog)$glog_transformation$lambda_opt
glog_plot_optimised_lambda(df=MTBLS79_mv_imputed,
    optimised_lambda=opt_lambda,
    classes=MTBLS79_filtered$Class, qc_label="QC")
```

# Data integrity check and endomorphisms {#endomorphisms}

Functions in the `pmp` package are designed to validate input data if the user
chooses not to use the `r Biocpkg("SummarizedExperiment")` class object.

For example, if the input `matrix` consists of features stored in columns and
samples in rows or *vice versa*, any function within the `pmp` package will be
able to handle this in the correct manner.

```{r}
peak_matrix <- t(assay(MTBLS79))
sample_classes <- MTBLS79$Class

class(peak_matrix)
dim(peak_matrix)
class(sample_classes)
```

Let's try to use these objects as input for `mv_imputation` and 
`filter_peaks_by_rsd`.

```{r message=TRUE, warning=TRUE}
mv_imputed <- mv_imputation(df=peak_matrix, method="mn")
rsd_filtered <- filter_peaks_by_rsd(df=mv_imputed, max_rsd=30, 
    classes=sample_classes, qc_label="QC")

class (mv_imputed)
dim (mv_imputed)

class (rsd_filtered)
dim (rsd_filtered)
```

Note that `pmp` has automatically transposed the input object to use the
largest dimension as features, while the original R data type `matrix` has been
retained also for the function output.

# Session information

```{r}
sessionInfo()
```

# References

---
title: "Signal drift and batch effect correction for mass spectrometry"
author: 
- name: "Andris Jankevics"
  affiliation: Phenome Centre Birmingham, University of Birmingham, UK
  email: a.jankevics@bham.ac.uk
- name: Ralf Johannes Maria Weber
  affiliation: Phenome Centre Birmingham, University of Birmingham, UK
  email: r.j.weber@bham.ac.uk
package: pmp
date: "`r Sys.Date()`"
output: 
    BiocStyle::html_document:
        toc: true
        toc_depth: 2  
        number_sections: true  
        toc_float: true

bibliography: pmp.bib
csl: harvard-university-of-birmingham.csl

vignette: >
    %\VignetteIndexEntry{Signal drift and batch effect correction for mass spectrometry}
    %\VignetteEngine{knitr::rmarkdown}
    %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
    collapse = TRUE,
    comment = "#>"
)
```

# Introduction

This vignette demonstrates how to apply Quality Control-Robust Spline 
Correction (QC-RSC) [@kirwan2013] algorithm for signal drift and batch effect
correction within/across a multi-batch direct infusion mass spectrometry (DIMS)
and liquid chromatography mass spectrometry (LCMS) datasets.

Please read "Signal drift and batch effect correction and mass spectral quality
assessment" vignette to learn how to assess your dataset and details on
algorithm itself.

# Installation

You should have R version 4.0.0 or above and Rstudio installed to be able to
run this notebook.

Execute following commands from the R terminal.

```{r echo=TRUE, message=FALSE, warning=FALSE, eval=FALSE}
install.packages("gridExtra")

if (!requireNamespace("BiocManager", quietly = TRUE))
    install.packages("BiocManager")
BiocManager::install("pmp")
```

Load the required libraries into the R environment
```{r, message=FALSE, warning=FALSE}
library(S4Vectors)
library(SummarizedExperiment)
library(pmp)
library(ggplot2)
library(reshape2)
library(gridExtra)
```

# Dataset

In this tutorial we will be using  a direct infusion mass spectrometry (DIMS) 
dataset consisting of 172 samples measured across 8 batches and is included in
`pmp` package as `SummarizedExperiemnt` class object `MTBLS79`. 
More detailed description of the dataset is available from  @kirwan2014, 
[MTBLS79](https://www.ebi.ac.uk/metabolights/MTBLS79) and R man page.

```{r, eval=FALSE}
help ("MTBLS79")
```

```{r}
data("MTBLS79")

class <- MTBLS79$Class
batch <- MTBLS79$Batch
sample_order <- c(1:ncol(MTBLS79))

# Input data structure
MTBLS79

class[1:10]
batch[1:10]
sample_order[1:10]
```

# Missing values

Current implementation of `QCRSC` algorithm does support missing values in the 
input data object, but we would recommend to filter out features which were net 
reproducibly measured across quality control (QC) sample. In this example we 
will use 80% detection threshold.

```{r}
data <- filter_peaks_by_fraction(df=MTBLS79, classes=class, method="QC",
    qc_label="QC", min_frac=0.8)
```

# Applying signal drift and batch effect correction 

Function `QCRSC` should be used to apply signal batch correction. 

Argument `df` should be `SummarizedExperiment` object or matrix-like R data 
structure with all `numeric()` values.

Argument `order` should be `numeric()` vector containing sample injection 
order during analytical measurement and should be the same length as number of
sample in the input object. 

Argument `batch` should be `numeric()` or `character()` vector containing values
of sample batch identifier. If all samples were measured in 1 batch, then all 
values in the `batch` vector should be identical. 

Values for `classes` should be character vector containing sample class labels.
Class label for quality control sample has to be `QC`.

```{r}
corrected_data <- QCRSC(df=data, order=sample_order, batch=batch, 
    classes=class, spar=0, minQC=4)
```

# Visual comparison of the results

Function 'sbc_plot' provides visual comparison of the data before and after 
correction. For example we can check output for features '1', '5', and '30' in 
peak matrix.
```{r message=FALSE, warning=FALSE, fig.height=5, fig.width=5}
plots <- sbc_plot (df=MTBLS79, corrected_df=corrected_data, classes=class, 
    batch=batch, output=NULL, indexes=c(1, 5, 30))
plots
```

The scores plots of principal components analysis (PCA) before 
and after correction can be used to asses effects of data correction. 

In this example, probabilistic quotient normalisation (`PQN`) method is used to
normalise data, k-nearest neighbours (`KNN`) for missing value imputation and
`glog` for data scaling. All functions are availiable as a part of
`r Biocpkg("pmp")` package.

See @guida2016 for a more detailed review on common pre-processing steps and
methods.

```{r, fig.width=6, fig.height=8}

manual_color = c("#386cb0", "#ef3b2c", "#7fc97f", "#fdb462", "#984ea3", 
    "#a6cee3", "#778899", "#fb9a99", "#ffff33")

pca_data <- pqn_normalisation(MTBLS79, classes=class, qc_label="QC")
pca_data <- mv_imputation(pca_data, method="KNN", k=5, rowmax=0.5,
    colmax=0.5, maxp=NULL, check_df=FALSE)
pca_data <- glog_transformation(pca_data, classes=class, qc_label="QC")

pca_corrected_data <- pmp::pqn_normalisation(corrected_data, classes=class,
    qc_label="QC")
pca_corrected_data <- pmp::mv_imputation(pca_corrected_data, method="KNN", k=5,
    rowmax=0.5, colmax=0.5, maxp=NULL, check_df=FALSE)
pca_corrected_data <- pmp::glog_transformation(pca_corrected_data, 
    classes=class, qc_label="QC")

pca_data <- prcomp(t(assay(pca_data)), center=TRUE, scale=FALSE)
pca_corrected_data <- prcomp(t(assay(pca_corrected_data)),
    center=TRUE, scale=FALSE)

# Calculate percentage of explained variance of the first two PC's
exp_var_pca <- round(((pca_data$sdev^2)/sum(pca_data$sdev^2)*100)[1:2],2)
exp_var_pca_corrected <- round(((pca_corrected_data$sdev^2) /
    sum(pca_corrected_data$sdev^2)*100)[1:2],2)

plots <- list()

plotdata <- data.frame(PC1=pca_data$x[, 1], PC2=pca_data$x[, 2], 
    batch=as.factor(batch), class=class)

plots[[1]] <- ggplot(data=plotdata, aes(x=PC1, y=PC2, col=batch)) +
    geom_point(size=2) + theme(panel.background=element_blank()) +
    scale_color_manual(values=manual_color) +
    ggtitle("PCA scores, before correction") +
    xlab(paste0("PC1 (", exp_var_pca[1] ," %)")) +
    ylab(paste0("PC2 (", exp_var_pca[2] ," %)"))

plots[[2]] <- ggplot(data=plotdata, aes(x=PC1, y=PC2, col=class)) +
    geom_point(size=2) + theme(panel.background=element_blank()) +
    scale_color_manual(values=manual_color) +
    ggtitle("PCA scores, before correction") +
    xlab(paste0("PC1 (", exp_var_pca[1] ," %)")) +
    ylab(paste0("PC2 (", exp_var_pca[2] ," %)"))

plotdata_corr <- data.frame(PC1=pca_corrected_data$x[, 1], 
    PC2=pca_corrected_data$x[, 2], batch=as.factor(batch), class=class)

plots[[3]] <- ggplot(data=plotdata_corr, aes(x=PC1, y=PC2, col=batch)) +
    geom_point(size=2) +
    theme(panel.background=element_blank()) +
    scale_color_manual(values=manual_color) +
    ggtitle("PCA scores, after correction") +
    xlab(paste0("PC1 (", exp_var_pca_corrected[1] ," %)")) +
    ylab(paste0("PC2 (", exp_var_pca_corrected[2] ," %)"))

plots[[4]] <- ggplot(data=plotdata_corr, aes(x=PC1, y=PC2, col=class)) +
    geom_point(size=2) +
    theme(panel.background=element_blank()) +
    scale_color_manual(values=manual_color) +
    ggtitle("PCA scores, after correction") +
    xlab(paste0("PC1 (", exp_var_pca_corrected[1] ," %)")) +
    ylab(paste0("PC2 (", exp_var_pca_corrected[2] ," %)"))

grid.arrange(ncol=2, plots[[1]], plots[[2]], plots[[3]], plots[[4]])
```

# Session information

```{r}
sessionInfo()
```

# References

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{MTBLS79}
\alias{MTBLS79}
\title{Direct-infusion mass spectrometry (DIMS) data set}
\format{
A \link[SummarizedExperiment]{RangedSummarizedExperiment-class} 
object. \cr
\code{assay(MTBLS79)} Peak intensities of the DIMS data set. 
Contains 172 samples and 2488 features. \cr
\code{colData(MTBLS79)} Sample meta data containing 4 columns. \cr
    \code{Batch} - \code{character()}, sample batch name. \cr
    \code{Sample_Rep} - \code{character()}, sample replicate code. \cr
    \code{Class} - \code{character()}, sample class labels. \cr
    \code{Class2} - \code{character()}, alternative sample class labels
    grouping together replicate samples. \cr
}
\source{
https://www.ebi.ac.uk/metabolights/MTBLS79
}
\usage{
MTBLS79
}
\description{
Data set of 20 biological (cow vs sheep) serum samples that were analysed
repeatedly, in 8 batches across 7 days.
}
\details{
Code below includes all commands used to generate MTBLS79 object. \cr

\preformatted{
library (openxlsx)
library (SummarizedExperiment)

download.file(destfile = "MTBLS79.xlsx", mode="wb",
url = "ftp://ftp.ebi.ac.uk/pub/databases/metabolights/studies/public/ 
   MTBLS79/Dataset07__SFPM.xlsx") 
wb <- openxlsx::loadWorkbook(xlsxFile="MTBLS79.xlsx")

MTBLS79 <- list()

MTBLS79$assay <- openxlsx::readWorkbook(wb, "data", colNames=T, rowNames=T)

# Last row of the peak matrix represent mean intensities across all samples.
MTBLS79$assay <- MTBLS79$assay[-c(nrow(MTBLS79$assay)), ]

# Transpose peak matrix, so that features are in rows and samples in columns.
MTBLS79$assay <- as.matrix(t(MTBLS79$assay))

# Missing values in the input data are stored as 0, replace with NA
MTBLS79$assay[MTBLS79$assay == 0] <- NA

rownames(MTBLS79$assay) <- round(as.numeric(rownames(MTBLS79$assay)), 5)

MTBLS79$colData <- openxlsx::readWorkbook(wb, "meta", colNames=T, rowNames=F)
MTBLS79$colData <- MTBLS79$colData[-c(nrow(MTBLS79$colData)), 1:4]

MTBLS79 <- SummarizedExperiment(assays=list(MTBLS79$assay), 
colData=DataFrame(MTBLS79$colData))
}
}
\references{
Kirwan et al, Scientific Data volume 1,
Article number: 140012 (2014) https://www.nature.com/articles/sdata201412
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/normalisation.R
\name{pqn_normalisation}
\alias{pqn_normalisation}
\title{Probabilistic quotient normalisation (PQN)}
\usage{
pqn_normalisation(
  df,
  classes,
  qc_label,
  ref_mean = NULL,
  qc_frac = 0,
  sample_frac = 0,
  ref_method = "mean"
)
}
\arguments{
\item{df}{A matrix-like (e.g. an ordinary matrix, a data frame) or 
\link[SummarizedExperiment]{RangedSummarizedExperiment-class} object with 
all values of class \code{numeric()} or \code{integer()} of peak 
intensities, areas or other quantitative characteristic.}

\item{classes}{\code{character()}, vector of class labels. Must be the same 
length as the number of sample in the input peak table. If input is 
\code{SummarizedExperiment} object, use 
\code{SummarizedExperiment_object$meta_data_column_name}.}

\item{qc_label}{\code{character(1)} or \code{NULL}, class label used to
identify QC samples.}

\item{ref_mean}{\code{numeric()} or \code{NULL}, Vector of reference mean
values to use instead of calculating from QC sample group. If set to 
\code{NULL}, QC sample data will be used.}

\item{qc_frac}{\code{numeric()} A value between 0 and 1 to indicate the 
minimum proportion of QC samples a feature must be present in for it to be included 
when computing the reference. Default \code{qc_frac = 0}.}

\item{sample_frac}{\code{numeric()} A value between 0 and 1 to indicate the 
minimum proportion of samples a feature must be present in for it to be considered 
when computing the normalisation coefficients. Default \code{sample_frac = 0}.}

\item{ref_method}{\code{character()} Method used to compute the reference from
the QC samples. Default \code{ref_method = 'mean'}. Allowed
values are "mean" or "median".}
}
\value{
Object of class \code{SummarizedExperiment}. If input data are 
matrix-like (e.g. an ordinary matrix, a data frame) object, the same R data 
structure as the input will be returned with all values of the data type.

\code{numeric()}.
}
\description{
For every feature the mean response is calculated across all QC samples. A 
reference vector is then generated. The median between the reference vector
and every sample is computed obtaining a vector of coefficients related to
each sample. Each sample is then divided by the median value of the vector
of coefficients; this median value is different for each sample. This 
method was adapted by Dieterle et al. (2006) (see references). Its purpose 
is to take into account the concentration changes of some metabolite 
features that affect limited regions of the data.
}
\examples{
df <- MTBLS79[ , MTBLS79$Batch==1]
pqn_normalisation(df=df,
    classes=df$Class, qc_label='QC')

}
\references{
Dieterle F. et al., Anal. Chem., 78(13), 2006. 
http://dx.doi.org/10.1021/ac051632c
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/glog_transformation.R
\name{glog_transformation}
\alias{glog_transformation}
\title{Variance stabilising generalised logarithm (glog) transformation}
\usage{
glog_transformation(df, classes, qc_label, lambda = NULL)
}
\arguments{
\item{df}{A matrix-like (e.g. an ordinary matrix, a data frame) or 
\link[SummarizedExperiment]{RangedSummarizedExperiment-class} object with 
all values of class \code{numeric()} or \code{integer()} of peak 
intensities, areas or other quantitative characteristic.}

\item{classes}{\code{character()}, vector of class labels. Must be the same 
length as the number of sample in the input peak table. If input is 
\code{SummarizedExperiment} object, use 
\code{SummarizedExperiment_object$meta_data_column_name}.}

\item{qc_label}{\code{character(1)} or \code{NULL}, class label used to
identify QC samples.}

\item{lambda}{\code{NULL} or \code{numeric(1)}, if not \code{NULL} will use
provided value for glog lambda.}
}
\value{
Object of class \code{SummarizedExperiment}. If input data are a 
matrix-like (e.g. an ordinary matrix, a data frame) object, function returns 
the same R data structure as input with all value of data type 
\code{numeric()}.
}
\description{
Performs glog transformation on the data set. QC samples can be used to 
estimate technical variation in the data set and calculate transformation
parameter \eqn{\lambda} (lambda). QC samples usually comprise a pool of 
aliquots taken from all other samples in the study  and analysed repeatedly 
throughout an analytical batch.
}
\details{
Many univariate and multivariate statistical tests require homogeneity and
n ormality of dataset variance. Real-world metabolomics datasets often fail 
to meet these criteria due to asymmetric (i.e. non-'normal') and/or 
heteroscedatic (i.e. non-homogenous) variance structure. To address this 
issue, \code{glog} data transformations may be applied. \cr
For each cell within the data matrix, transform the raw value (x) according 
to:  \eqn{log10(x + sqrt(x^2 + \lambda))}. The parameter \eqn{\lambda} is 
typically calculated using quality control (QC) samples analysed throughout
an analysis batch.
}
\examples{
df <- MTBLS79[, MTBLS79$Batch == 1]
out <- mv_imputation(df=df, method="knn")
out <- glog_transformation (df=out, classes=df$Class,
    qc_label="QC")

}
\references{
Parsons HM et. al., BMC Bionf., 8(234), 2007. 
https://doi.org/10.1186/1471-2105-8-234
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/glog_transformation.R
\name{glog_plot_optimised_lambda}
\alias{glog_plot_optimised_lambda}
\title{Plot SSE error of lambda optimisation process}
\usage{
glog_plot_optimised_lambda(
  df,
  optimised_lambda,
  classes,
  qc_label,
  plot_grid = 100
)
}
\arguments{
\item{df}{A matrix-like (e.g. an ordinary matrix, a data frame) or 
\link[SummarizedExperiment]{RangedSummarizedExperiment-class} object with 
all values of class \code{numeric()} or \code{integer()} of peak 
intensities, areas or other quantitative characteristic.}

\item{optimised_lambda}{\code{numeric(1)}, value of optimised lambda from 
glog_transformation output.}

\item{classes}{\code{character()}, vector of class labels. Must be the same 
length as the number of sample in the input peak table. If input is 
\code{SummarizedExperiment} object, use 
\code{SummarizedExperiment_object$meta_data_column_name}.}

\item{qc_label}{\code{character(1)} or \code{NULL}, class label used to
identify QC samples.}

\item{plot_grid}{\code{integer(1)}, number of data points to use for SSE 
optimisation.}
}
\value{
Class \code{ggplot} object containing optimisation plot.
}
\description{
Plot SSE error of lambda optimisation process
}
\examples{
data <- MTBLS79[, MTBLS79$Batch == 1]
classes <- data$Class

data <- mv_imputation(df=data, method='knn')
out <- glog_transformation (df=data, classes=classes,
    qc_label='QC')

optimised_lambda <- S4Vectors::metadata(out)
optimised_lambda <- 
    optimised_lambda$processing_history$glog_transformation$lambda_opt

glog_plot_optimised_lambda(df=data, classes=classes,
    qc_label="QC", optimised_lambda=optimised_lambda)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/filters.R
\name{filter_peaks_by_blank}
\alias{filter_peaks_by_blank}
\title{Filter features by blank samples}
\usage{
filter_peaks_by_blank(
  df,
  fold_change,
  classes,
  blank_label,
  qc_label = NULL,
  remove_samples = TRUE,
  remove_peaks = TRUE,
  fraction_in_blank = 0
)
}
\arguments{
\item{df}{A matrix-like (e.g. an ordinary matrix, a data frame) or 
\link[SummarizedExperiment]{RangedSummarizedExperiment-class} object with 
all values of class \code{numeric()} or \code{integer()} of peak 
intensities, areas or other quantitative characteristic.}

\item{fold_change}{\code{numeric(1)}, fold_change minimum fold change 
between analytical and blank samples.}

\item{classes}{\code{character()}, vector of class labels. Must be the same 
length as the number of sample in the input peak table. If input is 
\code{SummarizedExperiment} object, use 
\code{SummarizedExperiment_object$meta_data_column_name}.}

\item{blank_label}{\code{character(1)}, class label used to identify blank
samples.}

\item{qc_label}{\code{character(1)} or \code{NULL}, class label used to
identify QC samples.}

\item{remove_samples}{\code{logical(1)}, remove blank samples from peak 
matrix or not.}

\item{remove_peaks}{\code{logical(1)}, remove filtered features from peak 
matrix or not.}

\item{fraction_in_blank}{\code{numeric(1)}, value between 0 to 1 to specify 
fraction in how many blanks peaks should be present.}
}
\value{
Object of class \code{SummarizedExperiment}. If input data are a 
matrix-like (e.g. an ordinary matrix, a data frame) object, function returns 
\code{numeric()} matrix-like object of filtered data set. Function 
\code{flags} are added to the object \code{attributes} and is a 
\link[S4Vectors]{DataFrame-class} with five columns. The same 
\code{DataFrame} object containing flags is added to \code{rowData()} 
element of \code{SummarizedExperiment} object as well.\cr
\cr
Columns in \code{rowData()} or \code{flags} element contain: \cr
\code{median_non_blanks} median intensities of features of non-blank
samples; \cr
\code{median_blanks} median intensities of features of blank samples; \cr
\code{fold_change} fold change between analytical and blank samples; \cr
\code{blank_flags} \code{integer()}, if 0 feature is flagged to be 
removed; \cr
\code{blank_fraction_flags} \code{numeric()}, fraction in how many blank
samples peaks is present. \cr
}
\description{
Metabolomics datasets often contain many features of non-biological 
origin e.g. those associated with extraction and analysis solvents. 
This tool facilitates the removal of such features from the data matrix, 
as defined using an appropriate blank sample.
}
\details{
If parameter \code{qc_label} is not \code{NULL}, QC samples which will be 
used to calculate the median signal intensity.
}
\examples{
df <- MTBLS79[ ,MTBLS79$Batch == 1]
df$Class[1:2] <- "Blank"
out <- filter_peaks_by_blank(df=df, fold_change=1.2, 
   classes=df$Class, blank_label="Blank", qc_label=NULL, 
   remove_samples=FALSE, remove_peaks=TRUE, fraction_in_blank=0)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/filters.R
\name{filter_peaks_by_rsd}
\alias{filter_peaks_by_rsd}
\title{Filter features by RSD\% of QC samples}
\usage{
filter_peaks_by_rsd(df, max_rsd, classes, qc_label, remove_peaks = TRUE)
}
\arguments{
\item{df}{A matrix-like (e.g. an ordinary matrix, a data frame) or 
\link[SummarizedExperiment]{RangedSummarizedExperiment-class} object with 
all values of class \code{numeric()} or \code{integer()} of peak 
intensities, areas or other quantitative characteristic.}

\item{max_rsd}{\code{numeric()}, threshold of QC RSD\% value}

\item{classes}{\code{character()}, vector of class labels. Must be the same 
length as the number of sample in the input peak table. If input is 
\code{SummarizedExperiment} object, use 
\code{SummarizedExperiment_object$meta_data_column_name}.}

\item{qc_label}{\code{character(1)} or \code{NULL}, class label used to
identify QC samples.}

\item{remove_peaks}{\code{logical(1)}, remove filtered features from peak 
matrix or not.}
}
\value{
Object of class \code{SummarizedExperiment}. If input data are a 
matrix-like (e.g. an ordinary matrix, a data frame) object, function returns 
\code{numeric()} matrix-like object of filtered data set. Function 
\code{flags} are added to the object \code{attributes} and is a 
\link[S4Vectors]{DataFrame-class} with five columns. The same 
\code{DataFrame-class} object containing flags is added to \code{rowData()} 
element of \code{SummarizedExperiment} object as well.\cr
\cr
Columns in \code{rowData()} or \code{flags} element contain: \cr
\code{rsd_QC} \code{numeric()}, RSD\% value of QC samples per feature; \cr
\code{rsd_flags} \code{integer()},if 0 feature is flagged to be removed. \cr
}
\description{
Metabolomics datasets often contain 'features' with irreproducible peak 
intensity values, or with large numbers of missing values. This tool 
facilitates the remove of such features from a data matrix, based upon 
relative standard deviation of intensity values for a given feature within 
specified QC samples.
}
\examples{

df <- MTBLS79[ ,MTBLS79$Batch == 1]
out <- filter_peaks_by_rsd(df=df, max_rsd=20,
    classes=df$Class, qc_label='QC')

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/filters.R
\name{filter_samples_by_mv}
\alias{filter_samples_by_mv}
\title{Filter samples by missing values}
\usage{
filter_samples_by_mv(df, max_perc_mv, classes = NULL, remove_samples = TRUE)
}
\arguments{
\item{df}{A matrix-like (e.g. an ordinary matrix, a data frame) or 
\link[SummarizedExperiment]{RangedSummarizedExperiment-class} object with 
all values of class \code{numeric()} or \code{integer()} of peak 
intensities, areas or other quantitative characteristic.}

\item{max_perc_mv}{\code{numeric(1)}, Value between 0 and 1 of threshold 
of missing value percentage in sample.}

\item{classes}{\code{character()}, vector of class labels. Must be the same 
length as the number of sample in the input peak table. If input is 
\code{SummarizedExperiment} object, use 
\code{SummarizedExperiment_object$meta_data_column_name}.}

\item{remove_samples}{\code{logical(1)}, remove blank samples from peak 
matrix or not.}
}
\value{
Object of class \code{SummarizedExperiment}. If input data are a 
matrix-like (e.g. an ordinary matrix, a data frame) object, function returns 
\code{numeric()} matrix-like object of filtered data set. Function 
\code{flags} are added to the object \code{attributes} and is a 
\link[S4Vectors]{DataFrame-class} with five columns. The same 
\code{DataFrame} object containing flags is added to \code{rowData()} 
element of \code{SummarizedExperiment} object as well. If element 
\code{colData()} already exists flags are appended to existing values.\cr
\cr

Columns in \code{colData()} or \code{flags} element contain: \cr
\code{perc_mv} \code{numeric()}, fraction of missing values per sample; \cr
\code{flags} \code{integer()},if 0 feature is flagged to be removed. \cr
}
\description{
Missing values in mass spectrometry metabolomic datasets occur widely and 
can originate from a number of sources, including for both technical and 
biological reasons. In order for robust conclusions to be drawn from 
down-stream statistical testing procedures, the issue of missing values 
must first be addressed. This tool facilitates the removal of samples 
containing a user-defined maximum percentage of missing values.
}
\examples{

df <- MTBLS79
out <- filter_samples_by_mv (df=df, max_perc_mv=0.8)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sbc_plot.R
\name{sbc_plot}
\alias{sbc_plot}
\title{Plot QCRSC corrected outputs}
\usage{
sbc_plot(
  df,
  corrected_df,
  classes,
  batch,
  indexes = NULL,
  qc_label = "QC",
  output = "sbcms_plots.pdf"
)
}
\arguments{
\item{df}{A matrix-like (e.g. an ordinary matrix, a data frame) or 
\link[SummarizedExperiment]{RangedSummarizedExperiment-class} object with 
all values of class \code{numeric()} or \code{integer()} of peak 
intensities, areas or other quantitative characteristic.}

\item{corrected_df}{Output from \link[pmp]{QCRSC} function.}

\item{classes}{\code{character()}, vector of class labels. Must be the same 
length as the number of sample in the input peak table. If input is 
\code{SummarizedExperiment} object, use 
\code{SummarizedExperiment_object$meta_data_column_name}.}

\item{batch}{\code{numeric()} or \code{character()}, a vector indicating
the batch each sample was measured in. If only one batch was measured then
all values should be set to 1}

\item{indexes}{\code{numeric()}, a vector defining which features to plot.
If set to \code{NULL} will plot the first 100.}

\item{qc_label}{\code{character(1)} or \code{NULL}, class label used to
identify QC samples.}

\item{output}{\code{character()}, a filename of the output pdf file. 
Can include the path. If set to \code{NULL} output will be list object 
containing class \code{ggplot} plots.}
}
\value{
Pdf file or \code{list()} object \code{ggplot} class showing data 
before and after signal correction.
}
\description{
Plot the output from signal batch correction for the selected or
the first 100 features.
}
\examples{

order <- c(1:ncol(MTBLS79))
data <- MTBLS79[1:10, ]

out <- QCRSC(df =data, order=order, batch=MTBLS79$Batch, 
   classes=MTBLS79$Class, spar=0, minQC=4)
plots <- sbc_plot (df=data, corrected_df=out, classes=MTBLS79$Class,
   batch=MTBLS79$Batch, output=NULL) 

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/filters.R
\name{remove_peaks}
\alias{remove_peaks}
\title{Remove features from peak intensity matrix}
\usage{
remove_peaks(df, rem_index)
}
\arguments{
\item{df}{A matrix-like (e.g. an ordinary matrix, a data frame) or 
\link[SummarizedExperiment]{RangedSummarizedExperiment-class} object with 
all values of class \code{numeric()} or \code{integer()} of peak 
intensities, areas or other quantitative characteristic.}

\item{rem_index}{\code{logical()}, vector containing \code{TRUE} vales for 
features to remove. Should be the same length as number of features in input 
data.}
}
\value{
Object of class \code{SummarizedExperiment}. If input data are a 
matrix-like (e.g. an ordinary matrix, a data frame) object, function returns 
the same R data structure as input with all value of data type 
\code{numeric()}.
}
\description{
Filter to remove features.
}
\examples{
df <- MTBLS79[ ,MTBLS79$Batch == 1]
rem_index <- vector(mode="logical", 
    length=nrow(SummarizedExperiment::assay(df)))
rem_index[c(1, 20, 456, 789)] <- TRUE
out <- remove_peaks(df=df, rem_index=rem_index)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sbc_main.R
\name{QCRSC}
\alias{QCRSC}
\title{Quality Control-Robust Spline Correction (QC-RSC)}
\usage{
QCRSC(
  df,
  order,
  batch,
  classes,
  spar = 0,
  log = TRUE,
  minQC = 5,
  qc_label = "QC",
  spar_lim = c(-1.5, 1.5)
)
}
\arguments{
\item{df}{A matrix-like (e.g. an ordinary matrix, a data frame) or 
\link[SummarizedExperiment]{RangedSummarizedExperiment-class} object with 
all values of class \code{numeric()} or \code{integer()} of peak 
intensities, areas or other quantitative characteristic.}

\item{order}{\code{numeric()}, A numeric vector indicating the order in 
which samples were measured.}

\item{batch}{\code{numeric()} or \code{character()}, a vector indicating
the batch each sample was measured in. If only one batch was measured then
all values should be set to 1}

\item{classes}{\code{character()}, vector of class labels. Must be the same 
length as the number of sample in the input peak table. If input is 
\code{SummarizedExperiment} object, use 
\code{SummarizedExperiment_object$meta_data_column_name}.}

\item{spar}{\code{numeric(1)}, Spline smoothing parameter. Should be in the 
range 0 to 1. If set to 0 it will be estimated using leave-one-out 
cross-validation.}

\item{log}{\code{logical(1)}, to perform the signal correction fit on the
log scaled data. Default is TRUE.}

\item{minQC}{\code{integer(1)}, Minimum number of measured quality control
(QC) samples required for signal correction within feature per batch. For 
features where signal was measured in less QC samples than threshold signal
correction won't be applied.}

\item{qc_label}{\code{character(1)} or \code{NULL}, class label used to
identify QC samples.}

\item{spar_lim}{A 2 element numeric vector containing the min and max
values of spar when searching for an optimum. Default \code{spar_lim = c(-1.5,1.5)}}
}
\value{
Object of class \code{SummarizedExperiment}. If input data are a 
matrix-like (e.g. an ordinary matrix, a data frame) object, function returns 
the same R data structure as input with all value of data type 
\code{numeric()}.
}
\description{
Implementation of Quality QC-RSC algorithm for signal drift and batch 
effect correction within/across a multi-batch direct infusion mass
spectrometry (DIMS) and liquid chromatography mass spectrometry (LCMS)
datasets.
This version supports missing values, but requires at least 4 data point 
for quality control (QC) samples measured within each analytical batch.
The smoothing parameter (spar) can be optimised using leave-one-out cross
validation to avoid overfitting.
}
\examples{
classes <- MTBLS79$Class
batch <- MTBLS79$Batch
order <- c(1:ncol(MTBLS79))

out <- QCRSC(df = MTBLS79[1:10, ], order = order, batch = MTBLS79$Batch,
classes = MTBLS79$Class, spar = 0, minQC = 4)

}
\references{
Kirwan et al, Anal. Bioanal. Chem., 405 (15), 2013
\url{https://dx.doi.org/10.1007/s00216-013-6856-7}
}
\author{
Andris Jankevics \email{a.jankevics@bham.ac.uk}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{processing_history}
\alias{processing_history}
\title{Return history of applied functions and argument from pmp package.}
\usage{
processing_history(df)
}
\arguments{
\item{df}{A matrix-like (e.g. an ordinary matrix, a data frame) or 
\link[SummarizedExperiment]{RangedSummarizedExperiment-class} object with 
all values of class \code{numeric()} or \code{integer()} of peak 
intensities, areas or other quantitative characteristic.}
}
\value{
List of function  names and argument values.
}
\description{
Return history of applied functions and argument from pmp package.
}
\examples{
df <- MTBLS79[ ,MTBLS79$Batch == 1]
df$Class[1:2] <- "Blank"
out <- filter_peaks_by_blank(df=df, fold_change=1.2, 
   classes=df$Class, blank_label="Blank", qc_label=NULL, 
   remove_samples=FALSE, remove_peaks=TRUE, fraction_in_blank=0)
processing_history(out)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/filters.R
\name{filter_peaks_by_fraction}
\alias{filter_peaks_by_fraction}
\title{Filter features by fraction of missing values}
\usage{
filter_peaks_by_fraction(
  df,
  min_frac,
  classes = NULL,
  method = "QC",
  qc_label = "QC",
  remove_peaks = TRUE
)
}
\arguments{
\item{df}{A matrix-like (e.g. an ordinary matrix, a data frame) or 
\link[SummarizedExperiment]{RangedSummarizedExperiment-class} object with 
all values of class \code{numeric()} or \code{integer()} of peak 
intensities, areas or other quantitative characteristic.}

\item{min_frac}{\code{numeric(1)}, value between 0 and 1, a threshold of 
fraction of detection.}

\item{classes}{\code{character()}, vector of class labels. Must be the same 
length as the number of sample in the input peak table. If input is 
\code{SummarizedExperiment} object, use 
\code{SummarizedExperiment_object$meta_data_column_name}.}

\item{method}{\code{character(1)}, method to use. \code{QC} - within 
QC samples, \code{within} - within each sample class or \code{across} - 
across all samples.}

\item{qc_label}{\code{character(1)} or \code{NULL}, class label used to
identify QC samples.}

\item{remove_peaks}{\code{logical(1)}, remove filtered features from peak 
matrix or not.}
}
\value{
Object of class \code{SummarizedExperiment}. If input data are a 
matrix-like (e.g. an ordinary matrix, a data frame) object, function returns 
\code{numeric()} matrix-like object of filtered data set. Function 
\code{flags} are added to the object \code{attributes} and is a 
\link[S4Vectors]{DataFrame-class} with five columns. The same 
\code{DataFrame} object containing flags is added to \code{rowData()} 
element of \code{SummarizedExperiment} object as well.\cr
\cr

Columns in \code{rowData()} or \code{flags} element contain \code{fractions} 
of missing values per feature within QC samples (mehtod \code{QC}), 
across (method \code{across}) or within (mehtod \code{within}) each sample 
group.
}
\description{
Metabolomics datasets often contain 'features' with irreproducible peak 
intensity values, or with large numbers of missing values. This tool 
facilitates the remove of such features from a data matrix, based upon the 
relative proportion (minimum fraction) of samples containing non-missing 
values.
}
\examples{
df <- MTBLS79[ ,MTBLS79$Batch == 1]
out <- filter_peaks_by_fraction(df=df, min_frac=1, 
    classes=df$Class, method='QC', qc_label='QC')
    
out <- filter_peaks_by_fraction(df=df, min_frac=1, 
    classes=df$Class, method='across', qc_label='QC')

out <- filter_peaks_by_fraction(df=df, min_frac=1, 
    classes=df$Class, method='within', qc_label='QC')

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/normalisation.R
\name{normalise_to_sum}
\alias{normalise_to_sum}
\title{Normalisation by total sum of the features per sample}
\usage{
normalise_to_sum(df, check_df = TRUE)
}
\arguments{
\item{df}{A matrix-like (e.g. an ordinary matrix, a data frame) or 
\link[SummarizedExperiment]{RangedSummarizedExperiment-class} object with 
all values of class \code{numeric()} or \code{integer()} of peak 
intensities, areas or other quantitative characteristic.}

\item{check_df}{\code{logical(1)}, if set to TRUE will check if input data 
needs to be transposed, so that features are in rows.}
}
\value{
Object of class \code{SummarizedExperiment}. If input data are a 
matrix-like (e.g. an ordinary matrix, a data frame) object, function returns 
the same R data structure as input with all value of data type 
\code{numeric()}.
}
\description{
For each sample, every feature intensity value is divided by the total sum of
all feature intensity values measured in that sample (\code{NA} values
ignored by default), before multiplication by 100; the unit is \%.
}
\examples{
df <- MTBLS79[ ,MTBLS79$Batch == 1]
out <- normalise_to_sum (df=df)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mv_imputation.R
\name{mv_imputation}
\alias{mv_imputation}
\title{Missing value imputation using different algorithms}
\usage{
mv_imputation(
  df,
  method,
  k = 10,
  rowmax = 0.5,
  colmax = 0.5,
  maxp = NULL,
  check_df = TRUE
)
}
\arguments{
\item{df}{A matrix-like (e.g. an ordinary matrix, a data frame) or 
\link[SummarizedExperiment]{RangedSummarizedExperiment-class} object with 
all values of class \code{numeric()} or \code{integer()} of peak 
intensities, areas or other quantitative characteristic.}

\item{method}{\code{character(1)}, missing value imputation method.
Supported methods are \code{knn}, \code{rf}, \code{bpca}, \code{sv},
\code{'mn'} and \code{'md'}.}

\item{k}{\code{numeric(1)}, for a given sample containing a missing value,
the number of nearest neighbours to include to calculate a replacement 
value. Used only for method \code{knn}.}

\item{rowmax}{\code{numeric(1)}, the maximum percentage of missing data 
allowed in any row. For any rows exceeding given limit, missing values are 
imputed using the overall mean per sample. Used only for method \code{knn}.}

\item{colmax}{\code{numeric(1)}, the maximum percent missing data allowed in
any column. If any column exceeds given limit, the function will report an
error Used only for method \code{knn}.}

\item{maxp}{\code{integer(1)}, number of features to run on single core.
If set to NULL will use total number of features.}

\item{check_df}{\code{logical(1)}, if set to TRUE will check if input data 
needs to be transposed, so that features are in rows.}
}
\value{
Object of class \code{SummarizedExperiment}. If input data are a 
matrix-like (e.g. an ordinary matrix, a data frame) object, function returns 
the same R data structure as input with all value of data type 
\code{numeric()}.
}
\description{
Missing values in metabolomics data sets occur widely and can originate from 
a number of sources, including technical and biological reasons. \cr
Missing values imputation is applied to replace non-existing values 
with an estimated values while maintaining the data structure. A number of 
different methods are available as part of this function. \cr
}
\details{
Supported missing value imputation methods are: \cr
\cr
\code{knn} - K-nearest neighbour. For each feature in each sample, missing 
values are replaced by the mean average value (non-weighted) calculated 
from its \code{k} closest neighbours in multivariate space (default distance 
metric: euclidean distance); \cr
\cr
\code{rf} - Random Forest. This method is a wrapper of 
\link[missForest]{missForest} function. For each feature, missing values are 
iteratively imputed until a maximum number of iterations (10), or until the
difference between consecutively-imputed matrices becomes positive. 
Trees per forest are set to 100, variables included per tree are calculate 
using formula \eqn{sqrt(total number of variables)}; \cr
\cr
\code{bpca} - Bayesian principal component analysis. This method is a 
wrapper of \link[pcaMethods]{pca} function. Missing values are replaced by
the values obtained from principal component analysis regression with a 
Bayesian method. Therefore every imputed missing value does not occur 
multiple times, neither across the samples nor across the metabolite 
features; \cr
\cr
\code{sv} - Small value. For each feature, replace missing values with half
of the lowest value recorded in the entire data matrix; \cr
\cr
\code{'mn'} - Mean. For each feature, replace missing values with the mean 
average (non-weighted) of all other non-missing values for that variable;\cr
\cr
\code{'md'} - Median. For each feature, replace missing values with the 
median of all other non-missing values for that variable. \cr
\cr
}
\examples{
df <- MTBLS79[ ,MTBLS79$Batch == 1]
out <- mv_imputation(df=df, method='knn')

}
